@@ -27,7 +27,6 @@
 
 typedef std::vector<std::string> FileNameList;
 
-
 class GraphicsContext {
     public:
         GraphicsContext()
@@ -103,16 +102,23 @@ public:
             osg::Texture2D* texture2D = dynamic_cast<osg::Texture2D*>(texture);
             osg::Texture3D* texture3D = dynamic_cast<osg::Texture3D*>(texture);
             
-            osg::Image* image = texture2D ? texture2D->getImage() : texture3D ? texture3D->getImage() : 0;
-            if (image && 
+            osg::ref_ptr<osg::Image> image = texture2D ? texture2D->getImage() : (texture3D ? texture3D->getImage() : 0);
+            if (image.valid() && 
                 (image->getPixelFormat()==GL_RGB || image->getPixelFormat()==GL_RGBA) &&
                 (image->s()>=32 && image->t()>=32))
             {
                 texture->setInternalFormatMode(_internalFormatMode);
-
+                
+                // need to disable the unref after apply, other the image could go out of scope.
+                bool unrefImageDataAfterApply = texture->getUnRefImageDataAfterApply();
+                texture->setUnRefImageDataAfterApply(false);
+                
                 // get OpenGL driver to create texture from image.
                 texture->apply(*state);
 
+                // restore the original setting
+                texture->setUnRefImageDataAfterApply(unrefImageDataAfterApply);
+
                 image->readImageFromCurrentTexture(0,true);
 
                 texture->setInternalFormatMode(osg::Texture::USE_IMAGE_DATA_FORMAT);
@@ -415,8 +421,10 @@ int main( int argc, char **argv )
     
     // set up the usage document, in case we need to print out how to use this program.
     arguments.getApplicationUsage()->setApplicationName(arguments.getApplicationName());
-    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is the standard OpenSceneGraph example which loads and visualises 3d models.");
+    arguments.getApplicationUsage()->setDescription(arguments.getApplicationName()+" is a utility for converting between various input and output databases formats.");
     arguments.getApplicationUsage()->setCommandLineUsage(arguments.getApplicationName()+" [options] filename ...");
+    arguments.getApplicationUsage()->addCommandLineOption("-h or --help","Display command line paramters");
+    arguments.getApplicationUsage()->addCommandLineOption("--help-env","Display environmental variables available");
 
 
     // if user request help write it out to cout.
@@ -426,7 +434,12 @@ int main( int argc, char **argv )
         //arguments.getApplicationUsage()->write(std::cout);
         return 1;
     }
-
+    
+    if (arguments.read("--help-env"))
+    { 
+        arguments.getApplicationUsage()->write(std::cout, osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE);
+        return 1;
+    }
     
     if (arguments.argc()<=1)
     {
