@@ -180,6 +180,11 @@ bool validName(const std::string& first)
     if (first=="Osfields") return false;
     if (first=="Optimizer") return false;
     if (first=="RenderStage") return false;
+    if (first=="Matrix*") return false;
+    if (first=="Vec4ub") return false;
+    if (first=="Proxy") return false;
+    if (first=="CullVistor") return false;
+    if (first=="TexMat(Matrix") return false;
     return true;
 }
 
