@@ -1456,8 +1456,9 @@ int main( int argc, char **argv )
 
     viewer.setRealizeCallback(new CompileStateCallback(seh));
 
-    // create the windows and run the threads.
-    viewer.realize();
+    // create the windows and run the threads, note has to be single threaded
+    // due to use osgParticle
+    viewer.realize(Producer::CameraGroup::SingleThreaded);
     
     viewer.requestWarpPointer(0.5f,0.5f);
         
@@ -1477,7 +1478,10 @@ int main( int argc, char **argv )
         viewer.frame();
         
     }
-    
+
+    // wait for all cull and draw threads to complete.
+    viewer.sync();
+
     // run a clean up frame to delete all OpenGL objects.
     viewer.cleanup_frame();
 
