@@ -577,7 +577,13 @@ int main( int argc, char **argv )
         
     }
 
-    // wait for all cull and draw threads to complete before exit.
+    // wait for all cull and draw threads to complete.
+    viewer.sync();
+
+    // run a clean up frame to delete all OpenGL objects.
+    viewer.cleanup_frame();
+
+    // wait for all the clean up frame to complete.
     viewer.sync();
 
     // if we are master clean up by telling all slaves that we're going down.
