@@ -13,6 +13,7 @@
 #include <osg/StateSet>
 #include <osg/Switch>
 #include <osg/Texture2D>
+#include <osg/TexEnv>
 
 #include <osgDB/ReadFile>
 #include <osgDB/FileUtils>
@@ -1132,7 +1133,13 @@ int main( int argc, char **argv )
         
     }
     
-    // wait for all cull and draw threads to complete before exit.
+    // wait for all cull and draw threads to complete.
+    viewer.sync();
+
+    // run a clean up frame to delete all OpenGL objects.
+    viewer.cleanup_frame();
+
+    // wait for all the clean up frame to complete.
     viewer.sync();
 
     return 0;
