@@ -371,11 +371,17 @@ int main( int argc, char **argv )
          
         // fire off the cull and draw traversals of the scene.
         viewer.frame();
-    
-        // wait for all cull and draw threads to complete before exit.
-        viewer.sync();
         
     }
 
+    // wait for all cull and draw threads to complete.
+    viewer.sync();
+
+    // run a clean up frame to delete all OpenGL objects.
+    viewer.cleanup_frame();
+
+    // wait for all the clean up frame to complete.
+    viewer.sync();
+
     return 0;
 }
