@@ -180,8 +180,7 @@ class TexMatCullCallback : public osg::NodeCallback
             osgUtil::CullVisitor* cv = dynamic_cast<osgUtil::CullVisitor*>(nv);
             if (cv)
             {
-                osg::Quat quat;
-                cv->getModelViewMatrix().get(quat);
+                osg::Quat quat = cv->getModelViewMatrix().getRotate();
                 _texmat->setMatrix(osg::Matrix::rotate(quat.inverse()));
             }
         }
@@ -355,7 +354,13 @@ int main(int argc, char** argv)
       viewer.frame();
     }
     
-    // wait for all cull and draw threads to complete before exit.
+    // wait for all cull and draw threads to complete.
+    viewer.sync();
+
+    // run a clean up frame to delete all OpenGL objects.
+    viewer.cleanup_frame();
+
+    // wait for all the clean up frame to complete.
     viewer.sync();
 
     return 0;
