@@ -579,8 +579,8 @@ osg::Node* ForestTechniqueManager::createScene(unsigned int /*numTreesToCreates*
                "    vec4 position;\n"
                "    position.x = gl_Vertex.x;\n"
                "    position.y = gl_Vertex.y;\n"
-               "    float zscale = 0.99;\n"
-               "    position.z = texture2D(terrainTexture, texcoord).r * zscale;\n"
+               "    position.z = texture2D(terrainTexture, texcoord).r;\n"
+               "    position.w = 1.0;\n"
                " \n"
                "    gl_Position     = gl_ModelViewProjectionMatrix * position;\n"
                 "   gl_FrontColor = vec4(1.0,1.0,1.0,1.0);\n"
@@ -846,7 +846,13 @@ int main( int argc, char **argv )
         
     }
     
-    // wait for all cull and draw threads to complete before exit.
+    // wait for all cull and draw threads to complete.
+    viewer.sync();
+
+    // run a clean up frame to delete all OpenGL objects.
+    viewer.cleanup_frame();
+
+    // wait for all the clean up frame to complete.
     viewer.sync();
 
     return 0;
