@@ -183,8 +183,7 @@ public:
             const osg::Matrix R = osg::Matrix::rotate( osg::DegreesToRadians(112.0f), 0.0f,0.0f,1.0f)*
                                   osg::Matrix::rotate( osg::DegreesToRadians(90.0f), 1.0f,0.0f,0.0f);
 
-            osg::Quat q;
-            MV.get(q);
+            osg::Quat q = MV.getRotate();
             const osg::Matrix C = osg::Matrix::rotate( q.inverse() );
 
             _texMat.setMatrix( C*R );
@@ -436,7 +435,13 @@ int main(int argc, char *argv[])
         
     }
     
-    // wait for all cull and draw threads to complete before exit.
+    // wait for all cull and draw threads to complete.
+    viewer.sync();
+
+    // run a clean up frame to delete all OpenGL objects.
+    viewer.cleanup_frame();
+
+    // wait for all the clean up frame to complete.
     viewer.sync();
 
     return 0;
