@@ -1658,8 +1658,13 @@ int main( int argc, char **argv )
 
         }
         
+        // wait for all cull and draw threads to complete.
+        viewer.sync();
 
-        // wait for all cull and draw threads to complete before exit.
+        // run a clean up frame to delete all OpenGL objects.
+        viewer.cleanup_frame();
+
+        // wait for all the clean up frame to complete.
         viewer.sync();
     }    
     
