@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -24,19 +24,27 @@ static char* s_ArrayNames[] =
     "UByteArray",    // 4
     "UShortArray",   // 5
     "UIntArray",     // 6
-    "UByte4Array",   // 7
+    "Vec4ubArray",   // 7
 
     "FloatArray",    // 8
     "Vec2Array",     // 9
     "Vec3Array",     // 10
-    "Vec4Array",      // 11
+    "Vec4Array",     // 11
+
+    "Vec2sArray",    // 12
+    "Vec3sArray",    // 13
+    "Vec4sArray",    // 14
+
+    "Vec2bArray",    // 15
+    "Vec3bArray",    // 16
+    "Vec4bArray",    // 17
 };
 
 const char* Array::className() const
 {
-    if (_arrayType>=ArrayType && _arrayType<=Vec4ArrayType)
+    if (_arrayType>=ArrayType && _arrayType<=Vec4bArrayType)
         return s_ArrayNames[_arrayType];
     else
-        return "UnkownArray";
+        return "UnknownArray";
 }
 
