@@ -30,7 +30,7 @@ using namespace osg;
 // by completely deleted once the appropriate OpenGL context
 // is set.  Used osg::BufferObject::deleteDisplayList(..) and flushDeletedBufferObjects(..) below.
 typedef std::multimap<unsigned int,GLuint> DisplayListMap;
-typedef std::map<unsigned int,DisplayListMap> DeletedBufferObjectCache;
+typedef osg::buffered_object<DisplayListMap> DeletedBufferObjectCache;
 
 static OpenThreads::Mutex s_mutex_deletedBufferObjectCache;
 static DeletedBufferObjectCache s_deletedBufferObjectCache;
@@ -61,28 +61,24 @@ void BufferObject::flushDeletedBufferObjects(unsigned int contextID,double /*cur
     {
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock(s_mutex_deletedBufferObjectCache);
 
-        DeletedBufferObjectCache::iterator citr = s_deletedBufferObjectCache.find(contextID);
-        if (citr!=s_deletedBufferObjectCache.end())
+        const Extensions* extensions = getExtensions(contextID,true);
+
+        unsigned int noDeleted = 0;
+
+        DisplayListMap& dll = s_deletedBufferObjectCache[contextID];
+
+        DisplayListMap::iterator ditr=dll.begin();
+        for(;
+            ditr!=dll.end() && elapsedTime<availableTime;
+            ++ditr)
         {
-            const Extensions* extensions = getExtensions(contextID,true);
-
-            unsigned int noDeleted = 0;
-
-            DisplayListMap& dll = citr->second;
-
-            DisplayListMap::iterator ditr=dll.begin();
-            for(;
-                ditr!=dll.end() && elapsedTime<availableTime;
-                ++ditr)
-            {
-                extensions->glDeleteBuffers(1,&(ditr->second));
-                elapsedTime = timer.delta_s(start_tick,timer.tick());
-                ++noDeleted;
-            }
-            if (ditr!=dll.begin()) dll.erase(dll.begin(),ditr);
-
-            if (noDeleted!=0) notify(osg::INFO)<<"Number VBOs deleted = "<<noDeleted<<std::endl;
+            extensions->glDeleteBuffers(1,&(ditr->second));
+            elapsedTime = timer.delta_s(start_tick,timer.tick());
+            ++noDeleted;
         }
+        if (ditr!=dll.begin()) dll.erase(dll.begin(),ditr);
+
+        if (noDeleted!=0) notify(osg::INFO)<<"Number VBOs deleted = "<<noDeleted<<std::endl;
     }    
     
     availableTime -= elapsedTime;
@@ -186,7 +182,7 @@ void BufferObject::Extensions::lowestCommonDenominator(const Extensions& rhs)
     if (!rhs._glGetBufferParameteriv) _glGetBufferPointerv = rhs._glGetBufferPointerv;
 }
 
-void BufferObject::Extensions::setupGLExtenions(unsigned int)
+void BufferObject::Extensions::setupGLExtenions(unsigned int contextID)
 {
     _glGenBuffers = ((GenBuffersProc)osg::getGLExtensionFuncPtr("glGenBuffers","glGenBuffersARB"));
     _glBindBuffer = ((BindBufferProc)osg::getGLExtensionFuncPtr("glBindBuffer","glBindBufferARB"));
@@ -199,6 +195,7 @@ void BufferObject::Extensions::setupGLExtenions(unsigned int)
     _glUnmapBuffer = ((UnmapBufferProc)osg::getGLExtensionFuncPtr("glUnmapBuffer","glUnmapBufferARB"));
     _glGetBufferParameteriv = ((GetBufferParameterivProc)osg::getGLExtensionFuncPtr("glGetBufferParameteriv","glGetBufferParameterivARB"));
     _glGetBufferPointerv = ((GetBufferPointervProc)osg::getGLExtensionFuncPtr("glGetBufferPointerv","glGetBufferPointervARB"));
+    _isPBOSupported = osg::isGLExtensionSupported(contextID,"GL_ARB_pixel_buffer_object");
 }
 
 void BufferObject::Extensions::glGenBuffers(GLsizei n, GLuint *buffers) const
