@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -49,7 +49,7 @@ Geode::~Geode()
 
 bool Geode::addDrawable( Drawable *drawable )
 {
-    if (drawable && !containsDrawable(drawable))
+    if (drawable /* && !containsDrawable(drawable)*/)
     {
         // note ref_ptr<> automatically handles incrementing drawable's reference count.
         _drawables.push_back(drawable);
@@ -77,10 +77,10 @@ bool Geode::addDrawable( Drawable *drawable )
 
 bool Geode::removeDrawable( Drawable *drawable )
 {
-    return removeDrawable(getDrawableIndex(drawable));
+    return removeDrawables(getDrawableIndex(drawable),1);
 }
 
-bool Geode::removeDrawable(unsigned int pos,unsigned int numDrawablesToRemove)
+bool Geode::removeDrawables(unsigned int pos,unsigned int numDrawablesToRemove)
 {
     if (pos<_drawables.size() && numDrawablesToRemove>0)
     {
