@@ -334,7 +334,7 @@ void Image::setPixelFormat(GLenum pixelFormat)
 {
     if (_pixelFormat==pixelFormat) return; // do nothing if the same.
 
-    if (computeNumComponents(_pixelFormat)==computeNumComponents(pixelFormat))
+    if (_pixelFormat==0 || computeNumComponents(_pixelFormat)==computeNumComponents(pixelFormat))
     {
        // if the two formats have the same number of componets then
        // we can do a straight swap.
@@ -346,6 +346,22 @@ void Image::setPixelFormat(GLenum pixelFormat)
     }
 }
 
+void Image::setDataType(GLenum dataType)
+{
+    if (_dataType==dataType) return; // do nothing if the same.
+
+    if (_dataType==0)
+    {
+        // setting the datatype for the first time
+        _dataType = dataType;
+    }
+    else
+    {
+        notify(WARN)<<"Image::setDataType(..) - warning, attempt to reset the data type not permitted."<<std::endl;
+    }
+}
+
+
 void Image::allocateImage(int s,int t,int r,
                         GLenum format,GLenum type,
                         int packing)
@@ -723,32 +739,41 @@ void Image::flipHorizontal()
 {
     if (_data==NULL)
     {
-        notify(WARN) << "Error Image::flipHorizontal() do not succeed : cannot flip NULL image."<<std::endl;
+        notify(WARN) << "Error Image::flipHorizontal() did not succeed : cannot flip NULL image."<<std::endl;
         return;
     }
 
     unsigned int elemSize = getPixelSizeInBits()/8;
 
-    for(int r=0;r<_r;++r)
+    if (_mipmapData.empty())
     {
-        for (int t=0; t<_t; ++t)
-        {
-            unsigned char* rowData = _data+t*getRowSizeInBytes()+r*getImageSizeInBytes();
-            unsigned char* left  = rowData ;
-            unsigned char* right = rowData + ((_s-1)*getPixelSizeInBits())/8;
 
-            while (left < right)
+        for(int r=0;r<_r;++r)
+        {
+            for (int t=0; t<_t; ++t)
             {
-                char tmp[32];  // max elem size is four floats
-                memcpy(tmp, left, elemSize);
-                memcpy(left, right, elemSize);
-                memcpy(right, tmp, elemSize);
-                left  += elemSize;
-                right -= elemSize;
+                unsigned char* rowData = _data+t*getRowSizeInBytes()+r*getImageSizeInBytes();
+                unsigned char* left  = rowData ;
+                unsigned char* right = rowData + ((_s-1)*getPixelSizeInBits())/8;
+
+                while (left < right)
+                {
+                    char tmp[32];  // max elem size is four floats
+                    memcpy(tmp, left, elemSize);
+                    memcpy(left, right, elemSize);
+                    memcpy(right, tmp, elemSize);
+                    left  += elemSize;
+                    right -= elemSize;
+                }
             }
         }
     }
-    
+    else
+    {
+        notify(WARN) << "Error Image::flipHorizontal() did not succeed : cannot flip mipmapped image."<<std::endl;
+        return;
+    }
+        
     ++_modifiedCount;
 }
 
