@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -52,9 +52,9 @@ void LOD::traverse(NodeVisitor& nv)
             else
             {
                 osg::CullStack* cullStack = dynamic_cast<osg::CullStack*>(&nv);
-                if (cullStack)
+                if (cullStack && cullStack->getLODScale())
                 {
-                    required_range = cullStack->pixelSize(getBound());
+                    required_range = cullStack->clampedPixelSize(getBound()) / cullStack->getLODScale();
                 }
                 else
                 {
@@ -62,7 +62,7 @@ void LOD::traverse(NodeVisitor& nv)
                     // finding out the max range
                     for(unsigned int i=0;i<_rangeList.size();++i)
                     {
-                        required_range = osg::minimum(required_range,_rangeList[i].first);
+                        required_range = osg::maximum(required_range,_rangeList[i].first);
                     }
                 }
             }
@@ -103,8 +103,7 @@ bool LOD::addChild( Node *child )
 
         if (_children.size()>_rangeList.size()) 
         {
-            float maxRange = !_rangeList.empty()?
-                 maxRange=_rangeList.back().second : 0.0f;
+            float maxRange = !_rangeList.empty() ? _rangeList.back().second : 0.0f;
 
             _rangeList.resize(_children.size(),MinMaxPair(maxRange,maxRange));
         }
@@ -114,16 +113,6 @@ bool LOD::addChild( Node *child )
     return false;
 }
 
-void LOD::childRemoved(unsigned int /*pos*/, unsigned int /*numChildrenToRemove*/)
-{
-    //std::cout<<"LOD::childRemoved("<<pos<<","<<numChildrenToRemove<<")"<<std::endl;
-}
-
-void LOD::childInserted(unsigned int /*pos*/)
-{
-    //std::cout<<"LOD::childInserted("<<pos<<")"<<std::endl;
-}
-
 
 bool LOD::addChild(Node *child, float min, float max)
 {
@@ -137,15 +126,11 @@ bool LOD::addChild(Node *child, float min, float max)
     return false;
 }
 
-bool LOD::removeChild( Node *child )
+bool LOD::removeChildren( unsigned int pos,unsigned int numChildrenToRemove)
 {
-    // find the child's position.
-    unsigned int pos=getChildIndex(child);
-    if (pos==_children.size()) return false;
-    
-    _rangeList.erase(_rangeList.begin()+pos);
-    
-    return Group::removeChild(child);    
+    if (pos<_rangeList.size()) _rangeList.erase(_rangeList.begin()+pos, osg::minimum(_rangeList.begin()+(pos+numChildrenToRemove), _rangeList.end()) );
+
+    return Group::removeChildren(pos,numChildrenToRemove);
 }
 
 void LOD::setRange(unsigned int childNo, float min,float max)
