@@ -62,7 +62,7 @@ void Matrix_implementation::set( value_type a00, value_type a01, value_type a02,
 #define QZ  q._v[2]
 #define QW  q._v[3]
 
-void Matrix_implementation::set(const Quat& q_in)
+void Matrix_implementation::setRotate(const Quat& q_in)
 {
     Quat q(q_in);
     double length2 = q.length2();
@@ -103,77 +103,163 @@ void Matrix_implementation::set(const Quat& q_in)
     _mat[0][0] = 1.0 - (yy + zz);
     _mat[1][0] = xy - wz;
     _mat[2][0] = xz + wy;
-    _mat[3][0] = 0.0;
+
 
     _mat[0][1] = xy + wz;
     _mat[1][1] = 1.0 - (xx + zz);
     _mat[2][1] = yz - wx;
-    _mat[3][1] = 0.0;
 
     _mat[0][2] = xz - wy;
     _mat[1][2] = yz + wx;
     _mat[2][2] = 1.0 - (xx + yy);
-    _mat[3][2] = 0.0;
 
+#if 0
     _mat[0][3] = 0.0;
     _mat[1][3] = 0.0;
     _mat[2][3] = 0.0;
+
+    _mat[3][0] = 0.0;
+    _mat[3][1] = 0.0;
+    _mat[3][2] = 0.0;
     _mat[3][3] = 1.0;
+#endif
 }
 
-void Matrix_implementation::get( Quat& q ) const
+#if 1
+
+// David Spillings implementation Mk 2
+Quat Matrix_implementation::getRotate() const
 {
-    // Source: Gamasutra, Rotating Objects Using Quaternions
-    //
-    //http://www.gamasutra.com/features/programming/19980703/quaternions_01.htm
+    Quat q;
 
-    value_type  tr, s;
-    value_type tq[4];
-    int    i, j, k;
+    // From http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/index.htm
+    QW = 0.5 * sqrt( osg::maximum( 0.0, 1.0 + _mat[0][0] + _mat[1][1] + _mat[2][2] ) );
+    QX = 0.5 * sqrt( osg::maximum( 0.0, 1.0 + _mat[0][0] - _mat[1][1] - _mat[2][2] ) );
+    QY = 0.5 * sqrt( osg::maximum( 0.0, 1.0 - _mat[0][0] + _mat[1][1] - _mat[2][2] ) );
+    QZ = 0.5 * sqrt( osg::maximum( 0.0, 1.0 - _mat[0][0] - _mat[1][1] + _mat[2][2] ) );
 
-    int nxt[3] = {1, 2, 0};
+    QX = QX * osg::sign(  _mat[1][2] - _mat[2][1]) ;
+    QY = QY * osg::sign(  _mat[2][0] - _mat[0][2]) ;
+    QZ = QZ * osg::sign(  _mat[0][1] - _mat[1][0]) ;
 
-    tr = _mat[0][0] + _mat[1][1] + _mat[2][2]+1.0;
-
-    // check the diagonal
-    if (tr > 0.0)
-    {
-        s = (value_type)sqrt (tr);
-        QW = s / 2.0;
-        s = 0.5 / s;
-        QX = (_mat[1][2] - _mat[2][1]) * s;
-        QY = (_mat[2][0] - _mat[0][2]) * s;
-        QZ = (_mat[0][1] - _mat[1][0]) * s;
-    }
-    else
-    {
-        // diagonal is negative
-        i = 0;
-        if (_mat[1][1] > _mat[0][0])
-            i = 1;
-        if (_mat[2][2] > _mat[i][i])
-            i = 2;
-        j = nxt[i];
-        k = nxt[j];
-
-        s = (value_type)sqrt ((_mat[i][i] - (_mat[j][j] + _mat[k][k])) + 1.0);
-
-        tq[i] = s * 0.5;
-
-        if (s != 0.0)
-            s = 0.5 / s;
-
-        tq[3] = (_mat[j][k] - _mat[k][j]) * s;
-        tq[j] = (_mat[i][j] + _mat[j][i]) * s;
-        tq[k] = (_mat[i][k] + _mat[k][i]) * s;
-
-        QX = tq[0];
-        QY = tq[1];
-        QZ = tq[2];
-        QW = tq[3];
-    }
+    return q;
 }
 
+#else
+
+    #if 1
+    // David Spillings implementation Mk 1
+    void Matrix_implementation::get( Quat& q ) const
+    {
+        value_type s;
+        value_type tq[4];
+        int    i, j;
+
+        // Use tq to store the largest trace
+        tq[0] = 1 + _mat[0][0]+_mat[1][1]+_mat[2][2];
+        tq[1] = 1 + _mat[0][0]-_mat[1][1]-_mat[2][2];
+        tq[2] = 1 - _mat[0][0]+_mat[1][1]-_mat[2][2];
+        tq[3] = 1 - _mat[0][0]-_mat[1][1]+_mat[2][2];
+
+        // Find the maximum (could also use stacked if's later)
+        j = 0;
+        for(i=1;i<4;i++) j = (tq[i]>tq[j])? i : j;
+
+        // check the diagonal
+        if (j==0)
+        {
+            /* perform instant calculation */
+            QW = tq[0];
+            QX = _mat[1][2]-_mat[2][1]; 
+            QY = _mat[2][0]-_mat[0][2]; 
+            QZ = _mat[0][1]-_mat[1][0]; 
+        }
+        else if (j==1)
+        {
+            QW = _mat[1][2]-_mat[2][1]; 
+            QX = tq[1];
+            QY = _mat[0][1]+_mat[1][0]; 
+            QZ = _mat[2][0]+_mat[0][2]; 
+        }
+        else if (j==2)
+        {
+            QW = _mat[2][0]-_mat[0][2]; 
+            QX = _mat[0][1]+_mat[1][0]; 
+            QY = tq[2];
+            QZ = _mat[1][2]+_mat[2][1]; 
+        }
+        else /* if (j==3) */
+        {
+            QW = _mat[0][1]-_mat[1][0]; 
+            QX = _mat[2][0]+_mat[0][2]; 
+            QY = _mat[1][2]+_mat[2][1]; 
+            QZ = tq[3];
+        }
+
+        s = sqrt(0.25/tq[j]);
+        QW *= s;
+        QX *= s;
+        QY *= s;
+        QZ *= s;
+
+    }
+    #else
+    // Original implementation
+    void Matrix_implementation::get( Quat& q ) const
+    {
+        // Source: Gamasutra, Rotating Objects Using Quaternions
+        //
+        //http://www.gamasutra.com/features/programming/19980703/quaternions_01.htm
+
+        value_type  tr, s;
+        value_type tq[4];
+        int    i, j, k;
+
+        int nxt[3] = {1, 2, 0};
+
+        tr = _mat[0][0] + _mat[1][1] + _mat[2][2]+1.0;
+
+        // check the diagonal
+        if (tr > 0.0)
+        {
+            s = (value_type)sqrt (tr);
+            QW = s / 2.0;
+            s = 0.5 / s;
+            QX = (_mat[1][2] - _mat[2][1]) * s;
+            QY = (_mat[2][0] - _mat[0][2]) * s;
+            QZ = (_mat[0][1] - _mat[1][0]) * s;
+        }
+        else
+        {
+            // diagonal is negative
+            i = 0;
+            if (_mat[1][1] > _mat[0][0])
+                i = 1;
+            if (_mat[2][2] > _mat[i][i])
+                i = 2;
+            j = nxt[i];
+            k = nxt[j];
+
+            s = (value_type)sqrt ((_mat[i][i] - (_mat[j][j] + _mat[k][k])) + 1.0);
+
+            tq[i] = s * 0.5;
+
+            if (s != 0.0)
+                s = 0.5 / s;
+
+            tq[3] = (_mat[j][k] - _mat[k][j]) * s;
+            tq[j] = (_mat[i][j] + _mat[j][i]) * s;
+            tq[k] = (_mat[i][k] + _mat[k][i]) * s;
+
+            QX = tq[0];
+            QY = tq[1];
+            QZ = tq[2];
+            QW = tq[3];
+        }
+    }
+    #endif
+#endif
+
 int Matrix_implementation::compare(const Matrix_implementation& m) const
 {
     const Matrix_implementation::value_type* lhs = reinterpret_cast<const Matrix_implementation::value_type*>(_mat);
@@ -254,62 +340,78 @@ void Matrix_implementation::makeTranslate( value_type x, value_type y, value_typ
 
 void Matrix_implementation::makeRotate( const Vec3f& from, const Vec3f& to )
 {
+    makeIdentity();
+
     Quat quat;
     quat.makeRotate(from,to);
-    set(quat);
+    setRotate(quat);
 }
 void Matrix_implementation::makeRotate( const Vec3d& from, const Vec3d& to )
 {
+    makeIdentity();
+
     Quat quat;
     quat.makeRotate(from,to);
-    set(quat);
+    setRotate(quat);
 }
 
 void Matrix_implementation::makeRotate( value_type angle, const Vec3f& axis )
 {
+    makeIdentity();
+
     Quat quat;
     quat.makeRotate( angle, axis);
-    set(quat);
+    setRotate(quat);
 }
 void Matrix_implementation::makeRotate( value_type angle, const Vec3d& axis )
 {
+    makeIdentity();
+
     Quat quat;
     quat.makeRotate( angle, axis);
-    set(quat);
+    setRotate(quat);
 }
 
 void Matrix_implementation::makeRotate( value_type angle, value_type x, value_type y, value_type z ) 
 {
+    makeIdentity();
+
     Quat quat;
     quat.makeRotate( angle, x, y, z);
-    set(quat);
+    setRotate(quat);
 }
 
 void Matrix_implementation::makeRotate( const Quat& quat )
 {
-    set(quat);
+    makeIdentity();
+
+    setRotate(quat);
 }
 
 void Matrix_implementation::makeRotate( value_type angle1, const Vec3f& axis1, 
                          value_type angle2, const Vec3f& axis2,
                          value_type angle3, const Vec3f& axis3)
 {
+    makeIdentity();
+
     Quat quat;
     quat.makeRotate(angle1, axis1, 
                     angle2, axis2,
                     angle3, axis3);
-    set(quat);
+    setRotate(quat);
 }
 
 void Matrix_implementation::makeRotate( value_type angle1, const Vec3d& axis1, 
                          value_type angle2, const Vec3d& axis2,
                          value_type angle3, const Vec3d& axis3)
 {
+    makeIdentity();
+
     Quat quat;
     quat.makeRotate(angle1, axis1, 
                     angle2, axis2,
                     angle3, axis3);
-    set(quat);
+    setRotate(quat);
 }
 
 void Matrix_implementation::mult( const Matrix_implementation& lhs, const Matrix_implementation& rhs )
@@ -446,45 +548,6 @@ void Matrix_implementation::orthoNormalize(const Matrix_implementation& rhs)
 
 }
 
-bool Matrix_implementation::invert( const Matrix_implementation& rhs)
-{
-#if 1
-    return invert_4x4_new(rhs);
-#else
-    static const osg::Timer& timer = *Timer::instance();
-
-    Matrix_implementation a;
-    Matrix_implementation b;
-
-    Timer_t t1 = timer.tick();
-
-    a.invert_4x4_new(rhs);
-    
-    Timer_t t2 = timer.tick();
-
-    b.invert_4x4_orig(rhs);
-    
-    Timer_t t3 = timer.tick();
-
-    static double new_time = 0.0;
-    static double orig_time = 0.0;
-    static double count = 0.0;
-    
-    new_time += timer.delta_u(t1,t2);
-    orig_time += timer.delta_u(t2,t3);
-    ++count;
-    
-    std::cout<<"Average new="<<new_time/count<<"  orig = "<<orig_time/count<<std::endl;
-
-    std::cout<<"new matrix invert time="<<timer.delta_u(t1,t2)<<" "<<a<<std::endl;
-    std::cout<<"orig matrix invert time="<<timer.delta_u(t2,t3)<<" "<<b<<std::endl;
-    
-    set(b);
-    
-    return true;
-#endif
-}
-
 /******************************************
   Matrix inversion technique:
 Given a matrix mat, we want to invert it.
@@ -511,12 +574,12 @@ So the inverse is mat' = (trans * corr)' * rot', where rot' must be computed the
 This problem is simplified if [px py pz s] = [0 0 0 1], which will happen if mat was composed only of rotations, scales, and translations (which is common).  In this case, we can ignore corr entirely which saves on a lot of computations.
 ******************************************/
 
-bool Matrix_implementation::invert_4x4_new( const Matrix_implementation& mat )
+bool Matrix_implementation::invert_4x3( const Matrix_implementation& mat )
 {
     if (&mat==this)
     {
        Matrix_implementation tm(mat);
-       return invert_4x4_new(tm);
+       return invert_4x3(tm);
     }
 
     register value_type r00, r01, r02,
@@ -641,11 +704,11 @@ inline T SGL_ABS(T a)
 #define SGL_SWAP(a,b,temp) ((temp)=(a),(a)=(b),(b)=(temp))
 #endif
 
-bool Matrix_implementation::invert_4x4_orig( const Matrix_implementation& mat )
+bool Matrix_implementation::invert_4x4( const Matrix_implementation& mat )
 {
     if (&mat==this) {
        Matrix_implementation tm(mat);
-       return invert_4x4_orig(tm);
+       return invert_4x4(tm);
     }
 
     unsigned int indxc[4], indxr[4], ipiv[4];
