@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -16,11 +16,42 @@
 #include <osg/NodeVisitor>
 #include <osg/Notify>
 #include <osg/OccluderNode>
+#include <osg/CameraNode>
 
 #include <algorithm>
 
 using namespace osg;
 
+namespace osg
+{
+    /// Helper class for generating NodePathList.
+    class CollectParentPaths : public NodeVisitor
+    {
+    public:
+        CollectParentPaths(osg::Node* haltTraversalAtNode=0) : 
+            osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_PARENTS),
+            _haltTraversalAtNode(haltTraversalAtNode)
+        {
+        }
+
+        virtual void apply(osg::Node& node)
+        {
+            if (node.getNumParents()==0 || &node==_haltTraversalAtNode)
+            {
+                _nodePaths.push_back(getNodePath());
+            }
+            else
+            {
+                traverse(node);
+            }
+       }
+
+        Node*           _haltTraversalAtNode;
+        NodePath        _nodePath;
+        NodePathList    _nodePaths;
+    };
+}
+
 Node::Node()
 {
     _boundingSphereComputed = false;
@@ -41,7 +72,6 @@ Node::Node(const Node& node,const CopyOp& copyop):
         _initialBound(node._initialBound),
         _boundingSphere(node._boundingSphere),
         _boundingSphereComputed(node._boundingSphereComputed),
-        _name(node._name),
         _parents(), // leave empty as parentList is managed by Group.
         _updateCallback(node._updateCallback),
         _numChildrenRequiringUpdateTraversal(0), // assume no children yet.
@@ -58,6 +88,8 @@ Node::Node(const Node& node,const CopyOp& copyop):
 
 Node::~Node()
 {
+    // cleanly detatch any associated stateset (include remove parent links)
+    setStateSet(0);   
 }
 
 void Node::addParent(osg::Group* node)
@@ -132,6 +164,37 @@ osg::StateSet* Node::getOrCreateStateSet()
     return _stateset.get();
 }
 
+NodePathList Node::getParentalNodePaths(osg::Node* haltTraversalAtNode) const
+{
+    CollectParentPaths cpp(haltTraversalAtNode);
+    const_cast<Node*>(this)->accept(cpp);
+    return cpp._nodePaths;
+}
+
+MatrixList Node::getWorldMatrices(osg::Node* haltTraversalAtNode) const
+{
+    CollectParentPaths cpp(haltTraversalAtNode);
+    const_cast<Node*>(this)->accept(cpp);
+    
+    MatrixList matrices;
+    
+    for(NodePathList::iterator itr = cpp._nodePaths.begin();
+        itr != cpp._nodePaths.end();
+        ++itr)
+    {
+        NodePath& nodePath = *itr;
+        if (nodePath.empty())
+        {
+            matrices.push_back(osg::Matrix::identity());
+        }
+        else
+        {
+            matrices.push_back(osg::computeLocalToWorld(nodePath));
+        }
+    }
+    
+    return matrices;
+}
 
 void Node::setUpdateCallback(NodeCallback* nc)
 {
