@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -13,9 +13,11 @@
 #include <osg/Referenced>
 #include <osg/Notify>
 #include <osg/ApplicationUsage>
+#include <osg/observer_ptr>
 
 #include <typeinfo>
 #include <memory>
+#include <set>
 
 #include <OpenThreads/ScopedLock>
 #include <OpenThreads/Mutex>
@@ -25,8 +27,58 @@
 namespace osg
 {
 
+// specialzed smart pointer, used to get round auto_ptr<>'s lack of the destructor reseting itself to 0.
+struct DeleteHandlerPointer
+{
+    DeleteHandlerPointer():
+        _ptr(0) {}
+
+    DeleteHandlerPointer(DeleteHandler* ptr):
+        _ptr(ptr) {}
+
+    ~DeleteHandlerPointer()
+    {
+        delete _ptr;
+        _ptr = 0;
+    }
+
+    inline DeleteHandlerPointer& operator = (DeleteHandler* ptr)
+    {
+        if (_ptr==ptr) return *this;
+        delete _ptr;
+        _ptr = ptr;
+        return *this;
+    }
+
+    void reset(DeleteHandler* ptr)
+    {
+        if (_ptr==ptr) return;
+        delete _ptr;
+        _ptr = ptr;
+    }
+
+    inline DeleteHandler& operator*()  { return *_ptr; }
+
+    inline const DeleteHandler& operator*() const { return *_ptr; }
+
+    inline DeleteHandler* operator->() { return _ptr; }
+
+    inline const DeleteHandler* operator->() const   { return _ptr; }
+
+    DeleteHandler* get() { return _ptr; }
+
+    const DeleteHandler* get() const { return _ptr; }
+
+    DeleteHandler* _ptr;
+};
+
+
+typedef std::set<Observer*> ObserverSet;
+
 static bool s_useThreadSafeReferenceCounting = getenv("OSG_THREAD_SAFE_REF_UNREF")!=0;
-static std::auto_ptr<DeleteHandler> s_deleteHandler(0);
+// static std::auto_ptr<DeleteHandler> s_deleteHandler(0);
+static DeleteHandlerPointer s_deleteHandler(0);
+
 static ApplicationUsageProxy Referenced_e0(ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_THREAD_SAFE_REF_UNREF","");
 
 void Referenced::setThreadSafeReferenceCounting(bool enableThreadSafeReferenceCounting)
@@ -50,18 +102,28 @@ DeleteHandler* Referenced::getDeleteHandler()
     return s_deleteHandler.get();
 }
 
-Referenced::Referenced()
+Referenced::Referenced():
+    _refMutex(0),
+    _refCount(0),
+    _observers(0)
 {
     if (s_useThreadSafeReferenceCounting) _refMutex = new OpenThreads::Mutex;
-    else _refMutex = 0;
-   _refCount=0;
 }
 
-Referenced::Referenced(const Referenced&)
+Referenced::Referenced(bool threadSafeRefUnref):
+    _refMutex(0),
+    _refCount(0),
+    _observers(0)
+{
+    if (threadSafeRefUnref) _refMutex = new OpenThreads::Mutex;
+}
+
+Referenced::Referenced(const Referenced&):
+    _refMutex(0),
+    _refCount(0),
+    _observers(0)
 {
     if (s_useThreadSafeReferenceCounting) _refMutex = new OpenThreads::Mutex;
-    else _refMutex = 0;
-    _refCount=0;
 }
 
 Referenced::~Referenced()
@@ -72,6 +134,19 @@ Referenced::~Referenced()
         notify(WARN)<<"         the final reference count was "<<_refCount<<", memory corruption possible."<<std::endl;
     }
 
+    if (_observers)
+    {
+        ObserverSet* os = static_cast<ObserverSet*>(_observers);
+        for(ObserverSet::iterator itr = os->begin();
+            itr != os->end();
+            ++itr)
+        {
+            (*itr)->objectDeleted(this);
+        }
+        delete os;
+        _observers = 0;
+    }
+
     if (_refMutex)
     {
         OpenThreads::Mutex* tmpMutexPtr = _refMutex;
@@ -151,7 +226,36 @@ void Referenced::unref_nodelete() const
         --_refCount;
     }
 }
-        
+
+void Referenced::addObserver(Observer* observer)
+{
+    if (_refMutex)
+    {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*_refMutex); 
+
+        if (!_observers) _observers = new ObserverSet;
+        if (_observers) static_cast<ObserverSet*>(_observers)->insert(observer);
+    }
+    else
+    {
+        if (!_observers) _observers = new ObserverSet;
+        if (_observers) static_cast<ObserverSet*>(_observers)->insert(observer);
+    }
+}
+
+void Referenced::removeObserver(Observer* observer)
+{
+    if (_refMutex)
+    {
+        OpenThreads::ScopedLock<OpenThreads::Mutex> lock(*_refMutex); 
+
+        if (_observers) static_cast<ObserverSet*>(_observers)->erase(observer);
+    }
+    else
+    {
+        if (_observers) static_cast<ObserverSet*>(_observers)->erase(observer);
+    }
+}
 
 }; // end of namespace osg
 
