@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -28,7 +28,7 @@ void HeightField::allocate(unsigned int numColumns,unsigned int numRows)
 {
     if (_columns!=numColumns || _rows!=numRows)
     {
-    	_heights.resize(numColumns*numRows);
+        _heights.resize(numColumns*numRows);
     }
     _columns=numColumns;
     _rows=numRows;
@@ -37,33 +37,32 @@ void HeightField::allocate(unsigned int numColumns,unsigned int numRows)
 Vec3 HeightField::getNormal(unsigned int c,unsigned int r) const 
 {
     // four point normal generation.
-
-    float dz_dx;
+   float dz_dx;
     if (c==0)
     {
-    	dz_dx = (getHeight(c+1,r)-getHeight(c,r))/getXInterval();
+        dz_dx = (getHeight(c+1,r)-getHeight(c,r))/getXInterval();
     }
     else if (c==getNumColumns()-1)
     {
-    	dz_dx = (getHeight(c,r)-getHeight(c-1,r))/getXInterval();
+        dz_dx = (getHeight(c,r)-getHeight(c-1,r))/getXInterval();
     }
     else // assume 0<c<_numColumns-1
     {
-    	dz_dx = 0.5f*(getHeight(c+1,r)-getHeight(c-1,r))/getXInterval();
+        dz_dx = 0.5f*(getHeight(c+1,r)-getHeight(c-1,r))/getXInterval();
     }
     
     float dz_dy;
     if (r==0)
     {
-    	dz_dy = (getHeight(c,r+1)-getHeight(c,r))/getYInterval();
+        dz_dy = (getHeight(c,r+1)-getHeight(c,r))/getYInterval();
     }
     else if (r==getNumRows()-1)
     {
-    	dz_dy = (getHeight(c,r)-getHeight(c,r-1))/getYInterval();
+        dz_dy = (getHeight(c,r)-getHeight(c,r-1))/getYInterval();
     }
     else // assume 0<r<_numRows-1
     {
-    	dz_dy = 0.5f*(getHeight(c,r+1)-getHeight(c,r-1))/getYInterval();
+        dz_dy = 0.5f*(getHeight(c,r+1)-getHeight(c,r-1))/getYInterval();
     }
     
     Vec3 normal(-dz_dx,-dz_dy,1.0f);
@@ -72,4 +71,37 @@ Vec3 HeightField::getNormal(unsigned int c,unsigned int r) const
     return normal;
 }
 
+Vec2 HeightField::getHeightDelta(unsigned int c,unsigned int r) const
+{
+     // four point normal generation.
+    Vec2 heightDelta;
+    if (c==0)
+    {
+        heightDelta.x() = (getHeight(c+1,r)-getHeight(c,r));
+    }
+    else if (c==getNumColumns()-1)
+    {
+        heightDelta.x() = (getHeight(c,r)-getHeight(c-1,r));
+    }
+    else // assume 0<c<_numColumns-1
+    {
+        heightDelta.x() = 0.5f*(getHeight(c+1,r)-getHeight(c-1,r));
+    }
+    
+    if (r==0)
+    {
+        heightDelta.y() = (getHeight(c,r+1)-getHeight(c,r));
+    }
+    else if (r==getNumRows()-1)
+    {
+        heightDelta.y() = (getHeight(c,r)-getHeight(c,r-1));
+    }
+    else // assume 0<r<_numRows-1
+    {
+        heightDelta.y() = 0.5f*(getHeight(c,r+1)-getHeight(c,r-1));
+    }
+
+    return heightDelta;
+}
+
 
