@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -13,6 +13,7 @@
 #include <osg/Switch>
 #include <osg/BoundingBox>
 #include <osg/Transform>
+#include <osg/Notify>
 
 #include <algorithm>
 
@@ -50,12 +51,12 @@ bool Switch::addChild( Node *child )
     if (Group::addChild(child))
     {
         if (_children.size()>_values.size())
-	{
-	    _values.resize(_children.size(),_newChildDefaultValue);
-	}
+        {
+            _values.resize(_children.size(),_newChildDefaultValue);
+        }
         // note, we don't override any pre-existing _values[childPosition] setting
         // like in addChild(child,value) below.
-	return true;
+        return true;
     }
     return false;
 }
@@ -66,11 +67,11 @@ bool Switch::addChild( Node *child, bool value )
     if (Group::addChild(child))
     {
         if (_children.size()>_values.size())
-	{
-	    _values.resize(_children.size(),_newChildDefaultValue);
-	}
-	_values[childPosition]=value;
-	return true;
+        {
+            _values.resize(_children.size(),_newChildDefaultValue);
+        }
+        _values[childPosition]=value;
+        return true;
     }
     return false;
 }
@@ -85,34 +86,31 @@ bool Switch::insertChild( unsigned int index, Node *child, bool value )
     if (Group::insertChild(index,child))
     {
         if (index>=_values.size())
-	{
-	    _values.push_back(value);
-	}
+        {
+            _values.push_back(value);
+        }
         else
         {
-	    _values.insert(_values.begin()+index, value);
+            _values.insert(_values.begin()+index, value);
         }
         
-	return true;
+        return true;
     }
     return false;
 }
 
-bool Switch::removeChild( Node *child )
+bool Switch::removeChildren(unsigned int pos,unsigned int numChildrenToRemove)
 {
-    // find the child's position.
-    unsigned int pos=getChildIndex(child);
-    if (pos==_children.size()) return false;
-    
-    _values.erase(_values.begin()+pos);
-    
-    return Group::removeChild(child);    
+    if (pos<_values.size()) _values.erase(_values.begin()+pos, osg::minimum(_values.begin()+(pos+numChildrenToRemove), _values.end()) );
+
+    return Group::removeChildren(pos,numChildrenToRemove);
 }
 
 void Switch::setValue(unsigned int pos,bool value)
 {
     if (pos>=_values.size()) _values.resize(pos+1,_newChildDefaultValue);
     _values[pos]=value;
+    dirtyBound();
 }
 
 void Switch::setChildValue(const Node* child,bool value)
@@ -122,6 +120,7 @@ void Switch::setChildValue(const Node* child,bool value)
     if (pos==_children.size()) return;
     
     _values[pos]=value;
+    dirtyBound();
 }
 
 bool Switch::getValue(unsigned int pos) const
@@ -148,6 +147,7 @@ bool Switch::setAllChildrenOff()
     {
         *itr = false;
     }
+    dirtyBound();
     return true;
 }
 
@@ -160,6 +160,7 @@ bool Switch::setAllChildrenOn()
     {
         *itr = true;
     }
+    dirtyBound();
     return true;
 }
 
