@@ -65,7 +65,15 @@ class TextureObjectManager : public osg::Referenced
 public:
 
     TextureObjectManager():
-        _expiryDelay(0.0) {}
+        _expiryDelay(0.0)
+    {
+        // printf("Constructing TextureObjectManager\n");
+    }
+
+    ~TextureObjectManager()
+    {
+        // printf("Destructing TextureObjectManager\n");
+    }
 
     virtual Texture::TextureObject* generateTextureObject(unsigned int contextID,GLenum target);
 
@@ -191,12 +199,10 @@ void TextureObjectManager::addTextureObjects(Texture::TextureObjectListMap& tobl
 {
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
 
-    for(Texture::TextureObjectListMap::iterator itr = toblm.begin();
-        itr != toblm.end();
-        ++itr)
+    for(unsigned int i=0; i< toblm.size(); ++i)
     {
-        Texture::TextureObjectList& tol = _textureObjectListMap[itr->first];
-        tol.insert(tol.end(),itr->second.begin(),itr->second.end());
+        Texture::TextureObjectList& tol = _textureObjectListMap[i];
+        tol.insert(tol.end(),toblm[i].begin(),toblm[i].end());
     }
 }
 
@@ -211,19 +217,15 @@ void TextureObjectManager::flushAllTextureObjects(unsigned int contextID)
 {
     OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
 
-    Texture::TextureObjectListMap::iterator tmitr = _textureObjectListMap.find(contextID);
-    if (tmitr!=_textureObjectListMap.end())
-    {
-        Texture::TextureObjectList& tol = tmitr->second;
+    Texture::TextureObjectList& tol = _textureObjectListMap[contextID];
 
-        for(Texture::TextureObjectList::iterator itr=tol.begin();
-            itr!=tol.end();
-            ++itr)
-        {
-            glDeleteTextures( 1L, &((*itr)->_id));
-        }
-        tol.clear();
+    for(Texture::TextureObjectList::iterator itr=tol.begin();
+        itr!=tol.end();
+        ++itr)
+    {
+        glDeleteTextures( 1L, &((*itr)->_id));
     }
+    tol.clear();
 }
 
 void TextureObjectManager::flushTextureObjects(unsigned int contextID,double currentTime, double& availableTime)
@@ -242,43 +244,38 @@ void TextureObjectManager::flushTextureObjects(unsigned int contextID,double cur
     {    
         OpenThreads::ScopedLock<OpenThreads::Mutex> lock(_mutex);
 
-        Texture::TextureObjectListMap::iterator tmitr = _textureObjectListMap.find(contextID);
-        if (tmitr!=_textureObjectListMap.end())
+        Texture::TextureObjectList& tol = _textureObjectListMap[contextID];
+
+        // reset the time of any uninitialized objects.
+        Texture::TextureObjectList::iterator itr;
+        for(itr=tol.begin();
+            itr!=tol.end();
+            ++itr)
         {
-            Texture::TextureObjectList& tol = tmitr->second;
+            if ((*itr)->_timeStamp==0.0) (*itr)->_timeStamp=currentTime;
+        }
 
-            // reset the time of any uninitialized objects.
-            Texture::TextureObjectList::iterator itr;
-            for(itr=tol.begin();
-                itr!=tol.end();
-                ++itr)
+        double expiryTime = currentTime-_expiryDelay;
+
+        for(itr=tol.begin();
+            itr!=tol.end() && elapsedTime<availableTime && tol.size()>s_minimumNumberOfTextureObjectsToRetainInCache && numObjectsDeleted<maxNumObjectsToDelete;
+            )
+        {
+            if ((*itr)->_timeStamp<=expiryTime)
             {
-                if ((*itr)->_timeStamp==0.0) (*itr)->_timeStamp=currentTime;
+                --s_number;
+                ++Texture::s_numberDeletedTextureInLastFrame;
+
+                glDeleteTextures( 1L, &((*itr)->_id));
+                itr = tol.erase(itr);
+                ++numTexturesDeleted;
+                ++numObjectsDeleted;
             }
-
-            double expiryTime = currentTime-_expiryDelay;
-
-            for(itr=tol.begin();
-                itr!=tol.end() && elapsedTime<availableTime && tol.size()>s_minimumNumberOfTextureObjectsToRetainInCache && numObjectsDeleted<maxNumObjectsToDelete;
-                )
+            else
             {
-                if ((*itr)->_timeStamp<=expiryTime)
-                {
-                    --s_number;
-                    ++Texture::s_numberDeletedTextureInLastFrame;
-                
-                    glDeleteTextures( 1L, &((*itr)->_id));
-                    itr = tol.erase(itr);
-                    ++numTexturesDeleted;
-                    ++numObjectsDeleted;
-                }
-                else
-                {
-                    ++itr;
-                }
-                elapsedTime = timer.delta_s(start_tick,timer.tick());
+                ++itr;
             }
-
+            elapsedTime = timer.delta_s(start_tick,timer.tick());
         }
     }
     elapsedTime = timer.delta_s(start_tick,timer.tick());
@@ -290,15 +287,15 @@ void TextureObjectManager::flushTextureObjects(unsigned int contextID,double cur
 
 static TextureObjectManager* getTextureObjectManager()
 {
-    static ref_ptr<TextureObjectManager> s_textureObjectManager;
-    if (!s_textureObjectManager) s_textureObjectManager = new TextureObjectManager;
+    static ref_ptr<TextureObjectManager> s_textureObjectManager = new TextureObjectManager;
     return s_textureObjectManager.get();
 }
 
 
 Texture::TextureObject* Texture::generateTextureObject(unsigned int contextID,GLenum target)
 {
-    return getTextureObjectManager()->generateTextureObject(contextID,target);
+    if (getTextureObjectManager()) return getTextureObjectManager()->generateTextureObject(contextID,target);
+    else return 0;
 }
 
 Texture::TextureObject* Texture::generateTextureObject(unsigned int contextID,
@@ -310,7 +307,8 @@ Texture::TextureObject* Texture::generateTextureObject(unsigned int contextID,
                                              GLsizei   depth,
                                              GLint     border)
 {
-    return getTextureObjectManager()->reuseOrGenerateTextureObject(contextID,
+    if (getTextureObjectManager())    
+        return getTextureObjectManager()->reuseOrGenerateTextureObject(contextID,
                                              target,
                                              numMipmapLevels,
                                              internalFormat,
@@ -318,16 +316,18 @@ Texture::TextureObject* Texture::generateTextureObject(unsigned int contextID,
                                              height,
                                              depth,
                                              border);
+    else
+        return 0;
 }                                             
 
 void Texture::flushAllDeletedTextureObjects(unsigned int contextID)
 {
-    getTextureObjectManager()->flushAllTextureObjects(contextID);
+    if (getTextureObjectManager()) getTextureObjectManager()->flushAllTextureObjects(contextID);
 }
 
 void Texture::flushDeletedTextureObjects(unsigned int contextID,double currentTime, double& availbleTime)
 {
-    getTextureObjectManager()->flushTextureObjects(contextID, currentTime, availbleTime);
+    if (getTextureObjectManager()) getTextureObjectManager()->flushTextureObjects(contextID, currentTime, availbleTime);
 }
 
 Texture::Texture():
@@ -417,6 +417,17 @@ int Texture::compareTexture(const Texture& rhs) const
     return 0;
 }
 
+int Texture::compareTextureObjects(const Texture& rhs) const
+{
+    if (_textureObjectBuffer.size()<rhs._textureObjectBuffer.size()) return -1; 
+    if (rhs._textureObjectBuffer.size()<_textureObjectBuffer.size()) return 1;
+    for(unsigned int i=0; i<_textureObjectBuffer.size(); ++i)
+    {
+        if (_textureObjectBuffer[i] < rhs._textureObjectBuffer[i]) return -1;
+        else if (rhs._textureObjectBuffer[i] < _textureObjectBuffer[i]) return 1;
+    }
+    return 0;
+}
 
 void Texture::setWrap(WrapParameter which, WrapMode wrap)
 {
@@ -476,7 +487,7 @@ void Texture::setMaxAnisotropy(float anis)
 /** Force a recompile on next apply() of associated OpenGL texture objects.*/
 void Texture::dirtyTextureObject()
 {
-    getTextureObjectManager()->addTextureObjectsFrom(*this);
+    if (getTextureObjectManager()) getTextureObjectManager()->addTextureObjectsFrom(*this);
 }
 
 void Texture::takeTextureObjects(Texture::TextureObjectListMap& toblm)
@@ -886,7 +897,7 @@ void Texture::applyTexImage2D_load(State& state, GLenum target, const Image* ima
     unsigned char* dataPlusOffset = 0;
 
     const PixelBufferObject* pbo = image->getPixelBufferObject();
-    if (pbo && pbo->isBufferObjectSupported(contextID) && !needImageRescale)
+    if (pbo && pbo->isPBOSupported(contextID) && !needImageRescale)
     {
         pbo->compileBuffer(state);
         pbo->bindBuffer(contextID);
@@ -1138,7 +1149,7 @@ void Texture::applyTexImage2D_subload(State& state, GLenum target, const Image*
     unsigned char* dataPlusOffset=0;
 
     const PixelBufferObject* pbo = image->getPixelBufferObject();
-    if (pbo && pbo->isBufferObjectSupported(contextID) && !needImageRescale)
+    if (pbo && pbo->isPBOSupported(contextID) && !needImageRescale)
     {
         pbo->compileBuffer(state);
         pbo->bindBuffer(contextID);
@@ -1290,7 +1301,7 @@ void Texture::releaseGLObjects(State* state) const
     else
     {
         unsigned int contextID = state->getContextID();
-        if (_textureObjectBuffer[contextID].valid()) 
+        if (_textureObjectBuffer[contextID].valid() && getTextureObjectManager()) 
         {
             OpenThreads::ScopedLock<OpenThreads::Mutex> lock(getTextureObjectManager()->_mutex);
             
