@@ -99,6 +99,17 @@ int TextureRectangle::compare(const StateAttribute& sa) const
         }
     }
 
+    if (!_image && !rhs._image)
+    {
+        // no image attached to either Texture2D
+        // but could these textures already be downloaded?
+        // check the _textureObjectBuffer to see if they have been
+        // downloaded
+
+        int result = compareTextureObjects(rhs);
+        if (result!=0) return result;
+    }
+
     int result = compareTexture(rhs);
     if (result!=0) return result;
 
@@ -272,7 +283,7 @@ void TextureRectangle::applyTexImage_load(GLenum target, Image* image, State& st
     unsigned char* dataPlusOffset = 0;
 
     const PixelBufferObject* pbo = image->getPixelBufferObject();
-    if (pbo && pbo->isBufferObjectSupported(contextID))
+    if (pbo && pbo->isPBOSupported(contextID))
     {
         pbo->compileBuffer(state);
         pbo->bindBuffer(contextID);
@@ -339,7 +350,7 @@ void TextureRectangle::applyTexImage_subload(GLenum target, Image* image, State&
     unsigned char* dataPlusOffset = 0;
 
     const PixelBufferObject* pbo = image->getPixelBufferObject();
-    if (pbo && pbo->isBufferObjectSupported(contextID))
+    if (pbo && pbo->isPBOSupported(contextID))
     {
         pbo->compileBuffer(state);
         pbo->bindBuffer(contextID);
