@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -16,6 +16,7 @@
 
 #include <osg/Notify>
 
+#include <sstream>
 #include <stdio.h>
 
 using namespace std;
@@ -43,6 +44,8 @@ void Output::init()
     _indentStep = 2;
     _numIndicesPerLine = 10;
     _pathNameHint = AS_IS;
+    _outputTextureFiles = false;
+    _textureFileNameNumber = 0;
 }
 
 void Output::open(const char *name)
@@ -89,12 +92,12 @@ std::string Output::wrapString(const std::string& str)
     newstring += '"';
     for(unsigned int i=0;i<str.size();++i)
     {
-    	if (str[i]=='"')
-	{
-	    newstring += '\\';
-	    newstring += '"';
-	}
-	else newstring += (str[i]);
+        if (str[i]=='"')
+        {
+            newstring += '\\';
+            newstring += '"';
+        }
+        else newstring += (str[i]);
     }
     newstring += '"';
     return newstring;
@@ -157,3 +160,20 @@ std::string Output::getFileNameForOutput(const std::string& filename) const
         return filename;
     }
 }
+
+std::string Output::getTextureFileNameForOutput()
+{
+    std::string fileName = osgDB::getNameLessExtension(_filename);
+    if (_textureFileNameNumber>0)
+    {
+        std::ostringstream o;
+        o << '_' << _textureFileNameNumber;
+        fileName += o.str();
+    }
+    
+    fileName += ".dds";
+    ++_textureFileNameNumber;
+    
+    return fileName;
+}
+
