@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -20,13 +20,16 @@ using namespace osgGA;
 void GUIEventHandler::operator()(osg::Node* node, osg::NodeVisitor* nv)
 {
     osgGA::EventVisitor* ev = dynamic_cast<osgGA::EventVisitor*>(nv);
-    if (ev && ev->getActionAdapter() && !ev->getEventList().empty())
+    if (ev && ev->getActionAdapter() && !ev->getEvents().empty())
     {
-        for(osgGA::EventVisitor::EventList::iterator itr = ev->getEventList().begin();
-            itr != ev->getEventList().end();
+        for(osgGA::EventQueue::Events::iterator itr = ev->getEvents().begin();
+            itr != ev->getEvents().end();
             ++itr)
         {
-            handle(*(*itr), *(ev->getActionAdapter()), node, nv);
+            if (handle(*(*itr), *(ev->getActionAdapter()), node, nv))
+            {
+                return;
+            }
         }
     }
     traverse(node,nv);
@@ -35,10 +38,10 @@ void GUIEventHandler::operator()(osg::Node* node, osg::NodeVisitor* nv)
 void GUIEventHandler::event(osg::NodeVisitor* nv, osg::Drawable* drawable)
 {
     osgGA::EventVisitor* ev = dynamic_cast<osgGA::EventVisitor*>(nv);
-    if (ev && ev->getActionAdapter() && !ev->getEventList().empty())
+    if (ev && ev->getActionAdapter() && !ev->getEvents().empty())
     {
-        for(osgGA::EventVisitor::EventList::iterator itr = ev->getEventList().begin();
-            itr != ev->getEventList().end();
+        for(osgGA::EventQueue::Events::iterator itr = ev->getEvents().begin();
+            itr != ev->getEvents().end();
             ++itr)
         {
             handle(*(*itr), *(ev->getActionAdapter()), drawable, nv);
