@@ -207,6 +207,16 @@ void Type::getAllProperties(PropertyInfoList& props) const
     }
 }
 
+void Type::getPropertiesMap(PropertyInfoMap& props) const
+{
+    check_defined();
+    props[this] = _props;
+    for (TypeList::const_iterator i=_base.begin(); i!=_base.end(); ++i)
+    {
+        (*i)->getPropertiesMap(props);
+    }
+}
+
 void Type::getAllMethods(MethodInfoList& methods) const
 {
     check_defined();
@@ -217,6 +227,16 @@ void Type::getAllMethods(MethodInfoList& methods) const
     }
 }
 
+void Type::getMethodsMap(MethodInfoMap& methods) const
+{
+    check_defined();
+    methods[this] = _methods;
+    for (TypeList::const_iterator i=_base.begin(); i!=_base.end(); ++i)
+    {
+        (*i)->getMethodsMap(methods);
+    }
+}
+
 Value Type::createInstance(ValueList& args) const
 {
     if (isAbstract())
