@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -27,6 +27,20 @@
 
 using namespace osgParticle;
 
+ExplosionEffect::ExplosionEffect(bool automaticSetup):
+    ParticleEffect(automaticSetup)
+{
+    setDefaults();
+    
+    _position.set(0.0f,0.0f,0.0f);
+    _scale = 1.0f;
+    _intensity = 1.0f;
+    
+    _emitterDuration = 1.0;
+    
+    if (_automaticSetup) buildEffect();
+}
+
 ExplosionEffect::ExplosionEffect(const osg::Vec3& position, float scale, float intensity)
 {
     setDefaults();
@@ -37,7 +51,7 @@ ExplosionEffect::ExplosionEffect(const osg::Vec3& position, float scale, float i
     
     _emitterDuration = 1.0;
     
-    buildEffect();
+    if (_automaticSetup) buildEffect();
 }
 
 ExplosionEffect::ExplosionEffect(const ExplosionEffect& copy, const osg::CopyOp& copyop):
