@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -26,6 +26,21 @@
 
 using namespace osgParticle;
 
+FireEffect::FireEffect(bool automaticSetup):
+    ParticleEffect(automaticSetup)
+{
+    setDefaults();
+    
+    _position.set(0.0f,0.0f,0.0f);
+    _scale = 1.0f;
+    _intensity = 1.0f;
+     
+    _emitterDuration = 60.0;
+    _defaultParticleTemplate.setLifeTime(0.5+0.1*_scale);
+       
+    if (_automaticSetup) buildEffect();
+}
+
 FireEffect::FireEffect(const osg::Vec3& position, float scale, float intensity)
 {
     setDefaults();
@@ -37,7 +52,7 @@ FireEffect::FireEffect(const osg::Vec3& position, float scale, float intensity)
     _emitterDuration = 60.0;
     _defaultParticleTemplate.setLifeTime(0.5+0.1*_scale);
        
-    buildEffect();
+    if (_automaticSetup) buildEffect();
 }
 
 FireEffect::FireEffect(const FireEffect& copy, const osg::CopyOp& copyop):
@@ -100,6 +115,7 @@ void FireEffect::setUpEmitterAndProgram()
     if (!_emitter)
     {
         _emitter = new osgParticle::ModularEmitter;
+        _emitter->setNumParticlesToCreateMovementCompenstationRatio(1.5f);
         _emitter->setCounter(new osgParticle::RandomRateCounter);
         _emitter->setPlacer(new osgParticle::SectorPlacer);
         _emitter->setShooter(new osgParticle::RadialShooter);
