@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -22,6 +22,21 @@
 
 using namespace osgParticle;
 
+SmokeEffect::SmokeEffect(bool automaticSetup):
+    ParticleEffect(automaticSetup)
+{
+    setDefaults();
+    
+    _position.set(0.0f,0.0f,0.0f);
+    _scale = 1.0f;
+    _intensity = 1.0f;
+
+    _emitterDuration = 65.0;
+    _defaultParticleTemplate.setLifeTime(5.0*_scale);
+        
+    if (_automaticSetup) buildEffect();
+}
+
 SmokeEffect::SmokeEffect(const osg::Vec3& position, float scale, float intensity)
 {
     setDefaults();
@@ -33,7 +48,7 @@ SmokeEffect::SmokeEffect(const osg::Vec3& position, float scale, float intensity
     _emitterDuration = 65.0;
     _defaultParticleTemplate.setLifeTime(5.0*_scale);
         
-    buildEffect();
+    if (_automaticSetup) buildEffect();
 }
 
 SmokeEffect::SmokeEffect(const SmokeEffect& copy, const osg::CopyOp& copyop):
@@ -50,7 +65,7 @@ void SmokeEffect::setDefaults()
 
     // set up unit particle.
     _defaultParticleTemplate.setLifeTime(5.0*_scale);
-    _defaultParticleTemplate.setSizeRange(osgParticle::rangef(0.75f, 3.0f));
+    _defaultParticleTemplate.setSizeRange(osgParticle::rangef(0.75f, 2.0f));
     _defaultParticleTemplate.setAlphaRange(osgParticle::rangef(0.1f, 1.0f));
     _defaultParticleTemplate.setColorRange(osgParticle::rangev4(
                                             osg::Vec4(1, 1.0f, 1.0f, 1.0f), 
@@ -96,6 +111,7 @@ void SmokeEffect::setUpEmitterAndProgram()
     if (!_emitter)
     {
         _emitter = new osgParticle::ModularEmitter;
+        _emitter->setNumParticlesToCreateMovementCompenstationRatio(1.5f);
         _emitter->setCounter(new osgParticle::RandomRateCounter);
         _emitter->setPlacer(new osgParticle::SectorPlacer);
         _emitter->setShooter(new osgParticle::RadialShooter);
