@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -19,6 +19,14 @@
 
 FreeTypeFont::FreeTypeFont(const std::string& filename, FT_Face face):
     _filename(filename),
+    _buffer(0),
+    _face(face)
+{
+}
+
+FreeTypeFont::FreeTypeFont(FT_Byte* buffer, FT_Face face):
+    _filename(""),
+    _buffer(buffer),
     _face(face)
 {
 }
@@ -37,6 +45,13 @@ FreeTypeFont::~FreeTypeFont()
             // free the freetype font face itself
             FT_Done_Face(_face);
             _face = 0;
+
+            // release memory held for FT_Face to work
+            if (_buffer)
+            {
+                delete [] _buffer;
+                _buffer = 0;
+            }
         }
     }
 }
