@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -25,6 +25,7 @@ class FreeTypeFont : public osgText::Font::FontImplementation
 public:
 
     FreeTypeFont(const std::string& filename, FT_Face face);
+    FreeTypeFont(FT_Byte* buffer, FT_Face face);
 
     virtual ~FreeTypeFont();
 
@@ -41,6 +42,7 @@ public:
 protected:
 
     std::string     _filename;
+    FT_Byte*        _buffer;
     FT_Face         _face;
 
 };
