@@ -19,11 +19,12 @@
 #include "StateSet.h"
 #include "AlphaFunc.h"
 #include "BlendFunc.h"
+#include "Depth.h"
 #include "Material.h"
 #include "CullFace.h"
 #include "ClipPlane.h"
 #include "PolygonOffset.h"
-//#include "PolygonMode.h"
+#include "PolygonMode.h"
 #include "ShadeModel.h"
 #include "Point.h"
 #include "LineWidth.h"
@@ -31,6 +32,7 @@
 #include "Texture2D.h"
 #include "Texture3D.h"
 #include "TextureCubeMap.h"
+#include "TextureRectangle.h"
 #include "TexEnv.h"
 #include "TexEnvCombine.h"
 #include "TexGen.h"
@@ -41,9 +43,14 @@
 #include "ProxyNode.h"
 #include "FrontFace.h"
 #include "Program.h"
+#include "Viewport.h"
+#include "Scissor.h"
+#include "Image.h"
 
 #include "Group.h"
 #include "MatrixTransform.h"
+#include "CameraNode.h"
+#include "CameraView.h"
 #include "Geode.h"
 #include "LightSource.h"
 #include "TexGenNode.h"
@@ -53,6 +60,7 @@
 #include "LOD.h"
 #include "PagedLOD.h"
 #include "PositionAttitudeTransform.h"
+#include "AutoTransform.h"
 #include "DOFTransform.h"
 #include "Transform.h"
 #include "Switch.h"
@@ -78,7 +86,10 @@
 #include <osg/Notify>
 #include <osg/io_utils>
 #include <osgDB/ReadFile>
+#include <osgDB/FileNameUtils>
 
+#include <stdio.h>
+#include <sstream>
 
 using namespace ive;
 using namespace std;
@@ -194,10 +205,10 @@ unsigned int DataInputStream::readUInt(){
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readUInt(): Failed to read unsigned int value.");
 
-    if (_verboseOutput) std::cout<<"read/writeUInt() ["<<s<<"]"<<std::endl;
-    
     if (_byteswap) osg::swapBytes((char *)&s,INTSIZE) ;
     
+    if (_verboseOutput) std::cout<<"read/writeUInt() ["<<s<<"]"<<std::endl;
+    
     return s;
 }
 
@@ -215,9 +226,10 @@ int DataInputStream::readInt(){
     // if (_istream->rdstate() & _istream->failbit)
     //    throw Exception("DataInputStream::readInt(): Failed to read int value.");
 
-    if (_verboseOutput) std::cout<<"read/writeInt() ["<<i<<"]"<<std::endl;
     
     if (_byteswap) osg::swapBytes((char *)&i,INTSIZE) ;
+    
+    if (_verboseOutput) std::cout<<"read/writeInt() ["<<i<<"]"<<std::endl;
 
     return i;
 }
@@ -242,9 +254,9 @@ float DataInputStream::readFloat(){
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readFloat(): Failed to read float value.");
 
-    if (_verboseOutput) std::cout<<"read/writeFloat() ["<<f<<"]"<<std::endl;
-    
     if (_byteswap) osg::swapBytes((char *)&f,FLOATSIZE) ;
+
+    if (_verboseOutput) std::cout<<"read/writeFloat() ["<<f<<"]"<<std::endl;
     return f;
 }
 
@@ -254,9 +266,9 @@ long DataInputStream::readLong(){
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readLong(): Failed to read long value.");
 
-    if (_verboseOutput) std::cout<<"read/writeLong() ["<<l<<"]"<<std::endl;
-    
     if (_byteswap) osg::swapBytes((char *)&l,LONGSIZE) ;
+
+    if (_verboseOutput) std::cout<<"read/writeLong() ["<<l<<"]"<<std::endl;
     return l;
 }
 
@@ -266,6 +278,8 @@ unsigned long DataInputStream::readULong(){
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readULong(): Failed to read unsigned long value.");
 
+    if (_byteswap) osg::swapBytes((char *)&l,LONGSIZE) ;
+
     if (_verboseOutput) std::cout<<"read/writeULong() ["<<l<<"]"<<std::endl;
     
     return l;
@@ -277,9 +291,8 @@ double DataInputStream::readDouble(){
     if (_istream->rdstate() & _istream->failbit)
         throw Exception("DataInputStream::readDouble(): Failed to read double value.");
 
-    if (_verboseOutput) std::cout<<"read/writeDouble() ["<<d<<"]"<<std::endl;
-    
     if (_byteswap) osg::swapBytes((char *)&d,DOUBLESIZE) ;
+    if (_verboseOutput) std::cout<<"read/writeDouble() ["<<d<<"]"<<std::endl;
     return d;
 }
 
@@ -383,14 +396,14 @@ osg::Plane DataInputStream::readPlane(){
     return v;
 }
 
-osg::UByte4 DataInputStream::readUByte4(){
-    osg::UByte4 v;
+osg::Vec4ub DataInputStream::readVec4ub(){
+    osg::Vec4ub v;
     v.r()=readChar();
     v.g()=readChar();
     v.b()=readChar();
     v.a()=readChar();
 
-    if (_verboseOutput) std::cout<<"read/writeUByte4() ["<<v<<"]"<<std::endl;
+    if (_verboseOutput) std::cout<<"read/writeVec4ub() ["<<v<<"]"<<std::endl;
     
     return v;
 }
@@ -427,22 +440,28 @@ osg::Geometry::AttributeBinding DataInputStream::readBinding(){
 
 osg::Array* DataInputStream::readArray(){
     char c = readChar();
-
     switch((int)c){
         case 0: return readIntArray();
         case 1: return readUByteArray();
         case 2: return readUShortArray();
         case 3: return readUIntArray();
-        case 4: return readUByte4Array();
+        case 4: return readVec4ubArray();
         case 5: return readFloatArray();
         case 6:    return readVec2Array();
         case 7:    return readVec3Array();
         case 8:    return readVec4Array();
+        case 9:    return readVec2sArray();
+        case 10:   return readVec3sArray();
+        case 11:   return readVec4sArray();
+        case 12:   return readVec2bArray();        
+        case 13:   return readVec3bArray();        
+        case 14:   return readVec4bArray();        
         default: throw Exception("Unknown array type in DataInputStream::readArray()");
     }
 }
 
-osg::IntArray* DataInputStream::readIntArray(){
+osg::IntArray* DataInputStream::readIntArray()
+{
     int size = readInt();
     osg::IntArray* a = new osg::IntArray(size);
     
@@ -454,13 +473,14 @@ osg::IntArray* DataInputStream::readIntArray(){
     if (_verboseOutput) std::cout<<"read/writeIntArray() ["<<size<<"]"<<std::endl;  
 
     if (_byteswap) {
-       for (int  i = 0 ; i < size ; i++ ) osg::swapBytes((char *)&(a[i]),INTSIZE) ;
+       for (int  i = 0 ; i < size ; i++ ) osg::swapBytes((char *)&((*a)[i]),INTSIZE) ;
     }
        
     return a;
 }
 
-osg::UByteArray* DataInputStream::readUByteArray(){
+osg::UByteArray* DataInputStream::readUByteArray()
+{
     int size = readInt();
     osg::UByteArray* a = new osg::UByteArray(size);
 
@@ -474,7 +494,8 @@ osg::UByteArray* DataInputStream::readUByteArray(){
     return a;
 }
 
-osg::UShortArray* DataInputStream::readUShortArray(){
+osg::UShortArray* DataInputStream::readUShortArray()
+{
     int size = readInt();
     osg::UShortArray* a = new osg::UShortArray(size);
 
@@ -487,7 +508,8 @@ osg::UShortArray* DataInputStream::readUShortArray(){
     
     if (_byteswap)
     {
-       for (int i = 0 ; i < size ; i++ ) osg::swapBytes((char *)&(a[i]),SHORTSIZE) ;
+        for (int i = 0 ; i < size ; i++ ) 
+            osg::swapBytes((char *)&((*a)[i]),SHORTSIZE) ;
     }
     return a;
 }
@@ -506,22 +528,23 @@ osg::UIntArray* DataInputStream::readUIntArray()
     
     if (_byteswap)
     {
-       for (int i = 0 ; i < size ; i++ ) osg::swapBytes((char *)&(a[i]),INTSIZE) ;
+        for (int i = 0 ; i < size ; i++ )
+            osg::swapBytes((char *)&((*a)[i]),INTSIZE) ;
     }
     return a;
 }
 
-osg::UByte4Array* DataInputStream::readUByte4Array()
+osg::Vec4ubArray* DataInputStream::readVec4ubArray()
 {
     int size = readInt();
-    osg::UByte4Array* a = new osg::UByte4Array(size);
+    osg::Vec4ubArray* a = new osg::Vec4ubArray(size);
 
     _istream->read((char*)&((*a)[0]), INTSIZE*size);
 
     if (_istream->rdstate() & _istream->failbit)
-        throw Exception("DataInputStream::readUbyte4Array(): Failed to read UByte4 array.");
+        throw Exception("DataInputStream::readVec4ubArray(): Failed to read Vec4ub array.");
 
-    if (_verboseOutput) std::cout<<"read/writeUByte4Array() ["<<size<<"]"<<std::endl;
+    if (_verboseOutput) std::cout<<"read/writeVec4ubArray() ["<<size<<"]"<<std::endl;
     
     return a;
 }
@@ -541,7 +564,8 @@ osg::FloatArray* DataInputStream::readFloatArray()
     
     if (_byteswap)
     {
-       for (int i = 0 ; i < size ; i++ ) osg::swapBytes((char *)&(a[i]),FLOATSIZE) ;
+        for (int i = 0 ; i < size ; i++ )
+            osg::swapBytes((char *)&((*a)[i]),FLOATSIZE) ;
     }
     return a;
 }
@@ -585,7 +609,8 @@ osg::Vec3Array* DataInputStream::readVec3Array()
     if (_byteswap)
     {
        float *ptr = (float*)&((*a)[0]) ;
-       for (int i = 0 ; i < size*3 ; i++ ) {
+       for (int i = 0 ; i < size*3 ; i++ )
+       {
           osg::swapBytes((char *)&(ptr[i]),FLOATSIZE) ;
        }
     }
@@ -612,6 +637,124 @@ osg::Vec4Array* DataInputStream::readVec4Array(){
     return a;
 }
 
+osg::Vec2bArray* DataInputStream::readVec2bArray()
+{
+    int size = readInt();
+    osg::Vec2bArray* a = new osg::Vec2bArray(size);
+
+    _istream->read((char*)&((*a)[0]), CHARSIZE * 2 * size);
+
+    if (_istream->rdstate() & _istream->failbit)
+        throw Exception("DataInputStream::readVec2bArray(): Failed to read Vec2b array.");
+
+    if (_verboseOutput) std::cout<<"read/writeVec2bArray() ["<<size<<"]"<<std::endl;
+
+    return a;
+}
+
+osg::Vec3bArray* DataInputStream::readVec3bArray()
+{
+    int size = readInt();
+    osg::Vec3bArray* a = new osg::Vec3bArray(size);
+
+    _istream->read((char*)&((*a)[0]), CHARSIZE * 3 * size);
+
+    if (_istream->rdstate() & _istream->failbit)
+        throw Exception("DataInputStream::readVec3bArray(): Failed to read Vec3b array.");
+
+    if (_verboseOutput) std::cout<<"read/writeVec3bArray() ["<<size<<"]"<<std::endl;
+
+    return a;
+}
+
+osg::Vec4bArray* DataInputStream::readVec4bArray()
+{
+    int size = readInt();
+    osg::Vec4bArray* a = new osg::Vec4bArray(size);
+
+    _istream->read((char*)&((*a)[0]), CHARSIZE * 4 * size);
+
+    if (_istream->rdstate() & _istream->failbit)
+        throw Exception("DataInputStream::readVec4bArray(): Failed to read Vec4b array.");
+
+    if (_verboseOutput) std::cout<<"read/writeVec4bArray() ["<<size<<"]"<<std::endl;
+
+    return a;
+}
+
+osg::Vec2sArray* DataInputStream::readVec2sArray()
+{
+    int size = readInt();
+    osg::Vec2sArray* a = new osg::Vec2sArray(size);
+
+    _istream->read((char*)&((*a)[0]), SHORTSIZE * 2 * size);
+
+    if (_istream->rdstate() & _istream->failbit)
+        throw Exception("DataInputStream::readVec2sArray(): Failed to read Vec2s array.");
+
+    if (_verboseOutput) std::cout<<"read/writeVec2sArray() ["<<size<<"]"<<std::endl;
+
+    if (_byteswap)
+    {
+       short *ptr = (short*)&((*a)[0]) ;
+       for (int i = 0 ; i < size*2 ; i++ )
+       {
+          osg::swapBytes((char *)&(ptr[i]), SHORTSIZE) ;
+       }
+    }
+
+    return a;
+}
+
+osg::Vec3sArray* DataInputStream::readVec3sArray()
+{
+    int size = readInt();
+    osg::Vec3sArray* a = new osg::Vec3sArray(size);
+
+    _istream->read((char*)&((*a)[0]), SHORTSIZE * 3 * size);
+
+    if (_istream->rdstate() & _istream->failbit)
+        throw Exception("DataInputStream::readVec3sArray(): Failed to read Vec3s array.");
+
+    if (_verboseOutput) std::cout<<"read/writeVec3sArray() ["<<size<<"]"<<std::endl;
+
+
+    if (_byteswap)
+    {
+       short *ptr = (short*)&((*a)[0]) ;
+       for (int i = 0 ; i < size*3 ; i++ )
+       {
+          osg::swapBytes((char *)&(ptr[i]), SHORTSIZE) ;
+       }
+    }
+
+    return a;
+}
+
+osg::Vec4sArray* DataInputStream::readVec4sArray()
+{
+    int size = readInt();
+    osg::Vec4sArray* a = new osg::Vec4sArray(size);
+
+    _istream->read((char*)&((*a)[0]), SHORTSIZE * 4 * size);
+
+    if (_istream->rdstate() & _istream->failbit)
+        throw Exception("DataInputStream::readVec4sArray(): Failed to read Vec4s array.");
+
+    if (_verboseOutput) std::cout<<"read/writeVec4sArray() ["<<size<<"]"<<std::endl;
+
+    if (_byteswap)
+    {
+       short *ptr = (short*)&((*a)[0]) ;
+       for (int i = 0 ; i < size*4 ; i++ )
+       {
+          osg::swapBytes((char *)&(ptr[i]), SHORTSIZE) ;
+       }
+    }
+
+    return a;
+}
+
 osg::Matrixf DataInputStream::readMatrixf()
 {
     osg::Matrixf mat;
@@ -672,6 +815,70 @@ osg::Image* DataInputStream::readImage(std::string filename)
     return image;
 }
 
+osg::Image* DataInputStream::readImage(IncludeImageMode mode)
+{
+    switch(mode) {
+        case IMAGE_INCLUDE_DATA:
+            // Read image data from stream
+            if(readBool())
+            {
+                osg::Image* image = new osg::Image();
+                ((ive::Image*)image)->read(this);
+                return image;
+            }
+            break;
+        case IMAGE_REFERENCE_FILE:
+            // Only read image name from stream.
+            {
+                std::string filename = readString();
+                if(filename.compare("")!=0){
+                    return readImage(filename);
+                }
+            }
+            break;
+        case IMAGE_INCLUDE_FILE:
+        case IMAGE_COMPRESS_DATA:
+            // Read image file from stream
+            {
+                std::string filename = readString();
+                int size = readInt();
+                if(filename.compare("")!=0 && size > 0){
+
+                    //Read in file
+                    char *buffer = new char[size];
+                    readCharArray(buffer,size);
+
+                    //Get ReaderWriter from file extension
+                    std::string ext = osgDB::getFileExtension(filename);
+                    osgDB::ReaderWriter *reader = osgDB::Registry::instance()->getReaderWriterForExtension(ext);
+
+                    osgDB::ReaderWriter::ReadResult rr;
+                    if(reader) {
+                        //Convert data to istream
+                        std::stringstream inputStream;
+                        inputStream.write(buffer,size);
+
+                        //Attempt to read the image
+                        rr = reader->readImage(inputStream,_options.get());
+                    }
+
+                    //Delete buffer
+                    delete [] buffer;
+
+                    //Return result
+                    if(rr.validImage()) {
+                        return rr.takeImage();
+                    }
+                }
+            }
+            break;
+        default:
+            throw Exception("DataInputStream::readImage(): Invalid IncludeImageMode value.");
+            break;
+    }
+    return 0;
+}
+
 osg::StateSet* DataInputStream::readStateSet()
 {
     // Read statesets unique ID.
@@ -714,10 +921,23 @@ osg::StateAttribute* DataInputStream::readStateAttribute()
         attribute = new osg::AlphaFunc();
         ((ive::AlphaFunc*)(attribute))->read(this);
     }
-    else if(attributeID == IVEBLENDFUNC){
+    else if(attributeID == IVEBLENDFUNC ||
+            attributeID == IVEBLENDFUNCSEPARATE){
         attribute = new osg::BlendFunc();
         ((ive::BlendFunc*)(attribute))->read(this);
     }
+    else if(attributeID == IVEDEPTH){
+        attribute = new osg::Depth();
+        ((ive::Depth*)(attribute))->read(this);
+    }
+    else if(attributeID == IVEVIEWPORT){
+        attribute = new osg::Viewport();
+        ((ive::Viewport*)(attribute))->read(this);
+    }
+    else if(attributeID == IVESCISSOR){
+        attribute = new osg::Scissor();
+        ((ive::Scissor*)(attribute))->read(this);
+    }
     else if(attributeID == IVEMATERIAL){
         attribute = new osg::Material();
         ((ive::Material*)(attribute))->read(this);
@@ -734,10 +954,10 @@ osg::StateAttribute* DataInputStream::readStateAttribute()
         attribute = new osg::PolygonOffset();
         ((ive::PolygonOffset*)(attribute))->read(this);
     }
-/*    else if(attributeID == IVEPOLYGONMODE){
+    else if(attributeID == IVEPOLYGONMODE){
         attribute = new osg::PolygonMode();
         ((ive::PolygonMode*)(attribute))->read(this);
-    }*/
+    }
     else if(attributeID == IVESHADEMODEL){
         attribute = new osg::ShadeModel();
         ((ive::ShadeModel*)(attribute))->read(this);
@@ -766,6 +986,10 @@ osg::StateAttribute* DataInputStream::readStateAttribute()
         attribute = new osg::TextureCubeMap();
         ((ive::TextureCubeMap*)(attribute))->read(this);
     }
+    else if(attributeID == IVETEXTURERECTANGLE){
+        attribute = new osg::TextureRectangle();
+        ((ive::TextureRectangle*)(attribute))->read(this);
+    }
     else if(attributeID == IVETEXENV){
         attribute = new osg::TexEnv();
         ((ive::TexEnv*)(attribute))->read(this);
@@ -978,10 +1202,22 @@ osg::Node* DataInputStream::readNode()
         node = new osg::MatrixTransform();
         ((ive::MatrixTransform*)(node))->read(this);
     }
+    else if(nodeTypeID== IVECAMERANODE){
+        node = new osg::CameraNode();
+        ((ive::CameraNode*)(node))->read(this);
+    }
+    else if(nodeTypeID== IVECAMERAVIEW){
+        node = new osg::CameraView();
+        ((ive::CameraView*)(node))->read(this);
+    }
     else if(nodeTypeID== IVEPOSITIONATTITUDETRANSFORM){
         node = new osg::PositionAttitudeTransform();
         ((ive::PositionAttitudeTransform*)(node))->read(this);
     }
+    else if(nodeTypeID== IVEAUTOTRANSFORM){
+        node = new osg::AutoTransform();
+        ((ive::AutoTransform*)(node))->read(this);
+    }
     else if(nodeTypeID== IVEDOFTRANSFORM){
         node = new osgSim::DOFTransform();
         ((ive::DOFTransform*)(node))->read(this);
@@ -1074,3 +1310,5 @@ osg::Node* DataInputStream::readNode()
     
     return node;
 }
+
+
