@@ -22,9 +22,10 @@
 #include "BlendFunc.h"
 #include "Material.h"
 #include "CullFace.h"
+#include "Depth.h"
 #include "ClipPlane.h"
 #include "PolygonOffset.h"
-//#include "PolygonMode.h"
+#include "PolygonMode.h"
 #include "ShadeModel.h"
 #include "Point.h"
 #include "LineWidth.h"
@@ -32,6 +33,7 @@
 #include "Texture2D.h"
 #include "Texture3D.h"
 #include "TextureCubeMap.h"
+#include "TextureRectangle.h"
 #include "TexEnv.h"
 #include "TexEnvCombine.h"
 #include "TexGen.h"
@@ -44,9 +46,14 @@
 #include "Program.h"
 #include "Uniform.h"
 #include "Shader.h"
+#include "Viewport.h"
+#include "Scissor.h"
+#include "Image.h"
 
 #include "Group.h"
 #include "MatrixTransform.h"
+#include "CameraNode.h"
+#include "CameraView.h"
 #include "Geode.h"
 #include "LightSource.h"
 #include "TexGenNode.h"
@@ -56,6 +63,7 @@
 #include "LOD.h"
 #include "PagedLOD.h"
 #include "PositionAttitudeTransform.h"
+#include "AutoTransform.h"
 #include "DOFTransform.h"
 #include "Transform.h"
 #include "Switch.h"
@@ -78,6 +86,10 @@
 
 #include <osg/Notify>
 #include <osg/io_utils>
+#include <osgDB/FileUtils>
+
+#include <fstream>
+#include <sstream>
 
 using namespace ive;
 
@@ -88,8 +100,14 @@ void DataOutputStream::setOptions(const osgDB::ReaderWriter::Options* options)
 
     if (_options.get())
     {
-        setIncludeImageData(_options->getOptionString().find("noTexturesInIVEFile")==std::string::npos);
-        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setIncludeImageData()=" << getIncludeImageData() << std::endl;
+        if(_options->getOptionString().find("noTexturesInIVEFile")!=std::string::npos) {
+            setIncludeImageMode(IMAGE_REFERENCE_FILE);
+        } else if(_options->getOptionString().find("includeImageFileInIVEFile")!=std::string::npos) {
+            setIncludeImageMode(IMAGE_INCLUDE_FILE);
+        } else if(_options->getOptionString().find("compressImageData")!=std::string::npos) {
+            setIncludeImageMode(IMAGE_COMPRESS_DATA);
+        }
+        osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setIncludeImageMode()=" << getIncludeImageMode() << std::endl;
 
         setIncludeExternalReferences(_options->getOptionString().find("inlineExternalReferencesInIVEFile")!=std::string::npos);
         osg::notify(osg::DEBUG_INFO) << "ive::DataOutpouStream.setIncludeExternalReferences()=" << getIncludeExternalReferences() << std::endl;
@@ -106,7 +124,7 @@ DataOutputStream::DataOutputStream(std::ostream * ostream)
 {
     _verboseOutput = false;
 
-    _includeImageData= true;
+    _includeImageMode = IMAGE_INCLUDE_DATA;
 
     _includeExternalReferences     = false;
     _writeExternalReferenceFiles   = false;
@@ -148,6 +166,12 @@ void DataOutputStream::writeUShort(unsigned short s){
     if (_verboseOutput) std::cout<<"read/writeUShort() ["<<s<<"]"<<std::endl;
 }
 
+void DataOutputStream::writeShort(short s){
+    _ostream->write((char*)&s, SHORTSIZE);
+
+    if (_verboseOutput) std::cout<<"read/writeShort() ["<<s<<"]"<<std::endl;
+}
+
 void DataOutputStream::writeUInt(unsigned int s){
     _ostream->write((char*)&s, INTSIZE);
     
@@ -255,13 +279,37 @@ void DataOutputStream::writePlane(const osg::Plane& v)
     if (_verboseOutput) std::cout<<"read/writePlane() ["<<v<<"]"<<std::endl;
 }
 
-void DataOutputStream::writeUByte4(const osg::UByte4& v){
+void DataOutputStream::writeVec4ub(const osg::Vec4ub& v){
     writeChar(v.r());
     writeChar(v.g());
     writeChar(v.b());
     writeChar(v.a());
     
-    if (_verboseOutput) std::cout<<"read/writeUByte4() ["<<v<<"]"<<std::endl;
+    if (_verboseOutput) std::cout<<"read/writeVec4ub() ["<<v<<"]"<<std::endl;
+}
+
+void DataOutputStream::writeVec2b(const osg::Vec2b& v){
+    writeChar(v.r());
+    writeChar(v.g());    
+
+    if (_verboseOutput) std::cout<<"read/writeVec2b() ["<<v<<"]"<<std::endl;
+}
+
+void DataOutputStream::writeVec3b(const osg::Vec3b& v){
+    writeChar(v.r());
+    writeChar(v.g());
+    writeChar(v.b());
+
+    if (_verboseOutput) std::cout<<"read/writeVec3b() ["<<v<<"]"<<std::endl;
+}
+
+void DataOutputStream::writeVec4b(const osg::Vec4b& v){
+    writeChar(v.r());
+    writeChar(v.g());
+    writeChar(v.b());
+    writeChar(v.a());
+
+    if (_verboseOutput) std::cout<<"read/writeVec4b() ["<<v<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeQuat(const osg::Quat& q){
@@ -304,9 +352,9 @@ void DataOutputStream::writeArray(const osg::Array* a){
             writeChar((char)3);
             writeUIntArray(static_cast<const osg::UIntArray*>(a));
             break;
-        case osg::Array::UByte4ArrayType:
+        case osg::Array::Vec4ubArrayType:
             writeChar((char)4);
-            writeUByte4Array(static_cast<const osg::UByte4Array*>(a));
+            writeVec4ubArray(static_cast<const osg::Vec4ubArray*>(a));
             break;
         case osg::Array::FloatArrayType:
             writeChar((char)5);
@@ -324,6 +372,30 @@ void DataOutputStream::writeArray(const osg::Array* a){
             writeChar((char)8);
             writeVec4Array(static_cast<const osg::Vec4Array*>(a));
             break;
+         case osg::Array::Vec2sArrayType:
+             writeChar((char)9);
+             writeVec2sArray(static_cast<const osg::Vec2sArray*>(a));
+             break;
+         case osg::Array::Vec3sArrayType:
+             writeChar((char)10);
+             writeVec3sArray(static_cast<const osg::Vec3sArray*>(a));
+             break;
+         case osg::Array::Vec4sArrayType:
+             writeChar((char)11);
+             writeVec4sArray(static_cast<const osg::Vec4sArray*>(a));
+             break;
+         case osg::Array::Vec2bArrayType:
+             writeChar((char)12);
+             writeVec2bArray(static_cast<const osg::Vec2bArray*>(a));
+             break;
+         case osg::Array::Vec3bArrayType:
+             writeChar((char)13);
+             writeVec3bArray(static_cast<const osg::Vec3bArray*>(a));
+             break;
+         case osg::Array::Vec4bArrayType:
+             writeChar((char)14);
+             writeVec4bArray(static_cast<const osg::Vec4bArray*>(a));
+             break;            
         default: throw Exception("Unknown array type in DataOutputStream::writeArray()");
     }
 }
@@ -373,15 +445,15 @@ void DataOutputStream::writeUIntArray(const osg::UIntArray* a)
     if (_verboseOutput) std::cout<<"read/writeUIntArray() ["<<size<<"]"<<std::endl;
 }
 
-void DataOutputStream::writeUByte4Array(const osg::UByte4Array* a)
+void DataOutputStream::writeVec4ubArray(const osg::Vec4ubArray* a)
 {
     int size = a->getNumElements(); 
     writeInt(size);
     for(int i =0; i<size ;i++){
-        writeUByte4((*a)[i]);
+        writeVec4ub((*a)[i]);
     }
     
-    if (_verboseOutput) std::cout<<"read/writeUByte4Array() ["<<size<<"]"<<std::endl;
+    if (_verboseOutput) std::cout<<"read/writeVec4ubArray() ["<<size<<"]"<<std::endl;
 }
 
 void DataOutputStream::writeFloatArray(const osg::FloatArray* a)
@@ -429,6 +501,78 @@ void DataOutputStream::writeVec4Array(const osg::Vec4Array* a)
     if (_verboseOutput) std::cout<<"read/writeVec4Array() ["<<size<<"]"<<std::endl;
 }
 
+void DataOutputStream::writeVec2sArray(const osg::Vec2sArray* a)
+{
+    int size = a->getNumElements(); 
+    writeInt(size);
+    for(int i =0; i<size ;i++){
+        writeShort((*a)[i].x());
+        writeShort((*a)[i].y());
+    }
+
+    if (_verboseOutput) std::cout<<"read/writeVec2sArray() ["<<size<<"]"<<std::endl;
+}
+
+void DataOutputStream::writeVec3sArray(const osg::Vec3sArray* a)
+{
+    int size = a->getNumElements(); 
+    writeInt(size);
+    for(int i =0; i<size ;i++){
+        writeShort((*a)[i].x());
+        writeShort((*a)[i].y());
+        writeShort((*a)[i].z());
+    }
+
+    if (_verboseOutput) std::cout<<"read/writeVec3sArray() ["<<size<<"]"<<std::endl;
+}
+
+void DataOutputStream::writeVec4sArray(const osg::Vec4sArray* a)
+{
+    int size = a->getNumElements(); 
+    writeInt(size);
+    for(int i =0; i<size ;i++){
+        writeShort((*a)[i].x());
+        writeShort((*a)[i].y());
+        writeShort((*a)[i].z());
+        writeShort((*a)[i].w());
+    }
+
+    if (_verboseOutput) std::cout<<"read/writeVec4sArray() ["<<size<<"]"<<std::endl;
+}
+
+void DataOutputStream::writeVec2bArray(const osg::Vec2bArray* a)
+{
+    int size = a->getNumElements(); 
+    writeInt(size);
+    for(int i =0; i<size ;i++){
+        writeVec2b((*a)[i]);
+    }
+
+    if (_verboseOutput) std::cout<<"read/writeVec2bArray() ["<<size<<"]"<<std::endl;
+}
+
+void DataOutputStream::writeVec3bArray(const osg::Vec3bArray* a)
+{
+    int size = a->getNumElements(); 
+    writeInt(size);
+    for(int i =0; i<size ;i++){
+        writeVec3b((*a)[i]);
+    }
+
+    if (_verboseOutput) std::cout<<"read/writeVec3bArray() ["<<size<<"]"<<std::endl;
+}
+
+void DataOutputStream::writeVec4bArray(const osg::Vec4bArray* a)
+{
+    int size = a->getNumElements(); 
+    writeInt(size);
+    for(int i =0; i<size ;i++){
+        writeVec4b((*a)[i]);
+    }
+
+    if (_verboseOutput) std::cout<<"read/writeVec4bArray() ["<<size<<"]"<<std::endl;
+}
+
 void DataOutputStream::writeMatrixf(const osg::Matrixf& mat)
 {
     for(int r=0;r<4;r++)
@@ -512,6 +656,15 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
         else if(dynamic_cast<const osg::BlendFunc*>(attribute)){
             ((ive::BlendFunc*)(attribute))->write(this);
         }
+        else if(dynamic_cast<const osg::Depth*>(attribute)){
+            ((ive::Depth*)(attribute))->write(this);
+        }
+        else if(dynamic_cast<const osg::Viewport*>(attribute)){
+            ((ive::Viewport*)(attribute))->write(this);
+        }
+        else if(dynamic_cast<const osg::Scissor*>(attribute)){
+            ((ive::Scissor*)(attribute))->write(this);
+        }
         // This is a Material
         else if(dynamic_cast<const osg::Material*>(attribute)){
             ((ive::Material*)(attribute))->write(this);
@@ -528,10 +681,10 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
         else if(dynamic_cast<const osg::PolygonOffset*>(attribute)){
             ((ive::PolygonOffset*)(attribute))->write(this);
         }
-/*        // This is a PolygonMode
+        // This is a PolygonMode
         else if(dynamic_cast<const osg::PolygonMode*>(attribute)){
             ((ive::PolygonMode*)(attribute))->write(this);
-        }*/
+        }
         else if(dynamic_cast<const osg::ShadeModel*>(attribute)){
             ((ive::ShadeModel*)(attribute))->write(this);
         }
@@ -560,6 +713,10 @@ void DataOutputStream::writeStateAttribute(const osg::StateAttribute* attribute)
         else if(dynamic_cast<const osg::TextureCubeMap*>(attribute)){
             ((ive::TextureCubeMap*)(attribute))->write(this);
         }
+        // This is a TextureRectangle
+        else if(dynamic_cast<const osg::TextureRectangle*>(attribute)){
+            ((ive::TextureRectangle*)(attribute))->write(this);
+        }
         // This is a TexEnv
         else if(dynamic_cast<const osg::TexEnv*>(attribute)){
             ((ive::TexEnv*)(attribute))->write(this);
@@ -765,9 +922,18 @@ void DataOutputStream::writeNode(const osg::Node* node)
         if(dynamic_cast<const osg::MatrixTransform*>(node)){
             ((ive::MatrixTransform*)(node))->write(this);
         }
+        else if(dynamic_cast<const osg::CameraNode*>(node)){
+            ((ive::CameraNode*)(node))->write(this);
+        }
+        else if(dynamic_cast<const osg::CameraView*>(node)){
+            ((ive::CameraView*)(node))->write(this);
+        }
         else if(dynamic_cast<const osg::PositionAttitudeTransform*>(node)){
             ((ive::PositionAttitudeTransform*)(node))->write(this);
         }
+        else if(dynamic_cast<const osg::AutoTransform*>(node)){
+            ((ive::AutoTransform*)(node))->write(this);
+        }
         else if(dynamic_cast<const osgSim::DOFTransform*>(node)){
             ((ive::DOFTransform*)(node))->write(this);
         }
@@ -834,3 +1000,110 @@ void DataOutputStream::writeNode(const osg::Node* node)
         if (_verboseOutput) std::cout<<"read/writeNode() ["<<id<<"]"<<std::endl;
     }
 }
+
+void DataOutputStream::writeImage(IncludeImageMode mode, osg::Image *image)
+{
+    switch(mode) {
+        case IMAGE_INCLUDE_DATA:
+            // Include image data in stream
+            writeBool(image!=0);
+            if(image)
+                ((ive::Image*)image)->write(this);
+            break;
+        case IMAGE_REFERENCE_FILE:
+            // Only include image name in stream
+            if (image && !(image->getFileName().empty())){
+                writeString(image->getFileName());
+            }
+            else{ 
+                writeString("");
+            }    
+            break;
+        case IMAGE_INCLUDE_FILE:
+            // Include image file in stream
+            if(image && !(image->getFileName().empty())) {
+                std::string fullPath = osgDB::findDataFile(image->getFileName(),_options.get());
+                std::ifstream infile(fullPath.c_str(), std::ios::in | std::ios::binary);
+                if(infile) {
+
+                    //Write filename
+                    writeString(image->getFileName());
+
+                    //Get size of file
+                    infile.seekg(0,std::ios::end);
+                    int size = infile.tellg();
+                    infile.seekg(0,std::ios::beg);
+
+                    //Write file size
+                    writeInt(size);
+
+                    //Read file data
+                    char *buffer = new char[size];
+                    infile.read(buffer,size);
+
+                    //Write file data
+                    writeCharArray(buffer,size);
+
+                    //Delete buffer
+                    delete [] buffer;
+
+                    //Close file
+                    infile.close();
+                   
+                } else {
+                    writeString("");
+                    writeInt(0);
+                }
+            }
+            else{ 
+                writeString("");
+                writeInt(0);
+            }    
+            break;
+        case IMAGE_COMPRESS_DATA:
+            if(image)
+            {
+                //Get ReaderWriter for jpeg images
+                
+                std::string extension = "png";
+                if (image->getPixelFormat()==GL_RGB) extension = "jpg";
+                
+                osgDB::ReaderWriter* writer = osgDB::Registry::instance()->getReaderWriterForExtension(extension);
+
+                if(writer)
+                {
+                    //Attempt to write the image to an output stream.
+                    //The reason this isn't performed directly on the internal _ostream
+                    //is because the writer might perform seek operations which could
+                    //corrupt the output stream.
+                    std::stringstream outputStream;
+                    osgDB::ReaderWriter::WriteResult wr;
+                    wr = writer->writeImage(*image,outputStream,_options.get());
+
+                    if(wr.success()) {
+
+                        //Write file format. Do this for two reasons:
+                        // 1 - Same code can be used to read in as with IMAGE_INCLUDE_FILE mode
+                        // 2 - Maybe in future version user can specify which format to use
+                        writeString(std::string(".")+extension); //Need to add dot so osgDB::getFileExtension will work
+                        
+                        //Write size of stream
+                        int size = outputStream.tellp();
+                        writeInt(size);
+                       
+                        //Write stream
+                        writeCharArray(outputStream.str().c_str(),size);
+
+                        return;
+                    }
+                }
+            }
+            //Image compression failed, write blank data
+            writeString("");
+            writeInt(0);
+            break;
+        default:
+            throw Exception("DataOutputStream::writeImage(): Invalid IncludeImageMode value.");
+            break;
+    }
+}
