@@ -17,7 +17,7 @@
  * along with  this program;  if not, write to the  Free Software Foundation,
  * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id$
+ * $Id: chunk.cpp,v 1.6 2006/08/21 15:07:31 robert Exp $
  */
 #define LIB3DS_EXPORT
 #include "chunk.h"
@@ -61,7 +61,7 @@ static void
 lib3ds_chunk_debug_dump(Lib3dsChunk *c)
 {
   if (enable_dump) {
-    printf("%s%s (0x%X) size=%lu\n",
+    printf("%s%s (0x%X) size=%u\n",
       lib3ds_chunk_level,
       lib3ds_chunk_name(c->chunk),
       c->chunk,
