@@ -17,7 +17,7 @@
  * along with  this program;  if not, write to the  Free Software Foundation,
  * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id$
+ * $Id: mesh.cpp,v 1.6 2006/08/21 15:07:31 robert Exp $
  */
 #define LIB3DS_EXPORT
 #include "mesh.h"
@@ -495,7 +495,7 @@ lib3ds_mesh_dump(Lib3dsMesh *mesh)
   Lib3dsVector p;
 
   ASSERT(mesh);
-  printf("  %s vertices=%ld faces=%ld\n",
+  printf("  %s vertices=%d faces=%d\n",
     mesh->name,
     mesh->points,
     mesh->faces
