@@ -20,7 +20,7 @@
  * along with  this program;  if not, write to the  Free Software Foundation,
  * Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id$
+ * $Id: types.h,v 1.4 2006/08/21 15:07:31 robert Exp $
  */
 #ifdef __cplusplus
 extern "C" {
@@ -45,11 +45,11 @@ extern "C" {
 
 typedef int Lib3dsBool;
 typedef unsigned char Lib3dsByte;
-typedef unsigned short int Lib3dsWord;
-typedef unsigned long Lib3dsDword;
+typedef unsigned short Lib3dsWord;
+typedef unsigned int Lib3dsDword;
 typedef signed char Lib3dsIntb;
-typedef signed short int Lib3dsIntw;
-typedef signed long Lib3dsIntd;
+typedef signed short Lib3dsIntw;
+typedef signed int Lib3dsIntd;
 typedef float Lib3dsFloat;
 typedef double Lib3dsDouble;
 
