@@ -8,6 +8,8 @@
 #include <osgDB/FileUtils>
 #include <osgDB/FileNameUtils>
 
+#include <sstream>
+
 using namespace osg;
 
 extern "C"
@@ -30,22 +32,89 @@ typedef struct
     unsigned int Alpha;
 } pngInfo;
 
+void png_read_istream(png_structp png_ptr, png_bytep data, png_size_t length)
+{
+    std::istream *stream = (std::istream*)png_get_io_ptr(png_ptr); //Get pointer to istream
+    stream->read((char*)data,length); //Read requested amount of data
+}
+
+void png_write_ostream(png_structp png_ptr, png_bytep data, png_size_t length)
+{
+    std::ostream *stream = (std::ostream*)png_get_io_ptr(png_ptr); //Get pointer to ostream
+    stream->write((char*)data,length); //Write requested amount of data
+}
+
+void png_flush_ostream(png_structp png_ptr)
+{
+    std::ostream *stream = (std::ostream*)png_get_io_ptr(png_ptr); //Get pointer to ostream
+    stream->flush();
+}
+
 class ReaderWriterPNG : public osgDB::ReaderWriter
 {
     public:
         virtual const char* className() const { return "PNG Image Reader/Writer"; }
         virtual bool acceptsExtension(const std::string& extension) const { return osgDB::equalCaseInsensitive(extension,"png"); }
 
-        virtual ReadResult readImage(const std::string& file, const osgDB::ReaderWriter::Options* options) const
+        WriteResult::WriteStatus writePngStream(std::ostream& fout, const osg::Image& img, int compression_level) const
         {
-            std::string ext = osgDB::getLowerCaseFileExtension(file);
-            if (!acceptsExtension(ext)) return ReadResult::FILE_NOT_HANDLED;
+            png_structp png = NULL;
+            png_infop   info = NULL;
+            int color;
+            png_bytep *rows = NULL;
 
-            std::string fileName = osgDB::findDataFile( file, options );
-            if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
+            //Create write structure
+            png = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+            if(!png) return WriteResult::ERROR_IN_WRITING_FILE;
 
+            //Create infr structure
+            info = png_create_info_struct(png);
+            if(!info) return WriteResult::ERROR_IN_WRITING_FILE;
+
+            //Set custom write function so it will write to ostream
+            png_set_write_fn(png,&fout,png_write_ostream,png_flush_ostream);
+
+            //Set compression level
+            png_set_compression_level(png, compression_level);
+
+            switch(img.getPixelFormat()) {
+                case(GL_LUMINANCE): color = PNG_COLOR_TYPE_GRAY; break;
+                case(GL_ALPHA): color = PNG_COLOR_TYPE_GRAY; break; //Couldn't find a color type for pure alpha, using gray instead
+                case(GL_LUMINANCE_ALPHA): color = PNG_COLOR_TYPE_GRAY_ALPHA ; break;
+                case(GL_RGB): color = PNG_COLOR_TYPE_RGB; break;
+                case(GL_RGBA): color = PNG_COLOR_TYPE_RGB_ALPHA; break;
+                default: return WriteResult::ERROR_IN_WRITING_FILE; break;                
+            }
+
+            //Create row data
+            rows = new png_bytep[img.t()];
+            for(int i = 0; i < img.t(); ++i) {
+                rows[i] = (png_bytep)img.data(0,img.t() - i - 1);
+            }
+
+            //Write header info
+            png_set_IHDR(png, info, img.s(), img.t(),
+                        8, color, PNG_INTERLACE_NONE,
+                        PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
+
+            png_write_info(png, info);
+
+            //Write data
+            png_write_image(png, rows);
+
+            //End write
+            png_write_end(png, NULL);
+
+            //Cleanup
+            png_destroy_write_struct(&png,&info);
+            delete [] rows;
+
+            return WriteResult::FILE_SAVED;
+        }
+
+        ReadResult readPNGStream(std::istream& fin) const
+        {
             int trans = PNG_ALPHA;
-            FILE *fp = NULL;
             pngInfo pInfo;
             pngInfo *pinfo = &pInfo;
 
@@ -65,9 +134,9 @@ class ReaderWriterPNG : public osgDB::ReaderWriter
             info = png_create_info_struct(png);
             endinfo = png_create_info_struct(png);
 
-            fp = fopen(fileName.c_str(), "rb");
-            if (fp && fread(header, 1, 8, fp) && png_check_sig(header, 8))
-                png_init_io(png, fp);
+            fin.read((char*)header,8);
+            if (fin.gcount() == 8 && png_check_sig(header, 8))
+                png_set_read_fn(png,&fin,png_read_istream); //Use custom read function that will get data from istream
             else
             {
                 png_destroy_read_struct(&png, &info, &endinfo);
@@ -153,7 +222,7 @@ class ReaderWriterPNG : public osgDB::ReaderWriter
             png_read_end(png, endinfo);
 
             GLenum pixelFormat = 0;
-            GLenum dataType = depth==8?GL_UNSIGNED_BYTE:GL_UNSIGNED_SHORT;
+            GLenum dataType = depth<=8?GL_UNSIGNED_BYTE:GL_UNSIGNED_SHORT;
             switch(color)
             {
               case(PNG_SOLID): pixelFormat = GL_LUMINANCE; break;
@@ -180,15 +249,11 @@ class ReaderWriterPNG : public osgDB::ReaderWriter
 
             //    delete [] data;
 
-            if (fp)
-                fclose(fp);
-
             if (pixelFormat==0) 
                 return ReadResult::FILE_NOT_HANDLED;
 
             osg::Image* pOsgImage = new osg::Image();
 
-            pOsgImage->setFileName(fileName.c_str());
             pOsgImage->setImage(width, height, 1,
                 internalFormat,
                 pixelFormat, 
@@ -198,6 +263,60 @@ class ReaderWriterPNG : public osgDB::ReaderWriter
 
             return pOsgImage;
         }
+
+        int getCompressionLevel(const osgDB::ReaderWriter::Options *options) const
+        {
+            if(options) {
+                std::istringstream iss(options->getOptionString());
+                std::string opt;
+                while (iss >> opt) {
+                    if(opt=="PNG_COMPRESSION") {
+                        int level;
+                        iss >> level;
+                        return level;
+                    }
+                }
+            }
+
+            return Z_DEFAULT_COMPRESSION;
+        }
+
+        virtual ReadResult readImage(std::istream& fin,const Options* =NULL) const
+        {
+            return readPNGStream(fin);
+        }
+
+        virtual ReadResult readImage(const std::string& file, const osgDB::ReaderWriter::Options* options) const
+        {
+            std::string ext = osgDB::getLowerCaseFileExtension(file);
+            if (!acceptsExtension(ext)) return ReadResult::FILE_NOT_HANDLED;
+
+            std::string fileName = osgDB::findDataFile( file, options );
+            if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
+
+            std::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
+            if(!istream) return ReadResult::FILE_NOT_HANDLED;
+            ReadResult rr = readPNGStream(istream);
+            if(rr.validImage()) rr.getImage()->setFileName(file);
+            return rr;
+        }
+
+        virtual WriteResult writeImage(const osg::Image& img,std::ostream& fout,const osgDB::ReaderWriter::Options *options) const
+        {
+            WriteResult::WriteStatus ws = writePngStream(fout,img,getCompressionLevel(options));
+            return ws;
+        }
+
+        virtual WriteResult writeImage(const osg::Image &img,const std::string& fileName, const osgDB::ReaderWriter::Options *options) const
+        {
+            std::string ext = osgDB::getFileExtension(fileName);
+            if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
+
+            std::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
+            if(!fout) return WriteResult::ERROR_IN_WRITING_FILE;
+
+            return writeImage(img,fout,options);
+        }
 };
 
 // now register with Registry to instantiate the above
