@@ -8,7 +8,7 @@
  */
 
 #include <Carbon/Carbon.h>
-#include <Quicktime/Quicktime.h>
+#include <QuickTime/QuickTime.h>
 
 namespace osgQuicktime {
 
@@ -17,4 +17,4 @@ namespace osgQuicktime {
 	OSStatus MakeFSSpecFromPath(const char* path, FSSpec* spec);
 	OSStatus MakeMovieFromPath(const char* path, Movie* movie);
 
-}
\ No newline at end of file
+}
