@@ -40,7 +40,6 @@
  *
  **********************************************************************/
 
-
 #include <string.h>
 #include <stdarg.h>
 #include <assert.h>
@@ -53,6 +52,160 @@
 #define ERR_UNSUPPORTED 4
 #define ERR_TIFFLIB     5
 
+/* Functions to read TIFF image from memory
+ *
+ */
+
+tsize_t libtiffStreamReadProc(thandle_t fd, tdata_t buf, tsize_t size)
+{
+    std::istream *fin = (std::istream*)fd;
+
+    fin->read((char*)buf,size);
+
+    if(fin->bad())
+        return -1;
+
+    if(fin->gcount() < size)
+        return 0;
+
+    return size;
+}
+
+tsize_t libtiffStreamWriteProc(thandle_t, tdata_t, tsize_t)
+{
+    return 0;
+}
+
+toff_t libtiffStreamSeekProc(thandle_t fd, toff_t off, int i)
+{
+    std::istream *fin = (std::istream*)fd;
+
+    toff_t ret;
+    switch(i)
+    {
+        case SEEK_SET:
+            fin->seekg(off,std::ios::beg);
+            ret = fin->tellg();
+            if(fin->bad())
+                ret = 0;
+            break;
+
+        case SEEK_CUR:
+            fin->seekg(off,std::ios::cur);
+            ret = fin->tellg();
+            if(fin->bad())
+                ret = 0;
+            break;
+
+        case SEEK_END:
+            fin->seekg(off,std::ios::end);
+            ret = fin->tellg();
+            if(fin->bad())
+                ret = 0;
+            break;
+        default:
+            ret = 0;
+            break;
+    }
+    return ret;
+}
+
+int libtiffStreamCloseProc(thandle_t)
+{
+    return 0;
+}
+
+toff_t libtiffStreamSizeProc(thandle_t fd)
+{
+    std::istream *fin = (std::istream*)fd;
+
+    std::streampos curPos = fin->tellg();
+
+    fin->seekg(0, std::ios::end);
+    toff_t size = fin->tellg();
+    fin->seekg(curPos, std::ios::beg);
+
+    return size;
+}
+
+int libtiffStreamMapProc(thandle_t, tdata_t*, toff_t*)
+{
+    return 0;
+}
+
+void libtiffStreamUnmapProc(thandle_t, tdata_t, toff_t)
+{
+}
+
+/* Functions to write TIFF image from memory
+ *
+ */
+
+tsize_t libtiffOStreamReadProc(thandle_t, tdata_t, tsize_t)
+{
+    return 0;
+}
+
+tsize_t libtiffOStreamWriteProc(thandle_t fd, tdata_t buf, tsize_t size)
+{
+    std::ostream *fout = (std::ostream*)fd;
+
+    fout->write((const char*)buf,size);
+
+    if(fout->bad()) {
+        return -1;
+    }
+
+    return size;
+}
+
+toff_t libtiffOStreamSizeProc(thandle_t fd)
+{
+    std::ostream *fout = (std::ostream*)fd;
+
+    std::streampos curPos = fout->tellp();
+
+    fout->seekp(0, std::ios::end);
+    toff_t size = fout->tellp();
+    fout->seekp(curPos, std::ios::beg);
+
+    return size;
+}
+
+toff_t libtiffOStreamSeekProc(thandle_t fd, toff_t off, int i)
+{
+    std::ostream *fout = (std::ostream*)fd;
+
+    toff_t ret;
+    switch(i)
+    {
+        case SEEK_SET:
+            fout->seekp(off,std::ios::beg);
+            ret = fout->tellp();
+            if(fout->bad())
+                ret = 0;
+            break;
+
+        case SEEK_CUR:
+            fout->seekp(off,std::ios::cur);
+            ret = fout->tellp();
+            if(fout->bad())
+                ret = 0;
+            break;
+
+        case SEEK_END:
+            fout->seekp(off,std::ios::end);
+            ret = fout->tellp();
+            if(fout->bad())
+                ret = 0;
+            break;
+        default:
+            ret = 0;
+            break;
+    }
+    return ret;
+}
+
 static int tifferror = ERR_NO_ERROR;
 
 int
@@ -77,6 +230,10 @@ simage_tiff_error(char * buffer, int buflen)
 }
 
 
+
+
+
+
 static void
 tiff_error(const char*, const char*, va_list)
 {
@@ -203,7 +360,7 @@ int headerlen)
 #define pack(a,b)   ((a)<<8 | (b))
 
 unsigned char *
-simage_tiff_load(const char *filename,
+simage_tiff_load(std::istream& fin,
 int *width_ret,
 int *height_ret,
 int *numComponents_ret)
@@ -229,7 +386,15 @@ int *numComponents_ret)
     TIFFSetErrorHandler(tiff_error);
     TIFFSetWarningHandler(tiff_warn);
 
-    in = TIFFOpen(filename, "r");
+    in = TIFFClientOpen("inputstream", "r", (thandle_t)&fin,
+            libtiffStreamReadProc, //Custom read function
+            libtiffStreamWriteProc, //Custom write function
+            libtiffStreamSeekProc, //Custom seek function
+            libtiffStreamCloseProc, //Custom close function
+            libtiffStreamSizeProc, //Custom size function
+            libtiffStreamMapProc, //Custom map function
+            libtiffStreamUnmapProc); //Custom unmap function
+
     if (in == NULL)
     {
         tifferror = ERR_OPEN;
@@ -447,20 +612,14 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
             return false;
         }
 
-        virtual ReadResult readImage(const std::string& file, const osgDB::ReaderWriter::Options* options) const
+        ReadResult readTIFStream(std::istream& fin) const
         {
-            std::string ext = osgDB::getLowerCaseFileExtension(file);
-            if (!acceptsExtension(ext)) return ReadResult::FILE_NOT_HANDLED;
-
-            std::string fileName = osgDB::findDataFile( file, options );
-            if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
-
             unsigned char *imageData = NULL;
             int width_ret;
             int height_ret;
             int numComponents_ret;
 
-            imageData = simage_tiff_load(fileName.c_str(),&width_ret,&height_ret,&numComponents_ret);
+            imageData = simage_tiff_load(fin,&width_ret,&height_ret,&numComponents_ret);
 
             if (imageData==NULL) 
             {
@@ -485,7 +644,6 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
             unsigned int dataType = GL_UNSIGNED_BYTE;
 
             osg::Image* pOsgImage = new osg::Image;
-            pOsgImage->setFileName(fileName.c_str());
             pOsgImage->setImage(s,t,r,
                 internalFormat,
                 pixelFormat,
@@ -494,7 +652,112 @@ class ReaderWriterTIFF : public osgDB::ReaderWriter
                 osg::Image::USE_NEW_DELETE);
 
             return pOsgImage;
+        }
 
+        WriteResult::WriteStatus writeTIFStream(std::ostream& fout, const osg::Image& img) const
+        {
+            //Code is based from the following article on CodeProject.com
+            //http://www.codeproject.com/bitmap/BitmapsToTiffs.asp
+
+            TIFF *image;
+            int samplesPerPixel;
+            uint16 photometric;
+
+            image = TIFFClientOpen("outputstream", "w", (thandle_t)&fout,
+                                    libtiffOStreamReadProc, //Custom read function
+                                    libtiffOStreamWriteProc, //Custom write function
+                                    libtiffOStreamSeekProc, //Custom seek function
+                                    libtiffStreamCloseProc, //Custom close function
+                                    libtiffOStreamSizeProc, //Custom size function
+                                    libtiffStreamMapProc, //Custom map function
+                                    libtiffStreamUnmapProc); //Custom unmap function
+            
+            if(image == NULL)
+            {
+                return WriteResult::ERROR_IN_WRITING_FILE;
+            }
+
+            switch(img.getPixelFormat()) {
+                case GL_LUMINANCE:
+                case GL_ALPHA:
+                    photometric = PHOTOMETRIC_MINISBLACK;
+                    samplesPerPixel = 1;
+                    break;
+                case GL_LUMINANCE_ALPHA:
+                    photometric = PHOTOMETRIC_MINISBLACK;
+                    samplesPerPixel = 2;
+                    break;
+                case GL_RGB:
+                    photometric = PHOTOMETRIC_RGB;
+                    samplesPerPixel = 3;
+                    break;
+                case GL_RGBA:
+                    photometric = PHOTOMETRIC_RGB;
+                    samplesPerPixel = 4;
+                    break;
+                default:
+                    return WriteResult::ERROR_IN_WRITING_FILE;
+                    break;
+            }
+
+            TIFFSetField(image, TIFFTAG_IMAGEWIDTH,img.s());
+            TIFFSetField(image, TIFFTAG_IMAGELENGTH,img.t());
+            TIFFSetField(image, TIFFTAG_BITSPERSAMPLE,8);
+            TIFFSetField(image, TIFFTAG_SAMPLESPERPIXEL,samplesPerPixel);
+            TIFFSetField(image, TIFFTAG_PHOTOMETRIC, photometric);
+            TIFFSetField(image, TIFFTAG_COMPRESSION, COMPRESSION_PACKBITS); 
+            TIFFSetField(image, TIFFTAG_FILLORDER, FILLORDER_MSB2LSB);
+            TIFFSetField(image, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
+
+            //uint32 rowsperstrip = TIFFDefaultStripSize(image, -1); 
+            //TIFFSetField(image, TIFFTAG_ROWSPERSTRIP, rowsperstrip);
+            
+            // Write the information to the file
+            for(int i = 0; i < img.t(); ++i) {
+                TIFFWriteScanline(image,(tdata_t)img.data(0,img.t()-i-1),i,0);
+            }
+
+            // Close the file
+            TIFFClose(image);
+
+            return WriteResult::FILE_SAVED;
+        }
+
+        virtual ReadResult readImage(std::istream& fin,const osgDB::ReaderWriter::Options* =NULL) const
+        {
+            return readTIFStream(fin);
+        }
+
+        virtual ReadResult readImage(const std::string& file, const osgDB::ReaderWriter::Options* options) const
+        {
+            std::string ext = osgDB::getLowerCaseFileExtension(file);
+            if (!acceptsExtension(ext)) return ReadResult::FILE_NOT_HANDLED;
+
+            std::string fileName = osgDB::findDataFile( file, options );
+            if (fileName.empty()) return ReadResult::FILE_NOT_FOUND;
+
+            std::ifstream istream(fileName.c_str(), std::ios::in | std::ios::binary);
+            if(!istream) return ReadResult::FILE_NOT_HANDLED;
+            ReadResult rr = readTIFStream(istream);
+            if(rr.validImage()) rr.getImage()->setFileName(file);
+            return rr;
+        }
+
+        virtual WriteResult writeImage(const osg::Image& img,std::ostream& fout,const osgDB::ReaderWriter::Options* /*options*/) const
+        {
+            WriteResult::WriteStatus ws = writeTIFStream(fout,img);
+            return ws;
+        }
+
+        virtual WriteResult writeImage(const osg::Image &img,const std::string& fileName, const osgDB::ReaderWriter::Options *options) const
+        {
+            std::string ext = osgDB::getFileExtension(fileName);
+            if (!acceptsExtension(ext)) return WriteResult::FILE_NOT_HANDLED;
+
+            std::ofstream fout(fileName.c_str(), std::ios::out | std::ios::binary);
+            if(!fout) return WriteResult::ERROR_IN_WRITING_FILE;
+
+            return writeImage(img,fout,options);
         }
 };
 
