@@ -22,36 +22,79 @@
 
 #include "TXPParser.h"
 #include "TXPArchive.h"
+
 using namespace txp;
 
 #include <sstream>
 
 static osg::ApplicationUsageProxy TXP_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_TXP_DEFAULT_MAX_ANISOTROPY \"<value> [<value>]\"","1.0 | 2.0 | 4.0 | 8.0 | 16.0");
 
-TXPParser::TXPParser():
-_archive(0),
-_currentTop(0),
-_root(0),
-_underBillboardSubgraph(false),
-_numBillboardLevels(0),
-_underLayerSubgraph(false),
-_numLayerLevels(0),
-_layerGeode(0),
-_defaultMaxAnisotropy(1.0f),
-_realMinRange(0.0),
-_realMaxRange(0.0),
-_usedMaxRange(0.0)
+
+class LayerGroup : public osg::Group
 {
-    AddCallback(TRPG_ATTACH,new attachRead(this));
-    AddCallback(TRPG_GEOMETRY,new geomRead(this));
-    AddCallback(TRPG_GROUP,new groupRead(this));
-    AddCallback(TRPG_LOD,new lodRead(this));
-    AddCallback(TRPG_MODELREF,new modelRefRead(this));
-    AddCallback(TRPG_BILLBOARD,new billboardRead(this));
-    AddCallback(TRPG_LIGHT,new lightRead(this));
-    AddCallback(TRPG_LAYER,new layerRead(this));
-    AddCallback(TRPG_LABEL,new labelRead(this));
-    AddCallback(TRPGTILEHEADER,new tileHeaderRead(this));
+public:
+    LayerGroup() : osg::Group() {}
+    
+    LayerGroup(const LayerGroup& gg,const osg::CopyOp& copyop=osg::CopyOp::SHALLOW_COPY):
+        osg::Group(gg, copyop)
+    {};
+    
+    META_Node(txp, LayerGroup);
+protected:
+    virtual ~LayerGroup() {}
+};
+
+class LayerVisitor : public osg::NodeVisitor
+{
+public:
+    LayerVisitor():osg::NodeVisitor(osg::NodeVisitor::TRAVERSE_ALL_CHILDREN) {}
+    virtual void apply(osg::Group& node)
+    {
+        LayerGroup *lg = dynamic_cast<LayerGroup*>(&node);
+        if (lg)
+        {
+            for (unsigned int i=1; i < lg->getNumChildren(); ++i)
+            {
+                osg::Node *child = lg->getChild(i);
+                osg::StateSet *sset = child->getOrCreateStateSet();
+                osg::PolygonOffset* polyoffset = new osg::PolygonOffset;
+                polyoffset->setFactor(-1.0f);
+                polyoffset->setUnits(-200.0f*i);
+                sset->setAttributeAndModes(polyoffset,osg::StateAttribute::ON);
+            }
+        }
+        traverse(node);
+    }
+};
+
+TXPParser::TXPParser():
+    _archive(0),
+    _currentTop(0),
+    _root(0),
+    _underBillboardSubgraph(false),
+    _numBillboardLevels(0),
+    _underLayerSubgraph(false),
+    _numLayerLevels(0),
+    _layerGeode(0),
+    _defaultMaxAnisotropy(1.0f),
+    _realMinRange(0.0),
+    _realMaxRange(0.0),
+    _usedMaxRange(0.0),
+    _childRefCB(0)
+{
+    AddCallback(TRPG_ATTACH,    new attachRead(this));
+    AddCallback(TRPG_CHILDREF,  new childRefRead(this));
+    AddCallback(TRPG_GEOMETRY,  new geomRead(this));
+    AddCallback(TRPG_GROUP,     new groupRead(this));
+    AddCallback(TRPG_LOD,       new lodRead(this));
+    AddCallback(TRPG_MODELREF,  new modelRefRead(this));
+    AddCallback(TRPG_BILLBOARD, new billboardRead(this));
+    AddCallback(TRPG_LIGHT,     new lightRead(this));
+    AddCallback(TRPG_LAYER,     new layerRead(this));
+    AddCallback(TRPG_LABEL,     new labelRead(this));
+    AddCallback(TRPGTILEHEADER, new tileHeaderRead(this));
+
+    _childRefCB = dynamic_cast<childRefRead *>(GetCallback(TRPG_CHILDREF));
     
     if (getenv("OSG_TXP_DEFAULT_MAX_ANISOTROPY"))
     {
@@ -66,16 +109,19 @@ TXPParser::~TXPParser()
 
 osg::Group *TXPParser::parseScene(
     trpgReadBuffer &buf, 
-    std::vector<osg::ref_ptr<osg::StateSet> > &materials,
-    std::vector<osg::ref_ptr<osg::Node> > &models,
+    std::map<int,osg::ref_ptr<osg::StateSet> > &materials,
+    std::map<int,osg::ref_ptr<osg::Node> > &models,
     double realMinRange, double realMaxRange, double usedMaxRange)
 {
     if (_archive == 0) return NULL;
 
+    if(_childRefCB)
+       _childRefCB->Reset();
+
     _root = new osg::Group();
     _currentTop = _root.get();
 
-    _materials = &materials;
+    _materialMap = &materials;
     _localMaterials.clear();
     _models = &models;
 
@@ -102,6 +148,9 @@ osg::Group *TXPParser::parseScene(
     }
     _tileGroups.clear();
 
+	LayerVisitor lv;
+	_root->accept(lv);
+
     return _root.get();
 }
 
@@ -127,6 +176,23 @@ void TXPParser::replaceTileLod(osg::Group* group)
     }
 }
 
+
+unsigned int TXPParser::GetNbChildrenRef() const
+{
+   if(_childRefCB)
+         return _childRefCB->GetNbChildrenRef();
+      else
+         return 0;
+}
+
+const trpgChildRef* TXPParser::GetChildRef(unsigned int idx) const
+{
+   if(_childRefCB)
+      return _childRefCB->GetChildRef(idx);
+   else
+      return 0;
+}
+
 bool TXPParser::StartChildren(void * /*in*/)
 {
 
@@ -287,12 +353,19 @@ osg::Geode* TXPParser::createBoundingBox(int x,int y, int lod)
 
 void TXPParser::loadLocalMaterials()
 {
+    bool separateGeo = false;
+    int majorVer,minorVer;
+    this->getArchive()->GetVersion(majorVer,minorVer);
+    if((majorVer >= TRPG_NOMERGE_VERSION_MAJOR) && (minorVer>=TRPG_NOMERGE_VERSION_MINOR)) {
+	separateGeo = true;
+    }
     // new to 2.0 LOCAL materials
     trpgrImageHelper image_helper(
         _archive->GetEndian(),
         _archive->getDir(),
         *_archive->GetMaterialTable(),
-        *_archive->GetTexTable()
+        *_archive->GetTexTable(),
+		separateGeo
     );
     trpgTileHeader* tile_head = getTileHeaderRef();
 
@@ -312,8 +385,8 @@ void TXPParser::loadLocalMaterials()
             trpgLocalMaterial locmat;
             tile_head->GetLocalMaterial(i,locmat);
             
-            const trpgMaterial* mat;
-            const trpgTexture *tex;
+            const trpgMaterial* mat = 0;
+            const trpgTexture *tex = 0;
 
             int32 size;
             image_helper.GetImageInfoForLocalMat(&locmat, &mat,&tex,size);
@@ -334,7 +407,7 @@ void TXPParser::loadLocalMaterials()
                 {
                 case trpgTextureEnv::Alpha :
                     osg_texenv->setMode(osg::TexEnv::REPLACE);
-                break;
+		    break;
                 case trpgTextureEnv::Decal:
                     osg_texenv->setMode(osg::TexEnv::DECAL);
                     break;
@@ -425,10 +498,10 @@ void TXPParser::loadLocalMaterials()
             }
             
             
-        /* This controls what alpha values in a texture mean.  It can take the values:
-            None,Always,Equal,GreaterThanOrEqual,GreaterThan,
-            LessThanOrEqual,LessThan,Never,NotEqual
-        */
+	    /* This controls what alpha values in a texture mean.  It can take the values:
+	       None,Always,Equal,GreaterThanOrEqual,GreaterThan,
+	       LessThanOrEqual,LessThan,Never,NotEqual
+	    */
             int alphaFunc;
             mat->GetAlphaFunc(alphaFunc);
             if( alphaFunc>=GL_NEVER && alphaFunc<=GL_ALWAYS)
@@ -460,7 +533,7 @@ void TXPParser::loadLocalMaterials()
             }
             _localMaterials[i] = osg_state_set;
         }
-     }
+    }
 }
 
 bool TXPParser::requestModel(int ix) 
@@ -557,8 +630,8 @@ void *modelRefRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
     
     // Note: Array check before you do this
     osg::Node *osg_Model = NULL;
-    std::vector<osg::ref_ptr<osg::Node> >*modelList = _parse->getModels();
-    if( modelList->size() > size_t(modelID) )
+    std::map<int,osg::ref_ptr<osg::Node> >*modelList = _parse->getModels();
+    //if( modelList->size() )
     {
        osg_Model = (*modelList)[modelID].get();
 
@@ -651,6 +724,27 @@ void* attachRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
     return (void*)1;
 }
 
+void* childRefRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
+{
+   
+   // This object contribute nothing to the scenegraph, except
+   // where the children tile should connect.
+   // It only contain location info of the children tile
+   childRefList.push_back(trpgChildRef());
+   trpgReadWriteable& obj = childRefList.back();
+
+   if(obj.Read(buf))
+      return &obj;
+    else
+      return 0;
+
+}
+void childRefRead::Reset()
+{
+   childRefList.clear();
+}
+
+
 //----------------------------------------------------------------------------
 //
 // light Reader Class
@@ -664,7 +758,114 @@ void* lightRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
     int attr_index;
     light.GetAttrIndex(attr_index);
 
-    DeferredLightAttribute& dla = _parse->getLightAttribute(attr_index); 
+	uint32 nvert;
+    light.GetNumVertices(nvert);
+
+	const trpgLightTable *lt = _parse->getArchive()->GetLightTable();
+	trpgLightAttr *ref = const_cast<trpgLightAttr*>(lt->GetLightAttrRef(attr_index));
+	if (!ref)
+	{
+		osg::notify(osg::NOTICE) << "NULL LightAttr " << attr_index << std::endl;
+		return (void*)1;
+	}
+
+	osgSim::LightPointNode *lpNode = new osgSim::LightPointNode();
+
+    trpgColor col;
+    ref->GetFrontColor(col);
+
+    float64 inten;
+    ref->GetFrontIntensity(inten);
+
+    trpgLightAttr::PerformerAttr perfAttr;
+    ref->GetPerformerAttr(perfAttr);
+
+	lpNode->setMaxPixelSize(perfAttr.maxPixelSize);
+    lpNode->setMinPixelSize(perfAttr.minPixelSize);    
+
+	trpg3dPoint norm;
+	ref->GetNormal(norm);
+
+	trpgLightAttr::LightDirectionality direc;
+    ref->GetDirectionality(direc);
+
+	for ( unsigned i=0; i < nvert; i++ )
+	{
+		trpg3dPoint pt;
+        light.GetVertex(i, pt);
+
+		osgSim::LightPoint lp( 
+			true, 
+			osg::Vec3(pt.x,pt.y,pt.z), 
+			osg::Vec4(col.red, col.green,col.blue, 1.0), 
+			inten
+		);
+
+		switch (direc)
+		{
+		case trpgLightAttr::trpg_Unidirectional:
+			{
+				float lobeVert=0.f, lobeHorz=0.f, lobeRoll=0.f;
+				float64 tmp;
+
+				ref->GetHLobeAngle(tmp);
+				lobeHorz = osg::DegreesToRadians( tmp );
+				
+				ref->GetVLobeAngle(tmp);
+				lobeVert = osg::DegreesToRadians( tmp );
+				
+				ref->GetLobeRollAngle(tmp);
+				lobeRoll = osg::DegreesToRadians( tmp );
+
+				osg::Vec3 normal(norm.x,norm.y,norm.z);
+				lp._sector = new osgSim::DirectionalSector( normal, lobeHorz, lobeVert, lobeRoll );
+			}
+			break;
+		case trpgLightAttr::trpg_Bidirectional:
+			{
+				float lobeVert=0.f, lobeHorz=0.f, lobeRoll=0.f;
+				float64 tmp;
+
+				ref->GetHLobeAngle(tmp);
+				lobeHorz = osg::DegreesToRadians( tmp );
+				
+				ref->GetVLobeAngle(tmp);
+				lobeVert = osg::DegreesToRadians( tmp );
+				
+				ref->GetLobeRollAngle(tmp);
+				lobeRoll = osg::DegreesToRadians( tmp );
+
+				osg::Vec3 normal(norm.x,norm.y,norm.z);
+				lp._sector = new osgSim::DirectionalSector( normal, lobeHorz, lobeVert, lobeRoll );
+
+				ref->GetBackColor(col);
+				ref->GetBackIntensity(inten);
+
+				osgSim::LightPoint lp2( 
+					true, 
+					osg::Vec3(pt.x,pt.y,pt.z), 
+					osg::Vec4(col.red, col.green,col.blue, 1.0), 
+					inten
+				);
+
+				lp2._sector = new osgSim::DirectionalSector( -normal, lobeHorz, lobeVert, lobeRoll );
+				lpNode->addLightPoint( lp2 );
+			}
+			break;
+		default:
+			;
+		}
+
+		lpNode->addLightPoint( lp);
+
+	}
+
+	_parse->setCurrentNode(lpNode);
+	_parse->getCurrTop()->addChild(lpNode);
+
+#if 0
+
+    DefferedLightAttribute& dla = _parse->getLightAttribute(attr_index); 
     osgSim::LightPointNode* node = dla.lightPoint.get();
 
     uint32 nvert;
@@ -677,7 +878,7 @@ void* lightRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
             trpg3dPoint pt;
             light.GetVertex(i, pt);
             osg::Matrix matrix;
-    //        matrix.makeTranslate(pt.x,pt.y,pt.z);
+	    //        matrix.makeTranslate(pt.x,pt.y,pt.z);
             matrix.makeRotate(osg::Quat(0.0,dla.attitude));
             matrix.setTrans(pt.x,pt.y,pt.z);
             osg::ref_ptr<osg::MatrixTransform> trans = new osg::MatrixTransform();
@@ -715,6 +916,7 @@ void* lightRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
         _parse->setCurrentNode(g);
         _parse->getCurrTop()->addChild(g);
     }
+#endif
 
     return (void *) 1;
 }
@@ -729,6 +931,11 @@ void* layerRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
     trpgLayer group;
     if (!group.Read(buf)) return NULL;
 
+	osg::ref_ptr<LayerGroup> osgGroup = new LayerGroup();
+    _parse->setCurrentNode(osgGroup.get());
+    _parse->getCurrTop()->addChild(osgGroup.get());
+    return (void*)1;
+
 #if 0
     osg::Group* osgGroup = new osg::Group;
     _parse->setCurrentNode(osgGroup);
@@ -770,7 +977,7 @@ void* labelRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
 
     // Text
     std::ostringstream os;
-    unsigned int nl;
+    std::string::size_type nl;
     std::string lb = *labelText;
     while ( (nl=lb.find_first_of('\\')) != std::string::npos)
     {
@@ -1149,66 +1356,66 @@ void* geomRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
         normals = new osg::Vec3Array(numVert);
         geom.GetNormals((float*)&(normals->front()));
     }
- 
+    
     // Set up the primitive type
     switch (primType)
     {
     case trpgGeometry::Triangles:
-        {
-            geometry = new osg::Geometry;
-            geometry->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::TRIANGLES,0,numPrims*3));
-        }
-        break;
+    {
+	geometry = new osg::Geometry;
+	geometry->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::TRIANGLES,0,numPrims*3));
+    }
+    break;
     case trpgGeometry::Quads:
-        {
-            geometry = new osg::Geometry;
-            geometry->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::QUADS,0,numPrims*4));
-        }
-        break;
+    {
+	geometry = new osg::Geometry;
+	geometry->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::QUADS,0,numPrims*4));
+    }
+    break;
     case trpgGeometry::TriStrips:
-        {
-            geometry = new osg::Geometry;
-            osg::DrawArrayLengths* dal = new osg::DrawArrayLengths(osg::PrimitiveSet::TRIANGLE_STRIP,0,numPrims);
-            geom.GetPrimLengths(reinterpret_cast<int*>(&(dal->front())));
-            geometry->addPrimitiveSet(dal);
-        }
-        break;
+    {
+	geometry = new osg::Geometry;
+	osg::DrawArrayLengths* dal = new osg::DrawArrayLengths(osg::PrimitiveSet::TRIANGLE_STRIP,0,numPrims);
+	geom.GetPrimLengths(reinterpret_cast<int*>(&(dal->front())));
+	geometry->addPrimitiveSet(dal);
+    }
+    break;
     case trpgGeometry::TriFans:
-        {
-            geometry = new osg::Geometry;
-            osg::DrawArrayLengths* dal = new osg::DrawArrayLengths(osg::PrimitiveSet::TRIANGLE_FAN,0,numPrims);
-            geom.GetPrimLengths(reinterpret_cast<int*>(&(dal->front())));
-            geometry->addPrimitiveSet(dal);
+    {
+	geometry = new osg::Geometry;
+	osg::DrawArrayLengths* dal = new osg::DrawArrayLengths(osg::PrimitiveSet::TRIANGLE_FAN,0,numPrims);
+	geom.GetPrimLengths(reinterpret_cast<int*>(&(dal->front())));
+	geometry->addPrimitiveSet(dal);
 
-            // Need to flip the fans coords.
-            int ind = 0;
-            int i;
-            for (i=0;i<numPrims;++i)
-            {
-                int length = (*dal)[i];
-                int start=ind+1;
-                int end=ind+length-1;
-                // Swap from start+1 to end
-                // Swap vertices, texture coords & normals
-                for (; start < end; ++start, --end )
-                {
-                    std::swap((*vertices)[start], (*vertices)[end]);
-                    for(int texno = 0; texno < num_tex; texno ++ )
-                    {
-                        if( tex_coords[texno].valid() )
-                        {
-                            std::swap((*tex_coords[texno])[start], (*tex_coords[texno])[end]);
-                        }
-                    }
-                    if(normals.valid())
-                    {
-                        std::swap((*normals)[start], (*normals)[end]);
-                    }
-                }
-                ind += length;
-            }
-        }
-        break;
+	// Need to flip the fans coords.
+	int ind = 0;
+	int i;
+	for (i=0;i<numPrims;++i)
+	{
+	    int length = (*dal)[i];
+	    int start=ind+1;
+	    int end=ind+length-1;
+	    // Swap from start+1 to end
+	    // Swap vertices, texture coords & normals
+	    for (; start < end; ++start, --end )
+	    {
+		std::swap((*vertices)[start], (*vertices)[end]);
+		for(int texno = 0; texno < num_tex; texno ++ )
+		{
+		    if( tex_coords[texno].valid() )
+		    {
+			std::swap((*tex_coords[texno])[start], (*tex_coords[texno])[end]);
+		    }
+		}
+		if(normals.valid())
+		{
+		    std::swap((*normals)[start], (*normals)[end]);
+		}
+	    }
+	    ind += length;
+	}
+    }
+    break;
     default:
         break;
     };
@@ -1250,9 +1457,10 @@ void* geomRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
                 _parse->loadMaterial(matId);
                 tmp_ss = (*_parse->getMaterials())[matId];
             }
-            if(sset.valid()) 
+            if((sset!=0L) && sset.valid()) 
             {
-                if(tmp_ss.valid()){
+                if(tmp_ss.valid())
+		{
                     osg::StateAttribute* texenv0 = tmp_ss->getTextureAttribute(0,osg::StateAttribute::TEXENV);
                     if(texenv0) 
                         sset->setTextureAttribute(n_mat,texenv0);
@@ -1345,6 +1553,7 @@ void* geomRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
             top->addChild(billboard.get());
         }
         else
+#if 0
         if (_parse->underLayerSubgraph())
         {
             osg::Geode* layer = _parse->getLayerGeode();
@@ -1355,10 +1564,11 @@ void* geomRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
                 osg::PolygonOffset* polyoffset = new osg::PolygonOffset;
 
                 poStateSet->merge(*sset.get());
-                polyoffset->setFactor(-2.0f*layer->getNumDrawables());
-                polyoffset->setUnits(-10.0f*layer->getNumDrawables());
+                polyoffset->setFactor(-1.0f);//*layer->getNumDrawables());
+                polyoffset->setUnits(-20.0f);//*layer->getNumDrawables());
                 poStateSet->setAttributeAndModes(polyoffset,osg::StateAttribute::ON);
 
+
                 geometry->setStateSet(poStateSet);
             }
             else
@@ -1369,6 +1579,7 @@ void* geomRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
             layer->addDrawable(geometry.get());
         }
         else
+#else
         {
             geometry->setStateSet(sset.get());
             if (geodeTop)
@@ -1385,7 +1596,7 @@ void* geomRead::Parse(trpgToken /*tok*/,trpgReadBuffer &buf)
                 _parse->getCurrTop()->addChild(geode);
             }
         }
-        
+#endif        
         
     }
     else
@@ -1459,6 +1670,12 @@ namespace
                 pixelFormat = GL_COMPRESSED_RGBA_S3TC_DXT5_EXT;
             }
             break;
+        case trpgTexture::trpg_MCM5:
+        case trpgTexture::trpg_MCM6R:
+        case trpgTexture::trpg_MCM6A:
+        case trpgTexture::trpg_MCM7RA:
+        case trpgTexture::trpg_MCM7AR:
+            break;
         }
     }
 }
@@ -1526,6 +1743,7 @@ osg::Texture2D* txp::getLocalTexture(trpgrImageHelper& image_helper, const trpgT
             {
                 mipmaps[k-1] = tmp_tex->MipLevelOffset(k);
             }
+
             image->setMipmapLevels(mipmaps);
 
         }
@@ -1601,6 +1819,7 @@ osg::Texture2D* txp::getTemplateTexture(trpgrImageHelper& image_helper, trpgLoca
             {
                 mipmaps[k-1] = tmp_tex->MipLevelOffset(k);
             }
+
             image->setMipmapLevels(mipmaps);
 
         }
