@@ -56,7 +56,6 @@
 
 static int s_debugMessages = 0; 
 
-
 #define EVAL(exp) \
 {\
 	if(!(exp))\
@@ -85,6 +84,7 @@ static int s_debugMessages = 0;
 static inline void
 clear(void* dest, uint32_t size)
 {
+#ifdef COMPILE_ASSEMBLY
 	__asm__ __volatile__(
 
 		"xorl %%eax, %%eax\n\t"
@@ -96,6 +96,7 @@ clear(void* dest, uint32_t size)
 		: "=&D" (dest)
 		: "r" (size >> 2), "r" (size & 3), "0" (dest)
 		: "eax", "ecx");
+#endif
 }
 
 #ifdef DEBUG
@@ -190,6 +191,7 @@ static const uint16_t wmask[] =
 
 
 
+#ifdef COMPILE_ASSEMBLY
 
 static void
 __3dnow_convert_yuy2(uint8_t* yuv[], rgb_planar_t* rgb,
@@ -921,7 +923,7 @@ __mmx_convert_yv12(uint8_t* yuv[], rgb_planar_t* rgb,
 	__asm__ __volatile__("emms\n\t");
 }
 
-
+#endif
 
 
 #define range(x)  ((x < 0) ? 0 : ((x > 0xff) ? 0xff : x))
@@ -1069,9 +1071,11 @@ static const rgbout_converter_t convert_methods[] =
 {
 /*        <name>   <accel>      <convert yuy2>        <convert yv12>    */
 	{    NULL,        0, __dummy_convert_yuy2, __dummy_convert_yv12},
+#ifdef COMPILE_ASSEMBLY
 	{   "MMX",   MM_MMX,   __mmx_convert_yuy2,   __mmx_convert_yv12},
 	{   "SSE",   MM_SSE,   __sse_convert_yuy2,   __sse_convert_yv12},
 	{"3DNow!", MM_3DNOW, __3dnow_convert_yuy2, __3dnow_convert_yv12}
+#endif
 /* currently 3DNow is the best function, therefore it's preferred on AMD cpus */
 };
 
@@ -1095,6 +1099,7 @@ __pack_argb(rgb_planar_t* data, void* dest,
 	uint8_t* b_data = data->b;
 	uint32_t rest = pixels;
 
+#ifdef COMPILE_ASSEMBLY
 
 	if((accel & MM_MMXEXT) == MM_MMXEXT)
 	{
@@ -1182,7 +1187,7 @@ __pack_argb(rgb_planar_t* data, void* dest,
 		__asm__ __volatile__("emms\n\t");
 
 	}
-
+#endif
 
 	while(rest--)
 	{
@@ -1214,6 +1219,7 @@ __pack_argb1555(rgb_planar_t* data, void* dest,
 	uint8_t* b_data = data->b;
 	int32_t rest = pixels;
 
+#ifdef COMPILE_ASSEMBLY
 
 	if((accel & MM_MMXEXT) == MM_MMXEXT)
 	{
@@ -1318,7 +1324,7 @@ __pack_argb1555(rgb_planar_t* data, void* dest,
 
 	}
 
-
+#endif
 	while(rest--)
 	{
 		uint16_t r5, g5, b5;
@@ -1343,6 +1349,7 @@ __pack_rgb32(rgb_planar_t* data, void* dest,
 	uint8_t* b_data = data->b;
 	uint32_t rest = pixels;
 
+#ifdef COMPILE_ASSEMBLY
 
 	if((accel & MM_MMXEXT) == MM_MMXEXT)
 	{
@@ -1427,7 +1434,7 @@ __pack_rgb32(rgb_planar_t* data, void* dest,
 
 		__asm__ __volatile__("emms\n\t");
 	}
-
+#endif
 
 	while(rest--)
 	{
@@ -1453,6 +1460,7 @@ __pack_rgb24(rgb_planar_t* data, void* dest,
 
 
 /* MMXEXT doesn't speed up here */
+#ifdef COMPILE_ASSEMBLY
 
 	if((accel & MM_MMX) == MM_MMX)
 	{
@@ -1500,7 +1508,7 @@ __pack_rgb24(rgb_planar_t* data, void* dest,
 		__asm__ __volatile__("emms\n\t");
 
 	}
-
+#endif
 
 	while(rest--)
 	{
@@ -1524,6 +1532,7 @@ __pack_rgb16(rgb_planar_t* data, void* dest,
 	uint8_t* b_data = data->b;
 	uint32_t rest = pixels;
 
+#ifdef COMPILE_ASSEMBLY
 
 	if((accel & MM_MMXEXT) == MM_MMXEXT)
 	{
@@ -1619,7 +1628,7 @@ __pack_rgb16(rgb_planar_t* data, void* dest,
 		__asm__ __volatile__("emms\n\t");
 
 	}
-
+#endif
 
 	while(rest--)
 	{
@@ -1651,6 +1660,7 @@ __pack_bgra(rgb_planar_t* data, void* dest,
 	uint32_t rest = pixels;
 
 
+#ifdef COMPILE_ASSEMBLY
 	if((accel & MM_MMXEXT) == MM_MMXEXT)
 	{
 		uint32_t n = pixels >> 2; /* (width * height) / 4 */
@@ -1730,7 +1740,7 @@ __pack_bgra(rgb_planar_t* data, void* dest,
 
 	}
 
-
+#endif
 	while(rest--)
 	{
 		*(buffer)     = 0xff;
@@ -1759,6 +1769,7 @@ __pack_bgra5551(rgb_planar_t* data, void* dest,
 	uint32_t rest = pixels;
 
 
+#ifdef COMPILE_ASSEMBLY
 	if((accel & MM_MMXEXT) == MM_MMXEXT)
 	{
 		uint32_t n = pixels >> 2; /* (width * height) / 4 */
@@ -1855,7 +1866,7 @@ __pack_bgra5551(rgb_planar_t* data, void* dest,
 		__asm__ __volatile__("emms\n\t");
 
 	}
-
+#endif
 
 	while(rest--)
 	{
@@ -1881,6 +1892,7 @@ __pack_bgr32(rgb_planar_t* data, void* dest,
 	uint8_t* b_data = data->b;
 	uint32_t rest = pixels;
 
+#ifdef COMPILE_ASSEMBLY
 
 	if((accel & MM_MMXEXT) == MM_MMXEXT)
 	{
@@ -1966,7 +1978,7 @@ __pack_bgr32(rgb_planar_t* data, void* dest,
 		__asm__ __volatile__("emms\n\t");
 
 	}
-
+#endif
 
 	while(rest--)
 	{
@@ -1993,6 +2005,7 @@ __pack_bgr24(rgb_planar_t* data, void* dest,
 
 /* MMXEXT doesn't speed up here */
 
+#ifdef COMPILE_ASSEMBLY
 	if((accel & MM_MMX) == MM_MMX)
 	{
 		uint32_t n = pixels >> 2; /* (width * height) / 4 */
@@ -2038,6 +2051,7 @@ __pack_bgr24(rgb_planar_t* data, void* dest,
 		__asm__ __volatile__("emms\n\t");
 
 	}
+#endif
 
 
 	while(rest--)
@@ -2063,6 +2077,7 @@ __pack_bgr16(rgb_planar_t* data, void* dest,
 	uint32_t rest = pixels;
 
 
+#ifdef COMPILE_ASSEMBLY
 	if((accel & MM_MMXEXT) == MM_MMXEXT)
 	{
 		uint32_t n = pixels >> 2; /* (width * height) / 4 */
@@ -2157,6 +2172,7 @@ __pack_bgr16(rgb_planar_t* data, void* dest,
 		__asm__ __volatile__("emms\n\t");
 
 	}
+#endif
 
 
 	while(rest--)
@@ -2771,6 +2787,7 @@ plugin_info_t xine_plugin_info[] =
 {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_OUT, 20, "rgb", XINE_VERSION_CODE, &vo_info_rgbout, init_class},
+  { PLUGIN_VIDEO_OUT, 21, "rgb", XINE_VERSION_CODE, &vo_info_rgbout, init_class},
   { PLUGIN_NONE, 0, "", 0, NULL, NULL}
 };
 
