@@ -32,6 +32,8 @@
 using namespace Producer;
 using namespace osgProducer;
 
+static osg::ApplicationUsageProxy OsgCameraGroup_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE, "OSG_PROCESSOR_AFFINITY <mode>", "ON | OFF - Where supported, switch on or off the processor affinity." );
+
 OsgCameraGroup::RealizeCallback::~RealizeCallback()
 {
 }
@@ -42,11 +44,17 @@ public:
 
     RenderSurfaceRealizeCallback(OsgCameraGroup* cameraGroup,OsgSceneHandler* sceneHandler):
         _cameraGroup(cameraGroup),
-        _sceneHandler(sceneHandler) {}
+        _sceneHandler(sceneHandler)
+    {
+        const OsgCameraGroup::SceneHandlerList& shl = _cameraGroup->getSceneHandlerList();
+        for(unsigned int i=0; i<shl.size(); ++i)
+        {
+            if (shl[i]==sceneHandler) _sceneHandlerNumber = i;
+        }
+    }
     
     virtual void operator()( const Producer::RenderSurface & rs)
     {
-    
         osg::Timer timer;
         osg::Timer_t start_t = timer.tick();
     
@@ -62,8 +70,12 @@ public:
 
     }
 
-    OsgCameraGroup* _cameraGroup;
-    OsgSceneHandler* _sceneHandler;
+    OsgCameraGroup*     _cameraGroup;
+    OsgSceneHandler*    _sceneHandler;
+
+    int                 _numberOfProcessors;
+    int                 _sceneHandlerNumber;
+    bool                _enableProccessAffinityHint;
 
 };
 
@@ -130,6 +142,18 @@ OsgCameraGroup::OsgCameraGroup(osg::ArgumentParser& arguments):
     _init();
     _applicationUsage = arguments.getApplicationUsage();
     
+    while (arguments.read("--affinity"))
+    {
+        _enableProccessAffinityHint = true;
+    }
+
+    // report the usage options.
+    if (arguments.getApplicationUsage())
+    {
+        arguments.getApplicationUsage()->addCommandLineOption("--affinity","Enable processor affinity where supported.");
+    }
+
+
     for( unsigned int i = 0; i < _cfg->getNumberOfCameras(); i++ )
     {
         Producer::Camera *cam = _cfg->getCamera(i);
@@ -139,6 +163,9 @@ OsgCameraGroup::OsgCameraGroup(osg::ArgumentParser& arguments):
             rs->setWindowName(arguments.getApplicationName());
         }
     }    
+
+
+
 }
 
 
@@ -266,12 +293,7 @@ void OsgCameraGroup::_init()
         
     }
 
-    // initialize the number of graphics contexts to set up if there arn't already enough.
-    if (osg::DisplaySettings::instance()->getMaxNumberOfGraphicsContexts() < numContexts)
-    {    
-        osg::DisplaySettings::instance()->setMaxNumberOfGraphicsContexts(numContexts);
-    }
-
+    osg::DisplaySettings::instance()->setMaxNumberOfGraphicsContexts(numContexts);
 
     if ((_threadModel==ThreadPerCamera || _threadModel==ThreadPerRenderSurface) && _cfg->getNumberOfCameras()>1)
     {
@@ -298,6 +320,14 @@ void OsgCameraGroup::_init()
     if (!_frameStamp) _frameStamp = new osg::FrameStamp;
     
     _applicationUsage = osg::ApplicationUsage::instance();
+    
+    _enableProccessAffinityHint = false;
+
+    str = getenv("OSG_PROCESSOR_AFFINITY");
+    if (str && (strcmp(str,"ON")==0 || strcmp(str,"On")==0 || strcmp(str,"on")==0))
+    {
+        _enableProccessAffinityHint = true;
+    }
 
 }
 
@@ -466,6 +496,19 @@ bool OsgCameraGroup::realize()
 
     if (!_ds) _ds = osg::DisplaySettings::instance();
 
+    if (_enableProccessAffinityHint)
+    {
+        unsigned int numProcessors = OpenThreads::GetNumberOfProcessors();
+        if (numProcessors>1)
+        {
+            for( unsigned int i = 0; i < _cfg->getNumberOfCameras(); i++ )
+            {
+                Producer::Camera *cam = _cfg->getCamera(i);
+                cam->setProcessorAffinity(i % numProcessors);
+            }    
+        }
+    }
+
     _shvec.clear();
     
     osg::Node* node = getTopMostSceneData();
