@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -15,11 +15,13 @@
 #include <osgSim/LightPointSystem>
 
 #include "LightPointDrawable.h"
+#include "LightPointSpriteDrawable.h"
 
 #include <osg/Timer>
 #include <osg/BoundingBox>
 #include <osg/BlendFunc>
 #include <osg/Material>
+#include <osg/PointSprite>
 
 #include <osgUtil/CullVisitor>
 
@@ -46,7 +48,8 @@ LightPointNode::LightPointNode():
     _minPixelSize(0.0f),
     _maxPixelSize(30.0f),
     _maxVisibleDistance2(FLT_MAX),
-    _lightSystem(0)
+    _lightSystem(0),
+    _pointSprites(false)
 {
     setStateSet(getSingletonLightPointSystemSet());
 }
@@ -58,7 +61,8 @@ LightPointNode::LightPointNode(const LightPointNode& lpn,const osg::CopyOp& copy
     _minPixelSize(lpn._minPixelSize),
     _maxPixelSize(lpn._maxPixelSize),
     _maxVisibleDistance2(lpn._maxVisibleDistance2),
-    _lightSystem(lpn._lightSystem)
+    _lightSystem(lpn._lightSystem),
+    _pointSprites(lpn._pointSprites)
 {
 }
 
@@ -153,14 +157,14 @@ void LightPointNode::traverse(osg::NodeVisitor& nv)
     
         osg::Matrix matrix = cv->getModelViewMatrix();
         osg::RefMatrix& projection = cv->getProjectionMatrix();
-        osgUtil::RenderGraph* rg = cv->getCurrentRenderGraph();
+        osgUtil::StateGraph* rg = cv->getCurrentStateGraph();
 
         if (rg->leaves_empty())
         {
-            // this is first leaf to be added to RenderGraph
+            // this is first leaf to be added to StateGraph
             // and therefore should not already know to current render bin,
             // so need to add it.
-            cv->getCurrentRenderBin()->addRenderGraph(rg);
+            cv->getCurrentRenderBin()->addStateGraph(rg);
         }
         
 #ifdef USE_TIMER
@@ -178,17 +182,20 @@ void LightPointNode::traverse(osg::NodeVisitor& nv)
                 drawable = static_cast<LightPointDrawable*>(object);
 
             }
+            else if (typeid(*object)==typeid(LightPointSpriteDrawable))
+            {
+                drawable = static_cast<LightPointSpriteDrawable*>(object);
+            }
             else
             {
                 // will need to replace UserData.
-                osg::notify(osg::WARN) << "Warning: Replacing osgUtil::RenderGraph::_userData to support osgSim::LightPointNode, may have undefined results."<<std::endl;
+                osg::notify(osg::WARN) << "Warning: Replacing osgUtil::StateGraph::_userData to support osgSim::LightPointNode, may have undefined results."<<std::endl;
             }
         }
 
         if (!drawable)
         {
-            // set it for the frst time.
-            drawable = new LightPointDrawable;
+            drawable = _pointSprites ? new LightPointSpriteDrawable : new LightPointDrawable;
             rg->setUserData(drawable);
             
             if (cv->getFrameStamp())
@@ -197,9 +204,9 @@ void LightPointNode::traverse(osg::NodeVisitor& nv)
             }
         }
 
-        // search for a drawable in the RenderLead list equal to the attached the one attached to RenderGraph user data
+        // search for a drawable in the RenderLead list equal to the attached the one attached to StateGraph user data
         // as this will be our special light point drawable.
-        osgUtil::RenderGraph::LeafList::iterator litr;
+        osgUtil::StateGraph::LeafList::iterator litr;
         for(litr = rg->_leaves.begin();
             litr != rg->_leaves.end() && (*litr)->_drawable!=drawable;
             ++litr)
@@ -208,7 +215,7 @@ void LightPointNode::traverse(osg::NodeVisitor& nv)
         if (litr == rg->_leaves.end())
         {
             // havn't found the drawable added in the RenderLeaf list, there this my be the 
-            // first time through LightPointNode in this frame, so need to add drawable into the RenderGraph RenderLeaf list
+            // first time through LightPointNode in this frame, so need to add drawable into the StateGraph RenderLeaf list
             // and update its time signatures.
 
             drawable->reset();
@@ -251,14 +258,14 @@ void LightPointNode::traverse(osg::NodeVisitor& nv)
             ++itr)
         {
             const LightPoint& lp = *itr;
-	    
-	    if (!lp._on) continue;
-	    
+    
+            if (!lp._on) continue;
+
             const osg::Vec3& position = lp._position;
-            
+
             // skip light point if it is not contianed in the view frustum.
             if (computeClipping && !clipvol.contains(position)) continue;
-                        
+
             // delta vector between eyepoint and light point.
             osg::Vec3 dv(eyePoint-position);
 
@@ -267,25 +274,27 @@ void LightPointNode::traverse(osg::NodeVisitor& nv)
             // slip light point if it is intensity is 0.0 or negative.
             if (intensity<=minimumIntensity) continue;
 
-
+            // (SIB) Clip on distance, if close to limit, add transparancy
+            float distanceFactor = 1.0f;
             if (_maxVisibleDistance2!=FLT_MAX)
             {
                 if (dv.length2()>_maxVisibleDistance2) continue;
+                else if (_maxVisibleDistance2 > 0)
+                    distanceFactor = 1.0f - osg::square(dv.length2() / _maxVisibleDistance2);
             }
 
-
             osg::Vec4 color = lp._color;
 
             // check the sector.
             if (lp._sector.valid())
             {
                 intensity *= (*lp._sector)(dv);
-            
+
                 // slip light point if it is intensity is 0.0 or negative.
                 if (intensity<=minimumIntensity) continue;
-                
+
             }
-            
+
             // temporary accounting of intensity.
             //color *= intensity;
 
@@ -302,20 +311,24 @@ void LightPointNode::traverse(osg::NodeVisitor& nv)
                 color[2] *= bs[2];
                 color[3] *= bs[3];
             }
-            
+
             // if alpha value is less than the min intentsive then skip
             if (color[3]<=minimumIntensity) continue;
 
             float pixelSize = cv->pixelSize(position,lp._radius);
 
-//            cout << "pixelsize = "<<pixelSize<<endl;
-            
+            //            cout << "pixelsize = "<<pixelSize<<endl;
+
             // adjust pixel size to account for intensity.
             if (intensity!=1.0) pixelSize *= sqrt(intensity);
 
+            // adjust alfa to account for max range (Fade on distance)
+            color[3] *= distanceFactor;
+
             // round up to the minimum pixel size if required.
+            float orgPixelSize = pixelSize;
             if (pixelSize<_minPixelSize) pixelSize = _minPixelSize;
-            
+
             osg::Vec3 xpos(position*matrix);
 
             if (lp._blendingMode==LightPoint::BLENDED)
@@ -323,8 +336,8 @@ void LightPointNode::traverse(osg::NodeVisitor& nv)
                 if (pixelSize<1.0f)
                 {
                     // need to use alpha blending...
-                    //color[3] = pixelSize;
-                    color[3] *= osg::square(pixelSize);
+                    color[3] *= pixelSize;
+                    // color[3] *= osg::square(pixelSize);
 
                     if (color[3]<=minimumIntensity) continue;
 
@@ -334,12 +347,14 @@ void LightPointNode::traverse(osg::NodeVisitor& nv)
                 {
 
                     unsigned int lowerBoundPixelSize = (unsigned int)pixelSize;
-                    //float remainder = pixelSize-(float)lowerBoundPixelSize;
                     float remainder = osg::square(pixelSize-(float)lowerBoundPixelSize);
-                    float alpha = color[3];
-                    drawable->addBlendedLightPoint(lowerBoundPixelSize-1, xpos,color);
 
-                    color[3] = alpha*remainder;
+                    // (SIB) Add transparency if pixel is clamped to minpixelsize
+                    if (orgPixelSize<_minPixelSize)
+                        color[3] *= (2.0/3.0) + (1.0/3.0) * sqrt(orgPixelSize / pixelSize);
+
+                    drawable->addBlendedLightPoint(lowerBoundPixelSize-1, xpos,color);
+                    color[3] *= remainder;
                     drawable->addBlendedLightPoint(lowerBoundPixelSize, xpos,color);
                 }
                 else // use a billboard geometry.
@@ -352,8 +367,8 @@ void LightPointNode::traverse(osg::NodeVisitor& nv)
                 if (pixelSize<1.0f)
                 {
                     // need to use alpha blending...
-                    //color[3] = pixelSize;
-                    color[3] *= osg::square(pixelSize);
+                    color[3] *= pixelSize;
+                    // color[3] *= osg::square(pixelSize);
 
                     if (color[3]<=minimumIntensity) continue;
 
@@ -363,12 +378,15 @@ void LightPointNode::traverse(osg::NodeVisitor& nv)
                 {
 
                     unsigned int lowerBoundPixelSize = (unsigned int)pixelSize;
-                    //float remainder = pixelSize-(float)lowerBoundPixelSize;
                     float remainder = osg::square(pixelSize-(float)lowerBoundPixelSize);
+
+                    // (SIB) Add transparency if pixel is clamped to minpixelsize
+                    if (orgPixelSize<_minPixelSize)
+                        color[3] *= (2.0/3.0) + (1.0/3.0) * sqrt(orgPixelSize / pixelSize);
+
                     float alpha = color[3];
                     color[3] = alpha*(1.0f-remainder);
                     drawable->addAdditiveLightPoint(lowerBoundPixelSize-1, xpos,color);
-
                     color[3] = alpha*remainder;
                     drawable->addAdditiveLightPoint(lowerBoundPixelSize, xpos,color);
                 }
