@@ -17,7 +17,7 @@
 
 #include <osg/State>
 #include <osg/Notify>
-#include <osg/TexEnv>
+#include <osg/ApplicationUsage>
 
 #include <osgDB/ReadFile>
 #include <osgDB/FileUtils>
@@ -27,6 +27,9 @@
 using namespace osgText;
 using namespace std;
 
+static osg::ApplicationUsageProxy Font_e0(osg::ApplicationUsage::ENVIRONMENTAL_VARIABLE,"OSG_TEXT_INCREMENTAL_SUBLOADING <type>","ON | OFF");
+
+
 std::string osgText::findFontFile(const std::string& str)
 {
     // try looking in OSGFILEPATH etc first for fonts.
@@ -138,8 +141,8 @@ Font::Font(FontImplementation* implementation):
     _magFilterHint(osg::Texture::LINEAR)
 {
     setImplementation(implementation);
-    _texEnv = new osg::TexEnv(osg::TexEnv::BLEND);
 
+    _texenv = new osg::TexEnv;
     _stateset = new osg::StateSet;
     _stateset->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
 }
@@ -301,15 +304,11 @@ void Font::addGlyph(unsigned int width, unsigned int height, unsigned int charco
     if (!glyphTexture)
     {
         
-        osg::StateSet* stateset = new osg::StateSet;
-        _stateSetList.push_back(stateset);
-
         glyphTexture = new GlyphTexture;
         
         static int numberOfTexturesAllocated = 0;
         ++numberOfTexturesAllocated;
 
-        osg::notify(osg::INFO)<<"    Creating new GlyphTexture "<<glyphTexture<<"& StateSet "<<stateset<<std::endl;
         osg::notify(osg::INFO)<< "   Font " << this<< ", numberOfTexturesAllocated "<<numberOfTexturesAllocated<<std::endl;
 
         // reserve enough space for the glyphs.
@@ -321,12 +320,6 @@ void Font::addGlyph(unsigned int width, unsigned int height, unsigned int charco
         
         _glyphTextureList.push_back(glyphTexture);
         
-        glyphTexture->setStateSet(stateset);
-        stateset->setMode(GL_BLEND,osg::StateAttribute::ON);
-        stateset->setTextureAttributeAndModes(0,glyphTexture,osg::StateAttribute::ON);
-        if (_texEnv.valid()) stateset->setTextureAttribute(0,_texEnv.get());
-        stateset->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);
-
         if (!glyphTexture->getSpaceForGlyph(glyph,posX,posY))
         {
             osg::notify(osg::WARN)<<"Warning: unable to allocate texture big enough for glyph"<<std::endl;
@@ -342,7 +335,6 @@ void Font::addGlyph(unsigned int width, unsigned int height, unsigned int charco
 
 
 Font::GlyphTexture::GlyphTexture():
-    _stateset(0),
     _margin(2),
     _usedY(0),
     _partUsedX(0),
@@ -524,6 +516,12 @@ void Font::GlyphTexture::apply(osg::State& state) const
             // subloading bugs by loading all at once.
             s_subloadAllGlyphsTogether = true;
         }
+        
+        const char* str = getenv("OSG_TEXT_INCREMENTAL_SUBLOADING");
+        if (str && (strcmp(str,"OFF")==0 || strcmp(str,"Off")==0 || strcmp(str,"Off")==0))
+        {
+            s_subloadAllGlyphsTogether = true;
+        }
     }
 
 
@@ -562,7 +560,7 @@ void Font::GlyphTexture::apply(osg::State& state) const
         }
         else
         {
-            //std::cout<<"all loading"<<std::endl;
+            osg::notify(osg::INFO)<<"osgText::Font loading all glyphs as a single subload."<<std::endl;
 
             // Octane has bugs in OGL driver which mean that subloads smaller
             // than 32x32 produce errors, and also cannot handle general alignment,
@@ -652,9 +650,6 @@ void Font::Glyph::setTexture(GlyphTexture* texture) { _texture = texture; }
 Font::GlyphTexture* Font::Glyph::getTexture() { return _texture; }
 const Font::GlyphTexture* Font::Glyph::getTexture() const { return _texture; }
 
-osg::StateSet* Font::Glyph::getStateSet() { return _texture?_texture->getStateSet():0; }
-const osg::StateSet* Font::Glyph::getStateSet() const { return _texture?_texture->getStateSet():0; }
-
 void Font::Glyph::setTexturePosition(int posX,int posY) { _texturePosX = posX; _texturePosY = posY; }
 int Font::Glyph::getTexturePositionX() const { return _texturePosX; }
 int Font::Glyph::getTexturePositionY() const { return _texturePosY; }
