@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -18,6 +18,8 @@
 #include <osg/Notify>
 #include <osg/TriangleFunctor>
 #include <osg/Geometry>
+#include <osg/Projection>
+#include <osg/CameraNode>
 #include <osg/io_utils>
 
 #include <float.h>
@@ -111,24 +113,20 @@ bool IntersectVisitor::IntersectState::isCulled(const BoundingSphere& bs,LineSeg
     LineSegmentMask mask = 0x00000001;
     segMaskOut = 0x00000000;
     LineSegmentMask segMaskIn = _segmentMaskStack.back();
-    //    notify(INFO) << << "IntersectState::isCulled() mask in "<<segMaskIn<<"  ";
     for(IntersectState::LineSegmentList::iterator sitr=_segList.begin();
         sitr!=_segList.end();
         ++sitr)
     {
         if ((segMaskIn & mask) && (sitr->second)->intersect(bs))
         {
-            //            notify(INFO) << << "Hit ";
             segMaskOut = segMaskOut| mask;
             hit = true;
         }
         mask = mask << 1;
     }
-    //    notify(INFO) << << "mask = "<<segMaskOut<< std::endl;
     return !hit;
 }
 
-
 bool IntersectVisitor::IntersectState::isCulled(const BoundingBox& bb,LineSegmentMask& segMaskOut)
 {
     bool hit = false;
@@ -149,6 +147,36 @@ bool IntersectVisitor::IntersectState::isCulled(const BoundingBox& bb,LineSegmen
     return !hit;
 }
 
+void IntersectVisitor::IntersectState::addLineSegment(osg::LineSegment* seg)
+{
+    // create a new segment transformed to local coordintes.
+    LineSegment* ns = new LineSegment;
+
+    if (_model_inverse.valid()) 
+    {
+        if (_view_inverse.valid())
+        {
+            osg::Matrix matrix = (*(_view_inverse)) * (*(_model_inverse));
+            ns->mult(*seg,matrix);
+        }
+        else
+        { 
+            ns->mult(*seg,*(_model_inverse));
+        }
+    }
+    else if (_view_inverse.valid())
+    {
+        ns->mult(*seg,*(_view_inverse));
+    }
+    else
+    {
+        *ns = *seg;
+    }
+    
+    _segList.push_back(LineSegmentPair(seg,ns));
+}
+
+
 IntersectVisitor::IntersectVisitor()
 {
 
@@ -158,7 +186,7 @@ IntersectVisitor::IntersectVisitor()
     // Initialize eyepoint to 0,0,0
     setEyePoint(Vec3(0.0f,0.0f,0.0f));
 
-    setLODSelectionMode(USE_HEIGHEST_LEVEL_OF_DETAIL); // orignal IntersectVisitor behavior
+    setLODSelectionMode(USE_HIGHEST_LEVEL_OF_DETAIL); // orignal IntersectVisitor behavior
     //setLODSelectionMode(USE_SEGMENT_START_POINT_AS_EYE_POINT_FOR_LOD_LEVEL_SELECTION);
 
     reset();
@@ -185,6 +213,7 @@ float IntersectVisitor::getDistanceToEyePoint(const Vec3& pos, bool /*withLODSca
 {
     if (_lodSelectionMode==USE_SEGMENT_START_POINT_AS_EYE_POINT_FOR_LOD_LEVEL_SELECTION)
     {
+        // osg::notify(osg::NOTICE)<<"IntersectVisitor::getDistanceToEyePoint)"<<(pos-getEyePoint()).length()<<std::endl;
         // LODScale is not available to IntersectVisitor, so we ignore the withLODScale argument
         //if (withLODScale) return (pos-getEyePoint()).length()*getLODScale();
         //else return (pos-getEyePoint()).length();
@@ -208,6 +237,25 @@ bool IntersectVisitor::hits()
     return false;
 }
 
+osg::Vec3 IntersectVisitor::getEyePoint() const
+{
+    const IntersectState* cis = _intersectStateStack.empty() ? 0 : _intersectStateStack.back().get();
+    if (cis && (cis->_model_inverse.valid() || cis->_view_inverse.valid()))
+    {
+        
+        osg::Vec3 eyePoint = _pseudoEyePoint;
+        if (cis->_view_inverse.valid()) eyePoint = eyePoint * (*(cis->_view_inverse));
+        if (cis->_model_inverse.valid()) eyePoint = eyePoint * (*(cis->_model_inverse));
+        
+        //osg::notify(osg::NOTICE)<<"IntersectVisitor::getEyePoint()"<<eyePoint<<std::endl;
+
+        return eyePoint;
+    }
+    else
+    {
+        return _pseudoEyePoint;
+    }
+}
 
 void IntersectVisitor::addLineSegment(LineSegment* seg)
 {
@@ -219,9 +267,16 @@ void IntersectVisitor::addLineSegment(LineSegment* seg)
         notify(WARN)<<"         "<<seg->start()<<" "<<seg->end()<<" segment ignored.."<< std::endl;
         return;
     }
-
+    
     IntersectState* cis = _intersectStateStack.back().get();
     
+    if (cis->_segList.size()>=32)
+    {
+        notify(WARN)<<"Warning: excessive number of line segmenets passed to IntersectVisitor::addLineSegment(..), maximum permitted is 32 line segments."<<std::endl;
+        notify(WARN)<<"         "<<seg->start()<<" "<<seg->end()<<" segment ignored.."<< std::endl;
+        return;
+    }
+
     setEyePoint(seg->start()); // set start of line segment to be pseudo EyePoint for billboarding and LOD purposes
 
     // first check to see if segment has already been added.
@@ -232,36 +287,48 @@ void IntersectVisitor::addLineSegment(LineSegment* seg)
         if (itr->first == seg) return;
     }
 
-    // create a new segment transformed to local coordintes.
-    LineSegment* ns = new LineSegment;
-
-    if (cis->_inverse.valid()) ns->mult(*seg,*(cis->_inverse));
-    else *ns = *seg;
-    
-    cis->addLineSegmentPair(seg,ns);
+    cis->addLineSegment(seg);
 
 }
 
 
-void IntersectVisitor::pushMatrix(const Matrix& matrix)
+void IntersectVisitor::pushMatrix(RefMatrix* matrix, osg::Transform::ReferenceFrame rf)
 {
     IntersectState* nis = new IntersectState;
 
     IntersectState* cis = _intersectStateStack.back().get();
 
-    if (cis->_matrix.valid())
+    if (rf == osg::Transform::RELATIVE_RF)
     {
-        nis->_matrix = new RefMatrix;
-        nis->_matrix->mult(matrix,*(cis->_matrix));
+        // share the original view matrix
+        nis->_view_matrix = cis->_view_matrix;
+        nis->_view_inverse = cis->_view_inverse;
+        
+        // set up new model matrix
+        nis->_model_matrix = matrix;
+        if (cis->_model_matrix.valid())
+        {
+            nis->_model_matrix->postMult(*(cis->_model_matrix));
+        }
+
+        RefMatrix* inverse_world = new RefMatrix;
+        inverse_world->invert(*(nis->_model_matrix));
+        nis->_model_inverse = inverse_world;
     }
     else
     {
-        nis->_matrix = new RefMatrix(matrix);
+        // set a new view matrix
+        nis->_view_matrix = matrix;
+
+        RefMatrix* inverse_world = new RefMatrix;
+        inverse_world->invert(*(nis->_view_matrix));
+        nis->_view_inverse = inverse_world;
+
+        // model matrix now blank.
+        nis->_model_matrix = 0;
+        nis->_model_inverse = 0;
     }
 
-    RefMatrix* inverse_world = new RefMatrix;
-    inverse_world->invert(*(nis->_matrix));
-    nis->_inverse = inverse_world;
 
     IntersectState::LineSegmentMask segMaskIn = cis->_segmentMaskStack.back();
     IntersectState::LineSegmentMask mask = 0x00000001;
@@ -271,9 +338,7 @@ void IntersectVisitor::pushMatrix(const Matrix& matrix)
     {
         if ((segMaskIn & mask))
         {
-            LineSegment* seg = new LineSegment;
-            seg->mult(*(sitr->first),*inverse_world);
-            nis->addLineSegmentPair(sitr->first.get(),seg);
+            nis->addLineSegment(sitr->first.get());
         }
         mask = mask << 1;
     }
@@ -294,7 +359,7 @@ void IntersectVisitor::popMatrix()
 bool IntersectVisitor::enterNode(Node& node)
 {
     const BoundingSphere& bs = node.getBound();
-    if (bs.valid())
+    if (bs.valid() && node.isCullingActive())
     {
         IntersectState* cis = _intersectStateStack.back().get();
         IntersectState::LineSegmentMask sm=0xffffffff;
@@ -304,7 +369,13 @@ bool IntersectVisitor::enterNode(Node& node)
     }
     else
     {
-        return false;
+        IntersectState* cis = _intersectStateStack.back().get();
+        if (!cis->_segmentMaskStack.empty()) 
+            cis->_segmentMaskStack.push_back(cis->_segmentMaskStack.back());
+        else
+            cis->_segmentMaskStack.push_back(0xffffffff);
+        
+        return true;
     }
 }
 
@@ -533,8 +604,8 @@ bool IntersectVisitor::intersect(Drawable& drawable)
                 
                     Hit hit;
                     hit._nodePath = _nodePath;
-                    hit._matrix = cis->_matrix;
-                    hit._inverse = cis->_inverse;
+                    hit._matrix = cis->_model_matrix;
+                    hit._inverse = cis->_model_inverse;
                     hit._drawable = &drawable;
                     if (_nodePath.empty()) hit._geode = NULL;
                     else hit._geode = dynamic_cast<Geode*>(_nodePath.back());
@@ -605,10 +676,10 @@ void IntersectVisitor::apply(Billboard& node)
         osg::ref_ptr<RefMatrix> billboard_matrix = new RefMatrix;
         node.computeMatrix(*billboard_matrix,eye_local,pos);
 
-        pushMatrix(*billboard_matrix);
+        pushMatrix(billboard_matrix.get(), osg::Transform::RELATIVE_RF);
 
         intersect(*node.getDrawable(i));
-	
+
         popMatrix();
 
     }
@@ -634,7 +705,7 @@ void IntersectVisitor::apply(Transform& node)
     osg::ref_ptr<RefMatrix> matrix = new RefMatrix;
     node.computeLocalToWorldMatrix(*matrix,this);
 
-    pushMatrix(*matrix);
+    pushMatrix(matrix.get(), node.getReferenceFrame());
 
     traverse(node);
 
@@ -654,3 +725,108 @@ void IntersectVisitor::apply(LOD& node)
 {
     apply((Group&)node);
 }
+
+PickVisitor::PickVisitor(const osg::Viewport* viewport, const osg::Matrixd& proj, const osg::Matrixd& view, float mx, float my):
+    _mx(mx),
+    _my(my),
+    _lastViewport(viewport),
+    _lastProjectionMatrix(proj),
+    _lastViewMatrix(view)
+{
+    setLODSelectionMode(USE_SEGMENT_START_POINT_AS_EYE_POINT_FOR_LOD_LEVEL_SELECTION);
+
+    if (viewport && 
+        mx >= static_cast<float>(viewport->x()) && 
+        my >= static_cast<float>(viewport->y()) &&
+        mx < static_cast<float>(viewport->x()+viewport->width()) &&
+        my < static_cast<float>(viewport->y()+viewport->height()))
+    {
+
+        // mouse pointer intersect viewport so we can proceed to set up a line segment
+        osg::Matrix MVPW = proj * viewport->computeWindowMatrix();
+        osg::Matrixd inverseMVPW;
+        inverseMVPW.invert(MVPW);
+
+        osg::Vec3 nearPoint = osg::Vec3(mx,my,0.0f) * inverseMVPW;
+        osg::Vec3 farPoint = osg::Vec3(mx,my,1.0f) * inverseMVPW;
+        osg::LineSegment* lineSegment = new osg::LineSegment;
+        lineSegment->set(nearPoint, farPoint);
+
+        IntersectState* cis = !_intersectStateStack.empty() ? _intersectStateStack.back().get() : 0;
+        if (cis)
+        {
+            cis->_view_matrix = new RefMatrix(view);
+            cis->_view_inverse = new RefMatrix;
+            cis->_view_inverse->invert(*(cis->_view_matrix));
+
+            cis->_model_matrix = 0;
+            cis->_model_inverse = 0;
+        }
+        else
+        {
+            osg::notify(osg::NOTICE)<<"Warning: PickVisitor not set up correctly, picking errors likely"<<std::endl;
+        }
+        
+
+        addLineSegment(lineSegment);
+    }
+}
+
+void PickVisitor::runNestedPickVisitor(osg::Node& node, const osg::Viewport* viewport, const osg::Matrix& proj, const osg::Matrix& view, float mx, float my)
+{
+
+    PickVisitor newPickVisitor( viewport, proj, view, mx, my );
+    newPickVisitor.setTraversalMask(getTraversalMask());
+
+    newPickVisitor.getNodePath() = getNodePath();
+
+    // the new pickvisitor over the nodes children.
+    node.traverse( newPickVisitor );
+
+    for(LineSegmentHitListMap::iterator itr = newPickVisitor._segHitList.begin();
+        itr != newPickVisitor._segHitList.end();
+        ++itr)
+    {
+        _segHitList.insert(*itr);
+    }
+}
+
+void PickVisitor::apply(osg::Projection& projection)
+{
+    runNestedPickVisitor( projection, 
+                          _lastViewport.get(), 
+                          projection.getMatrix(),
+                          _lastViewMatrix,
+                          _mx, _my );
+}
+
+void PickVisitor::apply(osg::CameraNode& camera)
+{
+    if (!camera.isRenderToTextureCamera())
+    {
+        if (camera.getReferenceFrame()==osg::CameraNode::ABSOLUTE_RF)
+        {
+            runNestedPickVisitor( camera,
+                                  camera.getViewport() ? camera.getViewport() : _lastViewport.get(),
+                                  camera.getProjectionMatrix(), 
+                                  camera.getViewMatrix(),
+                                  _mx, _my );
+        }
+        else if (camera.getTransformOrder()==osg::CameraNode::POST_MULTIPLY)
+        {
+            runNestedPickVisitor( camera,
+                                  camera.getViewport() ? camera.getViewport() : _lastViewport.get(),
+                                  _lastProjectionMatrix * camera.getProjectionMatrix(), 
+                                  _lastViewMatrix * camera.getViewMatrix(),
+                                  _mx, _my );
+        }
+        else // PRE_MULTIPLY
+        {
+            runNestedPickVisitor( camera,
+                                  camera.getViewport() ? camera.getViewport() : _lastViewport.get(),
+                                  camera.getProjectionMatrix() * _lastProjectionMatrix, 
+                                  camera.getViewMatrix() * _lastViewMatrix,
+                                  _mx, _my );
+        }
+    }
+}
