@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -16,6 +16,7 @@
 
 #include <osg/Notify>
 #include <osg/ApplicationUsage>
+#include <osg/AlphaFunc>
 
 #include <algorithm>
 
@@ -26,8 +27,8 @@ using namespace osgUtil;
 class RenderBinPrototypeList : public osg::Referenced, public std::map< std::string, osg::ref_ptr<RenderBin> > 
 {
     public:
-	RenderBinPrototypeList() {}
-	~RenderBinPrototypeList() {}
+        RenderBinPrototypeList() {}
+        ~RenderBinPrototypeList() {}
 };
 
 // register a RenderStage prototype with the RenderBin prototype list.
@@ -128,6 +129,21 @@ RenderBin::RenderBin(SortMode mode)
     _parent = NULL;
     _stage = NULL;
     _sortMode = mode;
+
+#if 1
+    if (_sortMode==SORT_BACK_TO_FRONT)
+    {
+        _stateset  = new osg::StateSet;
+        _stateset->setThreadSafeRefUnref(true);
+        
+         // set up an alphafunc by default to speed up blending operations.
+        osg::AlphaFunc* alphafunc = new osg::AlphaFunc;
+        alphafunc->setFunction(osg::AlphaFunc::GREATER,0.0f);
+        alphafunc->setThreadSafeRefUnref(true);
+        
+        _stateset->setAttributeAndModes(alphafunc, osg::StateAttribute::ON);
+    }
+#endif    
 }
 
 RenderBin::RenderBin(const RenderBin& rhs,const CopyOp& copyop):
@@ -136,11 +152,12 @@ RenderBin::RenderBin(const RenderBin& rhs,const CopyOp& copyop):
         _parent(rhs._parent),
         _stage(rhs._stage),
         _bins(rhs._bins),
-        _renderGraphList(rhs._renderGraphList),
+        _stateGraphList(rhs._stateGraphList),
         _renderLeafList(rhs._renderLeafList),
         _sortMode(rhs._sortMode),
         _sortCallback(rhs._sortCallback),
-        _drawCallback(rhs._drawCallback)
+        _drawCallback(rhs._drawCallback),
+        _stateset(rhs._stateset)
 {
 
 }
@@ -151,7 +168,7 @@ RenderBin::~RenderBin()
 
 void RenderBin::reset()
 {
-    _renderGraphList.clear();
+    _stateGraphList.clear();
     _bins.clear();
 }
 
@@ -199,7 +216,7 @@ void RenderBin::sortImplementation()
 
 struct SortByStateFunctor
 {
-    bool operator() (const RenderGraph* lhs,const RenderGraph* rhs) const
+    bool operator() (const StateGraph* lhs,const StateGraph* rhs) const
     {
         return (*(lhs->_stateset)<*(rhs->_stateset));
     }
@@ -210,15 +227,15 @@ void RenderBin::sortByState()
     //osg::notify(osg::NOTICE)<<"sortByState()"<<std::endl;
     // actually we'll do nothing right now, as fine grained sorting by state
     // appears to cost more to do than it saves in draw.  The contents of
-    // the RenderGraph leaves is already coarse grained sorted, this
+    // the StateGraph leaves is already coarse grained sorted, this
     // sorting is as a function of the cull traversal.
     // cout << "doing sortByState "<<this<<endl;
 }
 
 
-struct RenderGraphFrontToBackSortFunctor
+struct StateGraphFrontToBackSortFunctor
 {
-    bool operator() (const RenderGraph* lhs,const RenderGraph* rhs) const
+    bool operator() (const StateGraph* lhs,const StateGraph* rhs) const
     {
         return (lhs->_minimumDistance<rhs->_minimumDistance);
     }
@@ -226,14 +243,14 @@ struct RenderGraphFrontToBackSortFunctor
 
 void RenderBin::sortByStateThenFrontToBack()
 {
-    for(RenderGraphList::iterator itr=_renderGraphList.begin();
-        itr!=_renderGraphList.end();
+    for(StateGraphList::iterator itr=_stateGraphList.begin();
+        itr!=_stateGraphList.end();
         ++itr)
     {
         (*itr)->sortFrontToBack();
         (*itr)->getMinimumDistance();
     }
-    std::sort(_renderGraphList.begin(),_renderGraphList.end(),RenderGraphFrontToBackSortFunctor());
+    std::sort(_stateGraphList.begin(),_stateGraphList.end(),StateGraphFrontToBackSortFunctor());
 }
 
 struct FrontToBackSortFunctor
@@ -247,7 +264,7 @@ struct FrontToBackSortFunctor
     
 void RenderBin::sortFrontToBack()
 {
-    copyLeavesFromRenderGraphListToRenderLeafList();
+    copyLeavesFromStateGraphListToRenderLeafList();
 
     // now sort the list into acending depth order.
     std::sort(_renderLeafList.begin(),_renderLeafList.end(),FrontToBackSortFunctor());
@@ -265,7 +282,7 @@ struct BackToFrontSortFunctor
 
 void RenderBin::sortBackToFront()
 {
-    copyLeavesFromRenderGraphListToRenderLeafList();
+    copyLeavesFromStateGraphListToRenderLeafList();
 
     // now sort the list into acending depth order.
     std::sort(_renderLeafList.begin(),_renderLeafList.end(),BackToFrontSortFunctor());
@@ -273,14 +290,14 @@ void RenderBin::sortBackToFront()
 //    cout << "sort back to front"<<endl;
 }
 
-void RenderBin::copyLeavesFromRenderGraphListToRenderLeafList()
+void RenderBin::copyLeavesFromStateGraphListToRenderLeafList()
 {
     _renderLeafList.clear();
 
     int totalsize=0;
-    RenderGraphList::iterator itr;
-    for(itr=_renderGraphList.begin();
-        itr!=_renderGraphList.end();
+    StateGraphList::iterator itr;
+    for(itr=_stateGraphList.begin();
+        itr!=_stateGraphList.end();
         ++itr)
     {
         totalsize += (*itr)->_leaves.size();
@@ -289,11 +306,11 @@ void RenderBin::copyLeavesFromRenderGraphListToRenderLeafList()
     _renderLeafList.reserve(totalsize);
     
     // first copy all the leaves from the render graphs into the leaf list.
-    for(itr=_renderGraphList.begin();
-        itr!=_renderGraphList.end();
+    for(itr=_stateGraphList.begin();
+        itr!=_stateGraphList.end();
         ++itr)
     {
-        for(RenderGraph::LeafList::iterator dw_itr = (*itr)->_leaves.begin();
+        for(StateGraph::LeafList::iterator dw_itr = (*itr)->_leaves.begin();
             dw_itr != (*itr)->_leaves.end();
             ++dw_itr)
         {
@@ -302,7 +319,7 @@ void RenderBin::copyLeavesFromRenderGraphListToRenderLeafList()
     }
     
     // empty the render graph list to prevent it being drawn along side the render leaf list (see drawImplementation.)
-    _renderGraphList.clear();
+    _stateGraphList.clear();
 }
 
 RenderBin* RenderBin::find_or_insert(int binNum,const std::string& binName)
@@ -322,7 +339,7 @@ RenderBin* RenderBin::find_or_insert(int binNum,const std::string& binName)
             rs->_binNum = binNum;
             rs->_parent = NULL;
             rs->_stage = rs;
-            _stage->addToDependencyList(rs);
+            _stage->addPreRenderStage(rs);
         }
         else
         {
@@ -346,6 +363,18 @@ void RenderBin::draw(osg::State& state,RenderLeaf*& previous)
 
 void RenderBin::drawImplementation(osg::State& state,RenderLeaf*& previous)
 {
+    // osg::notify(osg::NOTICE)<<"begin RenderBin::drawImplementation "<<className()<<" sortMode "<<getSortMode()<<std::endl;
+
+    if (_stateset.valid())
+    {
+        // first need to flush the stack
+        previous = 0;
+        state.popAllStateSets();
+
+        // now its safe to push RenderBin's StateSet as we now know its the root.        
+        state.pushStateSet(_stateset.get());
+    }
+
     // draw first set of draw bins.
     RenderBinList::iterator rbitr;
     for(rbitr = _bins.begin();
@@ -368,12 +397,12 @@ void RenderBin::drawImplementation(osg::State& state,RenderLeaf*& previous)
 
 
     // draw coarse grained ordering.
-    for(RenderGraphList::iterator oitr=_renderGraphList.begin();
-        oitr!=_renderGraphList.end();
+    for(StateGraphList::iterator oitr=_stateGraphList.begin();
+        oitr!=_stateGraphList.end();
         ++oitr)
     {
 
-        for(RenderGraph::LeafList::iterator dw_itr = (*oitr)->_leaves.begin();
+        for(StateGraph::LeafList::iterator dw_itr = (*oitr)->_leaves.begin();
             dw_itr != (*oitr)->_leaves.end();
             ++dw_itr)
         {
@@ -393,94 +422,76 @@ void RenderBin::drawImplementation(osg::State& state,RenderLeaf*& previous)
         rbitr->second->draw(state,previous);
     }
 
+    if (_stateset.valid())
+    {
+        state.popAllStateSets();
+        previous = 0;
+    }
 
+    //osg::notify(osg::NOTICE)<<"end RenderBin::drawImplementation "<<className()<<std::endl;
 }
 
 // stats
-bool RenderBin::getStats(Statistics* primStats)
-{ // different by return type - collects the stats in this renderrBin
-  bool somestats=false;
+bool RenderBin::getStats(Statistics& stats) const
+{
+    stats.addBins(1);
+
+    // different by return type - collects the stats in this renderrBin
+    bool statsCollected = false;
 
     // draw fine grained ordering.
-    for(RenderLeafList::iterator dw_itr = _renderLeafList.begin();
+    for(RenderLeafList::const_iterator dw_itr = _renderLeafList.begin();
         dw_itr != _renderLeafList.end();
         ++dw_itr)
     {
-      RenderLeaf* rl = *dw_itr;
-      Drawable* dw= rl->_drawable;
-      primStats->addDrawable(); // number of geosets
-      if (rl->_modelview.get()) primStats->addMatrix(); // number of matrices
-      if (dw)
+        const RenderLeaf* rl = *dw_itr;
+        const Drawable* dw= rl->_drawable;
+        stats.addDrawable(); // number of geosets
+        if (rl->_modelview.get())
         {
-          // then tot up the primtive types and no vertices.
-          dw->accept(*primStats); // use sub-class to find the stats for each drawable
+            stats.addMatrix(); // number of matrices
         }
-      somestats = true;
-
+        
+        if (dw)
+        {
+              // then tot up the primtive types and no vertices.
+              dw->accept(stats); // use sub-class to find the stats for each drawable
+        }
+        statsCollected = true;
     }
 
-  for(RenderGraphList::iterator oitr=_renderGraphList.begin();
-        oitr!=_renderGraphList.end();
+    for(StateGraphList::const_iterator oitr=_stateGraphList.begin();
+        oitr!=_stateGraphList.end();
         ++oitr)
     {
         
-        for(RenderGraph::LeafList::iterator dw_itr = (*oitr)->_leaves.begin();
+        for(StateGraph::LeafList::const_iterator dw_itr = (*oitr)->_leaves.begin();
             dw_itr != (*oitr)->_leaves.end();
             ++dw_itr)
         {
-            RenderLeaf* rl = dw_itr->get();
-            Drawable* dw= rl->_drawable;
-            primStats->addDrawable(); // number of geosets
-            if (rl->_modelview.get()) primStats->addMatrix(); // number of matrices
+            const RenderLeaf* rl = dw_itr->get();
+            const Drawable* dw= rl->_drawable;
+            stats.addDrawable(); // number of geosets
+            if (rl->_modelview.get()) stats.addMatrix(); // number of matrices
             if (dw)
             {
                 // then tot up the primtive types and no vertices.
-                dw->accept(*primStats); // use sub-class to find the stats for each drawable
+                dw->accept(stats); // use sub-class to find the stats for each drawable
             }
         }
-        somestats=true;
+        statsCollected = true;
     }
-    return somestats;
-}
 
-void RenderBin::getPrims(Statistics* primStats)
-{
-    static int ndepth;
-    ndepth++;
-    for(RenderBinList::iterator itr = _bins.begin();
+    // now collects stats for any subbins.
+    for(RenderBinList::const_iterator itr = _bins.begin();
         itr!=_bins.end();
         ++itr)
     {
-        primStats->addBins(1);
-        itr->second->getPrims(primStats);
+        if (itr->second->getStats(stats))
+        {
+            statsCollected = true;
+        }
     }
-    getStats(primStats);
-    ndepth--;
 
-}
-
-bool RenderBin::getPrims(Statistics* primStats, int nbin)
-{ // collect stats for array of bins, maximum nbin 
-    // (which will be modified on next call if array of primStats is too small);
-    // return 1 for OK;
-    static int ndepth;
-    bool ok=false;
-    ndepth++;
-    int nb=primStats[0].getBins();
-    if (nb<nbin)
-    { // if statement to protect against writing to bins beyond the maximum seen before
-        primStats[nb].setBinNo(nb);
-        primStats[nb].setDepth(ndepth);
-        getStats(primStats+nb);
-    }
-    primStats[0].addBins(1);
-    for(RenderBinList::iterator itr = _bins.begin();
-        itr!=_bins.end();
-        ++itr)
-    {
-        if (itr->second->getPrims(primStats, nbin)) ok = true;
-    }
-    ok=true;
-    ndepth--;
-    return ok;
+    return statsCollected;
 }
