@@ -1,4 +1,4 @@
-/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2005 Robert Osfield 
+/* -*-c++-*- OpenSceneGraph - Copyright (C) 1998-2006 Robert Osfield 
  *
  * This library is open source and may be redistributed and/or modified under  
  * the terms of the OpenSceneGraph Public License (OSGPL) version 0.0 or 
@@ -11,7 +11,8 @@
  * OpenSceneGraph Public License for more details.
 */
 #include <osgUtil/RenderLeaf>
-#include <osgUtil/RenderGraph>
+#include <osgUtil/StateGraph>
+#include <osg/Notify>
 
 using namespace osg;
 using namespace osgUtil;
@@ -33,12 +34,12 @@ void RenderLeaf::render(State& state,RenderLeaf* previous)
         state.applyModelViewMatrix(_modelview.get());
 
         // apply state if required.
-        RenderGraph* prev_rg = previous->_parent;
-        RenderGraph* prev_rg_parent = prev_rg->_parent;
-        RenderGraph* rg = _parent;
+        StateGraph* prev_rg = previous->_parent;
+        StateGraph* prev_rg_parent = prev_rg->_parent;
+        StateGraph* rg = _parent;
         if (prev_rg_parent!=rg->_parent)
         {
-            RenderGraph::moveRenderGraph(state,prev_rg_parent,rg->_parent);
+            StateGraph::moveStateGraph(state,prev_rg_parent,rg->_parent);
 
             // send state changes and matrix changes to OpenGL.
             state.apply(rg->_stateset);
@@ -63,11 +64,13 @@ void RenderLeaf::render(State& state,RenderLeaf* previous)
         state.applyModelViewMatrix(_modelview.get());
 
         // apply state if required.
-         RenderGraph::moveRenderGraph(state,NULL,_parent->_parent);
+        StateGraph::moveStateGraph(state,NULL,_parent->_parent);
 
         state.apply(_parent->_stateset);
 
         // draw the drawable
         _drawable->draw(state);
     }
+    
+    // osg::notify(osg::NOTICE)<<"RenderLeaf "<<_drawable->getName()<<" "<<_depth<<std::endl;
 }
