@@ -1230,6 +1230,11 @@ void SceneView::draw()
     // re apply the defalt OGL state.
     _state->popAllStateSets();
     _state->apply();
+    
+    if (_camera->getPostDrawCallback())
+    {
+        (*(_camera->getPostDrawCallback()))(*_camera);
+    }
 
     if (_state->getCheckForGLErrors()!=osg::State::NEVER_CHECK_GL_ERRORS)
     {
