@@ -27,11 +27,23 @@
 #include "Win32MutexPrivateData.h"
 using namespace OpenThreads;
 
+Win32MutexPrivateData::Win32MutexPrivateData()
+{
+#ifdef USE_CRITICAL_SECTION
+    InitializeCriticalSection( &_cs );
+#else
+    mutex  = 0;
+#endif
+}
 Win32MutexPrivateData::~Win32MutexPrivateData()
 {
+#ifdef USE_CRITICAL_SECTION
+    DeleteCriticalSection( &_cs );
+#endif
 }
 
 
+#ifndef USE_CRITICAL_SECTION
 
 template <int instance>
 struct WIN32MutexSpin {
@@ -54,16 +66,18 @@ static void _S_nsec_sleep(int __log_nsec) {
 
     if (__log_nsec <= 20) {
         SwitchToThread(); //Sleep(0); // adegli replaced it Sleep by SwitchToThread
-	} else {
+    } else {
         Sleep(1 << (__log_nsec - 20));
-	}
+    }
 }
 
 
 #if defined(_MSC_VER) && _MSC_VER <= 1300
-	template WIN32MutexSpin <0>;
+    template WIN32MutexSpin <0>;
 #endif
 
+#endif // USE_CRITICAL_SECTION
+
 //----------------------------------------------------------------------------
 //
 // Decription: Constructor
@@ -72,7 +86,6 @@ static void _S_nsec_sleep(int __log_nsec) {
 //
 Mutex::Mutex() {
     Win32MutexPrivateData *pd = new Win32MutexPrivateData();
-    pd->mutex  = 0;
     _prvData = static_cast<void *>(pd);
 }
 
@@ -96,10 +109,19 @@ int Mutex::lock() {
     Win32MutexPrivateData *pd =
         static_cast<Win32MutexPrivateData*>(_prvData);
 
+#ifdef USE_CRITICAL_SECTION
+
+    // Block until we can take this lock.
+    EnterCriticalSection( &(pd->_cs) );
+
+    return 0;
+
+#else
+
     volatile unsigned long* lock = &pd->mutex;
-	// InterlockedExchange returns old value
-	// if old_value  == 0 mutex wasn't locked , now it is
-	if( !InterlockedExchange((long*)lock, 1L)) {
+    // InterlockedExchange returns old value
+    // if old_value  == 0 mutex wasn't locked , now it is
+    if( !InterlockedExchange((long*)lock, 1L)) {
        return 0;
     }
 
@@ -135,7 +157,9 @@ int Mutex::lock() {
       }
       _S_nsec_sleep(__log_nsec);
     }
-	return -1;
+    return -1;
+
+#endif // USE_CRITICAL_SECTION
 }
 
 //----------------------------------------------------------------------------
@@ -148,11 +172,23 @@ int Mutex::unlock() {
     Win32MutexPrivateData *pd =
         static_cast<Win32MutexPrivateData*>(_prvData);
 
+#ifdef USE_CRITICAL_SECTION
+
+    // Release this lock. CRITICAL_SECTION is nested, thus
+    //   unlock() calls must be paired with lock() calls.
+    LeaveCriticalSection( &(pd->_cs) );
+
+    return 0;
+
+#else
+
     volatile unsigned long* lock = &pd->mutex;
     *lock = 0;
     // This is not sufficient on many multiprocessors, since
     // writes to protected variables and the lock may be reordered.
-	return 0;
+    return 0;
+
+#endif // USE_CRITICAL_SECTION
 }
 
 //----------------------------------------------------------------------------
@@ -165,12 +201,25 @@ int Mutex::trylock() {
     Win32MutexPrivateData *pd =
         static_cast<Win32MutexPrivateData*>(_prvData);
 
+#ifdef USE_CRITICAL_SECTION
+
+    // Take the lock if we can; regardless don't block.
+    // 'result' is FALSE if we took the lock or already held
+    //   it amd TRUE if another thread already owns the lock.
+    BOOL result = TryEnterCriticalSection( &(pd->_cs) );
+
+    return( (result==TRUE) ? 0 : 1 );
+
+#else
+
     volatile unsigned long* lock = &pd->mutex;
 
-	if( !InterlockedExchange((long*)lock, 1L)) {
+    if( !InterlockedExchange((long*)lock, 1L)) {
       return 1; // TRUE
     }
 
-	return 0; // FALSE
+    return 0; // FALSE
 
+#endif // USE_CRITICAL_SECTION
 }
+
