@@ -237,23 +237,15 @@ Thread::Thread() {
     Win32ThreadPrivateData *pd = new Win32ThreadPrivateData();
 
     pd->stackSize = 0;
-
     pd->isRunning = false;
-
     pd->cancelMode = 0;
-
     pd->uniqueId = 0;
-
     pd->threadPriority = Thread::THREAD_PRIORITY_DEFAULT;
-
     pd->threadPolicy = Thread::THREAD_SCHEDULE_DEFAULT;
-
     pd->detached = false;
-
     pd->cancelEvent.set(CreateEvent(NULL,TRUE,FALSE,NULL));
 
     _prvData = static_cast<void *>(pd);
-
 }
 
 
@@ -300,9 +292,9 @@ int Thread::getThreadId() {
 //
 // Use: public
 //
-int Thread::getProcessId() {
+size_t Thread::getProcessId() {
 
-    return (int) GetCurrentProcessId();
+    return (size_t) GetCurrentProcessId();
 
 }
 //-----------------------------------------------------------------------------
@@ -559,7 +551,7 @@ int Thread::setProcessorAffinity( unsigned int cpunum )
 {
     Win32ThreadPrivateData *pd = static_cast<Win32ThreadPrivateData *> (_prvData);
     DWORD affinityMask  = 0x1 << cpunum ; // thread affinity mask
-	DWORD res =
+	DWORD_PTR res =
 		SetThreadAffinityMask
 		(
 			pd->tid.get(),                  // handle to thread
@@ -640,3 +632,16 @@ int Thread::microSleep(unsigned int microsec)
     return 0;
 #endif
 }
+
+
+//-----------------------------------------------------------------------------
+//
+// Description:  Get the number of processors
+//
+int OpenThreads::GetNumberOfProcessors()
+{
+    SYSTEM_INFO sysInfo;
+    GetSystemInfo(&sysInfo);
+
+    return sysInfo.dwNumberOfProcessors;
+}
