@@ -58,7 +58,7 @@ class KBMCB : public Producer::KeyboardMouseCallback
             _toggleMotionMode(false),
             _setReference(false),
             _done(false) {}
-        ~KBMCB(){}
+
 
         void specialKeyPress( Producer::KeyCharacter key )
         {
@@ -150,6 +150,9 @@ class KBMCB : public Producer::KeyboardMouseCallback
         bool toggleMotionMode() { return checkToggle( _toggleMotionMode ); }
         bool setReference() { return checkToggle(_setReference); }
 
+    protected: 
+        virtual ~KBMCB(){}
+
     private:
         Producer::Trackball &_tb;
         float _mx, _my;
@@ -175,32 +178,32 @@ class KBMCB : public Producer::KeyboardMouseCallback
 int main()
 {
     Producer::ref_ptr<Producer::CameraConfig> cfg  = new Producer::CameraConfig;
-    if( cfg->parseFile( "oneWindow.cfg" ) == false )
+    if( cfg->parseFile( "./oneWindow.cfg" ) == false )
     {
         std::cerr << "This test program requires a camera configuration file named \"oneWindow.cfg\"\n";
         return -1;
     }
+
     Producer::ref_ptr<osgProducer::OsgCameraGroup> cg = new osgProducer::OsgCameraGroup(cfg.get());
 
-    osg::Node *root = osgDB::readNodeFile("cube.osg");
+    osg::ref_ptr<osg::Node> root = osgDB::readNodeFile("cube.osg");
     if( root == NULL )
     {
         std::cerr << "This test program requires an osg file named \"cube.osg\"\n";
         return -1;
     }
 
-    osg::MatrixTransform *tx = new osg::MatrixTransform;
-    tx->addChild(root);
-    cg->setSceneData(tx);
+    osg::ref_ptr<osg::MatrixTransform> tx = new osg::MatrixTransform;
+    tx->addChild(root.get());
+    cg->setSceneData(tx.get());
     cg->realize( Producer::CameraGroup::SingleThreaded );
 
-    osg::StateSet *sset = new osg::StateSet;
+    osg::ref_ptr<osg::StateSet> sset = new osg::StateSet;
     sset->setMode( GL_LIGHTING, osg::StateAttribute::OVERRIDE | osg::StateAttribute::OFF );
-    tx->setStateSet( sset );
+    tx->setStateSet( sset.get()  );
 
     osg::BoundingSphere bs = root->getBound();
 
-
     Producer::ref_ptr<Producer::Trackball> tb = new Producer::Trackball;
     tb->setOrientation(Producer::Trackball::Z_UP);
     tb->setDistance(root->getBound().radius() * 3);
@@ -219,22 +222,21 @@ int main()
     }
     Producer::ref_ptr<Producer::KeyboardMouse> kbm = new Producer::KeyboardMouse(ia.get());
     
-    //Producer::KeyboardMouse kbm(cg->getCamera(0)->getRenderSurface());
-    KBMCB kbmcb(*tb.get());
-    kbm->setCallback( &kbmcb );
+    Producer::ref_ptr<KBMCB> kbmcb = new KBMCB(*tb.get());
+    kbm->setCallback( kbmcb.get() );
     kbm->startThread();
     MotionMode motionMode = MoveCamera;
 
-    while( !kbmcb.done() )
+    while( !kbmcb->done() )
     {
         cg->sync();
 
-        if( kbmcb.setReference() )
+        if( kbmcb->setReference() )
             tb->setReference();
 
-        tb->input( kbmcb.mx(), kbmcb.my(), kbmcb.mbutton() );
+        tb->input( kbmcb->mx(), kbmcb->my(), kbmcb->mbutton() );
 
-        if( kbmcb.toggleMotionMode() )
+        if( kbmcb->toggleMotionMode() )
         {
             motionMode = motionMode == MoveCamera ?
                          motionMode = MoveTransformNode :  
@@ -261,13 +263,13 @@ int main()
             {
                 case Producer::Trackball::Z_UP :
                     cg->setViewByLookat( 0, 0, 0, 
-                                        0, 1, 0, 
-                                        0, 0, 1 );
+                                         0, 1, 0, 
+                                         0, 0, 1 );
                     break;
                 case Producer::Trackball::Y_UP:
-                    cg->setViewByLookat( 0, 0, 0, 
-                                        0, 0, -1, 
-                                        0, 1, 0 );
+                    cg->setViewByLookat( 0, 0,  0, 
+                                         0, 0, -1, 
+                                         0, 1,  0 );
                     break;
             }
         }
