@@ -1,7 +1,8 @@
 #include <iostream>
 #include <Producer/Version>
 
-main()
+int main()
 {
 	std::cout << Producer::Version() << std::endl;
+        return 0;
 }
