@@ -1,12 +1,16 @@
 #include <unistd.h>
 #include <Producer/RenderSurface>
+#ifdef __MINGW32__
+#include <stdlib.h>
+#define sleep(t)  _sleep((t)*1000)
+#endif
 
-main()
+int main()
 {
     Producer::RenderSurface *rs = new Producer::RenderSurface;
 
     rs->setWindowRectangle( 100, 100, 400, 400 );
-    rs->setWindowName( "Yer Momma" );
+    rs->setWindowName( "Window -1" );
 
     rs->realize();
     int count = 0;
@@ -14,7 +18,7 @@ main()
     {
         char buff[25];
         sleep(1);
-        sprintf( buff, "Window %d\n", count++ );
+        sprintf( buff, "Window %d", count++ );
         rs->setWindowName( buff );
     }
 }
