@@ -17,8 +17,6 @@
 
 #include <string.h>
 #include <algorithm>
-//#include <GL/gl.h>
-//#include <GL/glu.h>
 
 #include <Producer/Math>
 #include <Producer/Camera>
@@ -123,6 +121,9 @@ void Camera::_frameInstrumented( bool doSwap )
     }
 
     ++_frameCount;
+
+    _frameStamps.setFrameNumber( _frameCount );
+
     Timer_t stamps[LastStatsID];
     memset( stamps, 0, sizeof(stamps));
 
@@ -149,7 +150,6 @@ void Camera::_frameInstrumented( bool doSwap )
         return;
     }
 
-
     stamps[BeginCull] = _timer.tick();
     if( preCullCallbacks.size() )
     {
@@ -162,7 +162,6 @@ void Camera::_frameInstrumented( bool doSwap )
         stamps[EndPreCullCallbacks] = _timer.tick();
     }
 
-
     stamps[BeginInnerCull] = _timer.tick();
     _sh->cull( *this );
     stamps[EndInnerCull] = _timer.tick();
@@ -177,6 +176,7 @@ void Camera::_frameInstrumented( bool doSwap )
     }
     stamps[EndCull] = _timer.tick();
 
+
     _rs->makeCurrent();
 
     if( _block_on_vsync )
@@ -189,6 +189,7 @@ void Camera::_frameInstrumented( bool doSwap )
     
     
     stamps[BeginDraw] = _timer.tick();
+    _frameStamps.beginPipeTimer( DrawTime );
 
     stamps[BeginClear] = _timer.tick();
     _sh->clear( *this );
@@ -220,9 +221,14 @@ void Camera::_frameInstrumented( bool doSwap )
             (*((*p).get()))(*this);
         stamps[EndPostDrawCallbacks] = _timer.tick();
     }
+    _frameStamps.endPipeTimer();
 
     if( doSwap )
+    {
+        _frameStamps.beginPipeTimer( SwapBuffersTime );
         _rs->swapBuffers();
+        _frameStamps.endPipeTimer();
+    }
 
     if( postSwapCallbacks.size() )
     {
@@ -237,9 +243,10 @@ void Camera::_frameInstrumented( bool doSwap )
     stamps[EndCameraFrame] = _timer.tick();
 
     _frameStamps.clear();
-    _frameStamps.setFrameNumber( _frameCount );
     for( int i = 0; i < LastStatsID; i++ )
         _frameStamps[StatsID(i)] = _timer.delta_s(_initTime, stamps[i]);
+
+    _frameStamps.syncPipeStats();
 }
 
 void Camera::_frame( bool doSwap )
@@ -951,10 +958,6 @@ void Camera::run( void )
 }
 
 
-const Camera::FrameTimeStampSet &Camera::getFrameStats()
-{ 
-    return _frameStamps; 
-}
 
 bool Camera::removePreCullCallback( Callback *cb )
 {
@@ -993,3 +996,74 @@ bool Camera::_removeCallback( std::vector < ref_ptr<Callback> > &callbackList, C
     return true;
 }
 
+Camera::FrameTimeStampSet::FrameTimeStampSet():
+    _pipeStatsDoubleBufferIndex(0),
+    _pipeStatsFirstSync(true),
+    _initialized(false)
+{
+    for( unsigned int i = 0; i < LastPipeStatsID; i++ )
+        _pipeStats[i] = 0.0;
+}
+
+Camera::FrameTimeStampSet::~FrameTimeStampSet()
+{
+}
+
+void Camera::FrameTimeStampSet::syncPipeStats()
+{
+    if( !_initialized )
+        return;
+
+    if( _pipeStatsFirstSync == true )
+    {
+        _pipeStatsFirstSync = false;
+        return;
+    }
+
+    // We get the stats from the previous frame
+    for( int i = 0; i < LastPipeStatsID; i++ )
+    {
+        if( _pipeStatsSetMask[1 - _pipeStatsDoubleBufferIndex]  & (1<<i) )
+            _pipeStats[i] = PipeTimer::instance()->getElapsedTime( _pipeStatsNames[i][ 1 - _pipeStatsDoubleBufferIndex] );
+    }
+
+    _pipeStatsFrameNumber = _frameNumber - 1;
+    _pipeStatsDoubleBufferIndex = 1 - _pipeStatsDoubleBufferIndex;
+    _pipeStatsSetMask[_pipeStatsDoubleBufferIndex] = 0;
+}
+
+void Camera::FrameTimeStampSet::beginPipeTimer( PipeStatsID id)
+{
+    if( !_initialized )
+        _init();
+
+    PipeTimer::instance()->begin( _pipeStatsNames[id][_pipeStatsDoubleBufferIndex] ); 
+    _pipeStatsSetMask[_pipeStatsDoubleBufferIndex] |= (1<<id); 
+}
+
+void Camera::FrameTimeStampSet::endPipeTimer() 
+{ 
+    if( !_initialized )
+        return;
+
+    PipeTimer::instance()->end() ;
+}
+
+void Camera::FrameTimeStampSet::_init()
+{
+    if( _initialized )
+        return;
+
+    for( unsigned int i = 0; i < (unsigned int)LastPipeStatsID; i++ )
+        PipeTimer::instance()->genQueries( 2, _pipeStatsNames[i] );
+
+    _pipeStatsSetMask[0] = 0;
+    _pipeStatsSetMask[1] = 0;
+
+    _initialized = true;
+}
+
+const Camera::FrameTimeStampSet &Camera::getFrameStats()
+{ 
+    return _frameStamps; 
+}
