@@ -14,6 +14,7 @@
 #include <string>
 #include <set>
 #include <Producer/CameraGroup>
+#include <Producer/CameraConfig>
 
 using namespace Producer; 
 
@@ -33,7 +34,16 @@ void CameraGroup::_initVariables()
     //_lens = new Camera::Lens;
     _realized    = false;
     _instrumented = false;
-    _thread_model = SingleThreaded;
+
+    _threadModel = getDefaultThreadModel();
+    if( (ptr = getenv( "PRODUCER_THREAD_MODEL" )))
+    {
+        std::string tm(ptr);
+        _threadModel = 
+            tm == "SingleThreaded"          ? SingleThreaded :
+            tm == "ThreadPerRenderSurface"  ?  ThreadPerRenderSurface :
+            tm == "ThreadPerCamera"         ?  ThreadPerCamera : SingleThreaded;
+    }
 
     ptr = getenv( "PRODUCER_STACK_SIZE" );
     if( ptr )
@@ -197,10 +207,11 @@ void CameraGroup::setStackSize( size_t size )
 }
 
 
-bool CameraGroup::realize( ThreadingModel thread_model )
+bool CameraGroup::realize( ThreadModel threadModel )
 {
     if( _realized ) return _realized;
-    _thread_model = thread_model;
+    _threadModel = threadModel;
+    _cfg->setThreadModelDirective( _threadModel );
     return realize();
 }
 
@@ -211,7 +222,9 @@ bool CameraGroup::realize()
     _frameStats._frameTimeStampSets.resize(_cfg->getNumberOfCameras());
     _initTime = _timer.tick();
 
-    if( _thread_model == SingleThreaded )
+    _threadModel = _cfg->getThreadModelDirective();
+
+    if( _threadModel == SingleThreaded )
     {
         for( unsigned int i = 0; i < _cfg->getNumberOfCameras(); i++ )
         {
@@ -221,10 +234,11 @@ bool CameraGroup::realize()
 #endif
             rs->realize();
             _cfg->getCamera(i)->setBlockOnVsync(_block_on_vsync);
+            _cfg->getCamera(i)->setInitTime( _initTime );
         }
         _realized = true;
     }
-    else if( _thread_model == ThreadPerCamera )
+    else if( _threadModel == ThreadPerCamera )
     {
 #ifdef _X11_IMPLEMENTATION
         RenderSurface::initThreads();
@@ -249,7 +263,7 @@ bool CameraGroup::realize()
     }
     else
     {
-        std::cout << "CameraGroup::realize() : Threading model unsupported\n";
+        std::cout << "CameraGroup::realize() : Thread model unsupported\n";
         return _realized ;
     }
     _initLens();
@@ -317,6 +331,7 @@ void CameraGroup::_singleThreadedFrame()
     
     std::set<RenderSurface *>::iterator p;
     
+    /*
     bool _syncronizeSwapBuffers = true;
     if (_syncronizeSwapBuffers && u.size()>1)
     {
@@ -326,9 +341,14 @@ void CameraGroup::_singleThreadedFrame()
             glFinish();
         }
     }
+    */
     
     for( p =  u.begin(); p != u.end(); p++ )
     {
+        /*
+extern Timer_t B4_SWAP;
+B4_SWAP = Producer::Timer::instance()->tick();
+*/
         (*p)->swapBuffers();
     }
 }
@@ -348,11 +368,11 @@ void CameraGroup::_frame()
         std::cerr << "CameraGroup::frame() : Please call realize() first\n";
         return;
     }
-    if( _thread_model == SingleThreaded )
+    if( _threadModel == SingleThreaded )
     {
         _singleThreadedFrame();
     }
-    else if( _thread_model == ThreadPerCamera )
+    else if( _threadModel == ThreadPerCamera )
     {
         if( _sync_count == _frame_count ) 
             sync();
@@ -361,7 +381,7 @@ void CameraGroup::_frame()
     }
     else
     {
-        std::cout << "CameraGroup::frame() : Threading model unsupported\n";
+        std::cout << "CameraGroup::frame() : Thread model unsupported\n";
     }
     _frame_count++;
 }
@@ -384,7 +404,7 @@ void CameraGroup::_frameInstrumented()
     }
     _startOfFrame = _timer.tick(); 
 
-    if( _thread_model == SingleThreaded )
+    if( _threadModel == SingleThreaded )
     {
         //_endOfUpdate = _timer.tick();
         //_startOfFrame = _timer.tick(); 
@@ -393,7 +413,7 @@ void CameraGroup::_frameInstrumented()
         if( markStartOfUpdate )
             _startOfUpdate = _timer.tick();
     }
-    else if( _thread_model == ThreadPerCamera )
+    else if( _threadModel == ThreadPerCamera )
     {
         //_endOfUpdate =  _timer.tick();
         //_startOfFrame = _timer.tick();
@@ -401,7 +421,7 @@ void CameraGroup::_frameInstrumented()
     }
     else
     {
-        std::cout << "CameraGroup::frame() : Threading model unsupported\n";
+        std::cout << "CameraGroup::frame() : Thread model unsupported\n";
     }
 
     _frame_count++;
@@ -419,11 +439,11 @@ void CameraGroup::_sync()
 {
     if( _sync_count > _frame_count ) return;
 
-    if( _thread_model == SingleThreaded )
+    if( _threadModel == SingleThreaded )
     {
         ; // ignored
     }
-    else if( _thread_model == ThreadPerCamera )
+    else if( _threadModel == ThreadPerCamera )
     {
         if (_syncBarrier.valid())
         {
@@ -436,7 +456,7 @@ void CameraGroup::_sync()
     }
     else
     {
-        std::cout << "CameraGroup::_sync() : Threading model unsupported\n";
+        std::cout << "CameraGroup::_sync() : Thread model unsupported\n";
     }
     _sync_count = _frame_count + 1;
 }
@@ -462,7 +482,7 @@ void CameraGroup::_syncInstrumented( bool markStartOfUpdate )
     if( _sync_count > _frame_count ) return;
     _sync_count = _frame_count + 1;
 
-    if( _thread_model == ThreadPerCamera )
+    if( _threadModel == ThreadPerCamera )
     {
         if (_syncBarrier.valid())
         {
@@ -492,8 +512,7 @@ void CameraGroup::advance()
 
     for( unsigned int i = 0; i < _cfg->getNumberOfCameras(); i++ )
     {
-        _cfg->getCamera(i)->getRenderSurface()->makeCurrent();
-        _cfg->getCamera(i)->getRenderSurface()->swapBuffers();
+        _cfg->getCamera(i)->advance();
     }
 }
 
