@@ -217,8 +217,8 @@
 #include <fcntl.h>
 #endif
 
-#ifdef WIN32
-#define PRODUCER_AVOID_CPP 1
+#ifndef WIN32
+#define SUPPORT_CPP 1
 #endif
 
 #include <stdio.h>
@@ -2234,66 +2234,69 @@ bool CameraConfig::parseFile( const std::string &file )
         return false;
     }
 
-#if defined (PRODUCER_AVOID_CPP)
-
-    std::ifstream ifs(fileName.c_str());
-    flexer = new yyFlexLexer(&ifs);
-    cfg = this;
     bool retval = true;
-    if( ConfigParser_parse() )
-        retval = false;
-    ifs.close();
-    delete flexer;
-    return retval;
 
-#else
+#if defined (SUPPORT_CPP)
 
-    int pd[2];
-    bool retval = true;
-    pipe( pd );
-
-    flexer = new yyFlexLexer;
-    if( fork() == 0 )
-    {
-        // we don't want to read from the pipe in the child, so close it.
-        close( pd[0] );
-        close( 1 );
-        dup( pd[1] );
-
-        char *cpp_path =
-#if defined(__APPLE__)
+    char *cpp_path =
+  #if defined(__APPLE__)
                 "/usr/bin/cpp";
-#else
+  #else
                 "/lib/cpp";
-#endif
+  #endif
 
-        /* This was here to allow reading a config file from stdin.
-         * This has never been directly supported, so commenting out.
-        if( fileName.empty() )
-            execlp( cpp_path, "cpp",  "-P", 0L );
-        else
-        */
+    if( access( cpp_path, X_OK ) == 0 )
+    {
+
+        int pd[2];
+        pipe( pd );
+
+        flexer = new yyFlexLexer;
+        if( fork() == 0 )
+        {
+            // we don't want to read from the pipe in the child, so close it.
+            close( pd[0] );
+            close( 1 );
+            dup( pd[1] );
+
+
+            /* This was here to allow reading a config file from stdin.
+             * This has never been directly supported, so commenting out.
+            if( fileName.empty() )
+                execlp( cpp_path, "cpp",  "-P", 0L );
+            else
+            */
             execlp( cpp_path, "cpp",  "-P", fileName.c_str(), 0L );
-                                                                                                                     
-        // This should not execute unless an error happens
-        perror( "execlp" );
+
+            // This should not execute unless an error happens
+            perror( "execlp" );
+        }
+        else
+        {
+            close( pd[1]);
+            close( 0 );
+            dup( pd[0] );
+
+            cfg = this;
+
+            retval = ConfigParser_parse() == 0 ? true : false;
+
+            // Wait on child process to finish
+            int stat;
+            wait( &stat );
+        }
     }
     else
+#endif
     {
-        close( pd[1]);
-        close( 0 );
-        dup( pd[0] );
-
+        std::ifstream ifs(fileName.c_str());
+        flexer = new yyFlexLexer(&ifs);
         cfg = this;
-
-        retval = ConfigParser_parse();
-                                                                                                                     
-        // Wait on child process to finish
-        int stat;
-        wait( &stat );
+        retval = ConfigParser_parse() == 0 ? true : false;
+        ifs.close();
+        delete flexer;
     }
     return retval;
-#endif
 }
 
 
