@@ -19,7 +19,6 @@ KeyboardProxy proxy;
 
 Keyboard *Keyboard::theKeyboard = 0L;
 
-
 KeyboardProxy::KeyboardProxy() 
 { 
     Keyboard::getSingleton(); 
@@ -48,7 +47,7 @@ void *Keyboard::operator new( size_t size )
 Keyboard *Keyboard::getSingleton()
 {
     if( theKeyboard == 0L )
-        new Keyboard;
+        theKeyboard = ::new Keyboard;
     return theKeyboard;
 }
 
@@ -140,6 +139,15 @@ KeyCharacter Keyboard::win32KeyEventToKeyChar( LPARAM lParam, WPARAM wParam )
     else if( wParam == 0x12 && (lParam & KeyReleased))
         _keyModifier &= ~KeyMod_Alt;
 
+    // Special case for alt key
+    if( wParam == VK_MENU )
+    {
+        if( lParam & 0x1000000 )
+            wParam = VK_RMENU;
+        else
+            wParam = VK_LMENU;
+    }
+
     KeyboardKey key = _keymap[wParam];
     
     // Check control keys
@@ -388,8 +396,8 @@ void Keyboard::_initKeyboardKeyMap()
     _keymap[VK_LWIN         ] = Key_Super_L;
     _keymap[VK_SPACE        ] = Key_space;
 // ALT never really gets to the app on Win32
-    _keymap[0x01            ] = Key_Alt_L;
-    _keymap[0x02            ] = Key_Alt_R;
+    _keymap[VK_LMENU        ] = Key_Alt_L;
+    _keymap[VK_RMENU        ] = Key_Alt_R;
     _keymap[VK_RWIN         ] = Key_Super_R;
     _keymap[VK_APPS         ] = Key_Menu;
     _keymap[VK_RCONTROL     ] = Key_Control_R;
