@@ -27,7 +27,9 @@
 #endif
 
 #ifdef _X11_IMPLEMENTATION
+#define explicit c_explicit
 #include <X11/XKBlib.h>
+#undef explicit
 #endif
 
 #include <iostream>
@@ -351,25 +353,49 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
 
         virtual void fini() 
         {
-            /*
-            if( _changeKBStateMask )
+            // Force a KeyPress event here to kick the thread out of select/XNextEvent
+            if( _rs->isRealized() )
             {
-                XKeyboardControl kc;
-                if( _changeKBStateMask & KBAutoRepeatMode )
-                    kc.auto_repeat_mode = _originalKBState.global_auto_repeat;
-                XChangeKeyboardControl( dpy, _changeKBStateMask, &kc );
-            }
-            */
+                // Need a local, temporary connection to the server
+                char dpyname[128];
 
-            // This should only be called from the run() thread if 
-            // we are multi-threaded
-            /*
-             * commenting out these calls as they are suspect 
-             * 
-            XSync(dpy,0);
-            XCloseDisplay(dpy);
-            cancel();
-            */
+                std::string hostname = _rs->getHostName();
+                if( hostname.empty() )
+                    sprintf( dpyname, ":%d.%d",  _rs->getDisplayNum(), _rs->getScreenNum() );
+                else
+                    sprintf( dpyname, "%s:%d.%d",
+                            hostname.c_str(),
+                            _rs->getDisplayNum(), _rs->getScreenNum() );
+
+                Display *dpy = XOpenDisplay(dpyname);
+                if( dpy )
+                {
+                    XEvent ev;
+                    ev.xkey.keycode = 9; // 9 = ESC
+                    if( _inputArea != NULL )
+                    {
+                        for( unsigned int i = 0; i < _inputArea->getNumRenderSurfaces(); i++ )
+                        {
+                            RenderSurface *rs = _inputArea->getRenderSurface(i);
+                            ev.type = KeyPress;
+                            XSendEvent( dpy, _inputArea->getRenderSurface(i)->getWindow(), 
+                                True, KeyPressMask, &ev );
+                            ev.type = KeyRelease;
+                            XSendEvent( dpy, _inputArea->getRenderSurface(i)->getWindow(), 
+                                True, KeyPressMask, &ev );
+                        }
+                    }
+                    else
+                    {
+                        ev.type = KeyPress;
+                        XSendEvent( dpy, _rs->getWindow(), True, KeyPressMask, &ev );
+                        ev.type = KeyRelease;
+                        XSendEvent( dpy, _rs->getWindow(), True, KeyPressMask, &ev );
+                    }
+                    XFlush(dpy);XSync(dpy,0);
+                    XCloseDisplay(dpy);
+                }
+            }
         }
 
 
@@ -430,6 +456,7 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
         {
             XEvent ev;
             XNextEvent( dpy, &ev );
+
             switch( ev.type )
             {
                 case DestroyNotify:
@@ -685,7 +712,7 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
     
                         case Event::DoubleButtonPress :
                           {
-                            ButtonPressEvent *bp = dynamic_cast<ButtonPressEvent *>(ev);
+                            DoubleButtonPressEvent *bp = dynamic_cast<DoubleButtonPressEvent *>(ev);
                             transformMouseMotion( bp->window(), bp->mx(), bp->my() );
                             _mbutton |= (1<<(bp->button()-1));
     
@@ -719,6 +746,7 @@ class KeyboardMouseImplementation : public KeyboardMouseImplementationBase
                         Event::ButtonPressMask |
                         Event::ButtonReleaseMask |
                         Event::ButtonMotionMask |
+                        Event::DoubleButtonPressMask |
                         Event::MouseMotionMask|
                         Event::MouseScrollMask |
                         Event::WindowConfigMask|
@@ -1151,13 +1179,15 @@ KeyboardMouse::KeyboardMouse(Producer::InputArea *inputArea) :
 
 KeyboardMouse::~KeyboardMouse()
 {
-    if( /*threadI*/isRunning() )
+    if( /*thread*/isRunning() )
     {
         if( _implementation != NULL )
+        {
             _implementation->cancel();
+            _implementation->fini();
+        }
     }
     
-    // This is the 
     //cancel();
 
     while (isRunning())
@@ -1312,8 +1342,6 @@ void KeyboardMouse::run()
         _implementation->update(*_cb,true);
         testCancel();
     }
-
-    _implementation->fini();
 }
 
 
