@@ -50,9 +50,9 @@ RenderSurface::RenderSurface( void )
     _mayFullScreen   = true;
     _isFullScreen    = true;
 
-	// This used to be #ifdefed for the X11 implementation
-	// but the code is pure C++ and should compile anywhere
-	// The _dislayNum variable is used by CGL as well.
+    // This used to be #ifdefed for the X11 implementation
+    // but the code is pure C++ and should compile anywhere
+    // The _dislayNum variable is used by CGL as well.
     char *envptr = getenv( "DISPLAY" );
     if( envptr != NULL && *envptr != 0 )
     {
@@ -98,6 +98,7 @@ RenderSurface::RenderSurface( void )
     _parent          = 0;
     _readDrawableRenderSurface = 0L;
     _visualInfo     = NULL;
+    _visualID       = 0;
     _decorations     = true;
     _useCursorFlag   = true;
     _currentCursor   = 0;
@@ -110,6 +111,20 @@ RenderSurface::RenderSurface( void )
 
     _overrideRedirectFlag = false;
 
+    char *override_envptr = getenv( "PRODUCER_OVERRIDE_REDIRECT" );
+    if( override_envptr != NULL && *override_envptr != 0 )
+    {
+        if (strcmp(override_envptr,"true")==0 || strcmp(override_envptr,"True")==0 || strcmp(override_envptr,"TRUE")==0)
+        {
+            _overrideRedirectFlag = true;
+        }
+        else
+        {
+            _overrideRedirectFlag = false;
+        }
+    }
+
+
     _useDefaultEsc = true;
     _checkOwnEvents = true;
     _inputRectangle.set( -1.0,  1.0, -1.0, 1.0 );
@@ -376,7 +391,7 @@ void RenderSurface::setInputRectangle( const InputRectangle &inputRectangle )
     _inputRectangle = inputRectangle;
 }
 
-const RenderSurface::InputRectangle &RenderSurface::getInputRectangle()
+const RenderSurface::InputRectangle &RenderSurface::getInputRectangle() const
 {
     return _inputRectangle;
 }
@@ -441,6 +456,17 @@ GLContext RenderSurface::getGLContext( void) const
     return _glcontext;
 }
 
+void RenderSurface::setGLContext( GLContext glContext )
+{
+    if( _realized )
+    {
+        std::cerr << "RenderSurface::setGLContext() - Warning: Must be set before realize()." << std::endl;
+        return;
+    }
+
+    _glcontext = glContext;
+}
+
 
 bool RenderSurface::isRealized() const
 {
@@ -621,44 +647,14 @@ void RenderSurface::setRenderToTextureFace(CubeMapFace face)
 
 const std::vector<int> &RenderSurface::getPBufferUserAttributes() const
 {
-	return _user_pbattr;
+    return _user_pbattr;
 }
 
 std::vector<int> &RenderSurface::getPBufferUserAttributes()
 {
-	return _user_pbattr;
+    return _user_pbattr;
 }
 
-void RenderSurface::addPixelAttribute( PixelAttributeName attribute )
-{
-    _pixelAttributes.push_back( PixelAttribute( attribute ) );
-}
-
-void RenderSurface::addPixelAttribute( PixelAttributeName attribute, int parameter )
-{
-    _pixelAttributes.push_back( PixelAttribute( attribute, parameter ) );
-}
-
-void RenderSurface::addExtendedPixelAttribute( unsigned int attribute )
-{
-    _pixelAttributes.push_back( PixelAttribute( attribute ) );
-}
-
-void RenderSurface::addExtendedPixelAttribute( unsigned int attribute, int parameter )
-{
-    _pixelAttributes.push_back( PixelAttribute( attribute, parameter ) );
-}
-
-void RenderSurface::setSimplePixelConfiguration( void )
-{           
-    _pixelAttributes.clear();
-    addPixelAttribute( RenderSurface::DoubleBuffer );
-    addPixelAttribute( RenderSurface::RedSize, 8 );
-    addPixelAttribute( RenderSurface::GreenSize, 8 );
-    addPixelAttribute( RenderSurface::BlueSize, 8 );
-    addPixelAttribute( RenderSurface::DepthSize, 24);
-}           
-
 void RenderSurface::useOverrideRedirect(bool flag)
 {
     _useOverrideRedirect(flag);
