@@ -134,10 +134,21 @@ static double getDictDouble (CFDictionaryRef refDict, CFStringRef key)
 
 bool RenderSurface::_init()
 {
+    s_ini_mutex.lock();
+
     _dpy = new CGDirectDisplayID;     
     *_dpy = CGMainDisplayID();         
     CGDisplayCapture(*_dpy);
 
+    // Set up the VisualChooser
+    VisualChooser *vc
+    if(_visualChooser.valid()) vc = _visualChooser.get();
+    else
+    {
+      vc = new VisualChooser;
+      vc->setSimpleConfiguration(_drawableType != DrawableType_PBuffer);
+    }
+
     if( _windowWidth == UnknownDimension )
         _windowWidth = CGDisplayPixelsWide(*_dpy);
 
@@ -157,19 +168,22 @@ bool RenderSurface::_init()
     _screenWidth = _windowWidth  =  CGDisplayPixelsWide(*_dpy);
     _screenHeight = _windowHeight =  CGDisplayPixelsHigh(*_dpy);
 
-    CGLPixelFormatObj pixel_format;
-    _choosePixelFormat( &pixel_format );
-    CGLCreateContext( pixel_format, NULL, &_glcontext );
+    VisualInfo *vinfo = vc->choose(_dpy, 0); // screen arg doesn't matter
+    if(vinfo)
+    {
+      CGLCreateContext( *vinfo, NULL, &_glcontext );
 
-    CGLDestroyPixelFormat( pixel_format );
-    makeCurrent();
+      CGLDestroyPixelFormat( *vinfo );
+      makeCurrent();
 
-    CGLSetFullScreen(_glcontext);
+      CGLSetFullScreen(_glcontext);
 
-    // Add sync support by default:
-    GLint swap = 1;
-    CGLSetParameter (_glcontext, kCGLCPSwapInterval, &swap);
+      // Add sync support by default:
+      GLint swap = 1;
+      CGLSetParameter (_glcontext, kCGLCPSwapInterval, &swap);
+    }
 
+    s_init_mutex.unlock();
     return true;
 }
 
@@ -225,73 +239,6 @@ void RenderSurface::bindPBufferToTexture(BufferType /*buffer*/) const
     glCopyTexSubImage2D( GL_TEXTURE_2D, 0, 0, 0, 0, 0, _windowWidth, _windowHeight);
 }
 
-
-int RenderSurface::_adaptPixelAttribute( PixelAttribute &attr, VisualInfo *vinfo )
-{
-    switch( attr.attribute() )
-    {
-        case UseGL:          return kCGLPFACompliant;
-        case BufferSize:     return kCGLPFAColorSize;
-        //case Level:          return GLX_LEVEL;
-        //case RGBA:           return GLX_RGBA;
-        case DoubleBuffer:   return kCGLPFADoubleBuffer;
-        case Stereo:         return kCGLPFAStereo;
-        case AuxBuffers:     return kCGLPFAAuxBuffers;
-        case RedSize:        return kCGLPFAColorSize;
-        case GreenSize:      return kCGLPFAColorSize;
-        case BlueSize:       return kCGLPFAColorSize;
-        case AlphaSize:      return kCGLPFAAlphaSize;
-        case DepthSize:      return kCGLPFADepthSize;
-        case StencilSize:    return kCGLPFAStencilSize;
-        case AccumRedSize:   return kCGLPFAAccumSize;
-        case AccumGreenSize: return kCGLPFAAccumSize;
-        case AccumBlueSize:  return kCGLPFAAccumSize;
-        case AccumAlphaSize: return kCGLPFAAccumSize;
-        default: return (int)attr.attribute();
-    }
-    return 0;
-}
-
-
-void RenderSurface::_choosePixelFormat( VisualInfo *vinfo )
-{
-    if( _pixelAttributes.size() == 0 )
-        setSimplePixelConfiguration();
-
-    std::vector<CGLPixelFormatAttribute>va;
-
-    u_int32_t display_id = CGDisplayIDToOpenGLDisplayMask(*_dpy);
-
-    va.clear();
-    va.push_back( kCGLPFADisplayMask );
-    va.push_back( (CGLPixelFormatAttribute)display_id);
-    va.push_back( kCGLPFANoRecovery );
-    va.push_back( kCGLPFAAccelerated );
-    va.push_back( kCGLPFAFullScreen );
-
-    std::vector<PixelAttribute>::iterator p;
-    for( p = _pixelAttributes.begin(); p != _pixelAttributes.end(); p++ )
-    {
-        va.push_back( CGLPixelFormatAttribute(_adaptPixelAttribute((*p), 0L)));
-         if( (*p).hasParameter() )
-            va.push_back( CGLPixelFormatAttribute((*p).parameter()) );
-    }
-    va.push_back(CGLPixelFormatAttribute(0));
-
-    long nvirt;                                                          
-    CGLPixelFormatAttribute *array = new CGLPixelFormatAttribute[va.size()];
-
-    std::vector<CGLPixelFormatAttribute>::iterator pp;
-    int index = 0;
-    for( pp = va.begin(); pp != va.end(); pp++ )
-	array[index++] = *pp;
-
-    //CGLChoosePixelFormat(&(va.front()), vinfo, &nvirt);          
-    CGLChoosePixelFormat(array, vinfo, &nvirt);          
-}
-
-
-
 void _setCursor()
 {
 }
@@ -324,6 +271,5 @@ void RenderSurface::_initThreads()
     // Ignored...
 }
 
-
 #endif
 
