@@ -258,7 +258,11 @@ LONG WINAPI RenderSurface::proc( Window hWnd, UINT uMsg, WPARAM wParam, LPARAM l
             // an error!
             //KillGLWindow();
             _realized = false;
-            SetWindowLong(_win, GWL_WNDPROC, (long)_oldWndProc);
+#ifdef __VS6_IMPLEMENTATION
+            SetWindowLong(_win, GWL_WNDPROC, (LONG)_oldWndProc);
+#else
+            SetWindowLongPtr(_win, GWLP_WNDPROC, (LONG_PTR)_oldWndProc);
+#endif
             lRet = CallWindowProc(_oldWndProc, hWnd, uMsg, wParam, lParam);
             ev = new ShutdownEvent(hWnd);
             dispatch(ev);
@@ -276,7 +280,11 @@ LONG WINAPI RenderSurface::proc( Window hWnd, UINT uMsg, WPARAM wParam, LPARAM l
                         // an error!
             //KillGLWindow();
             _realized = false;
-            SetWindowLong(_win, GWL_WNDPROC, (long)_oldWndProc);
+#ifdef __VS6_IMPLEMENTATION
+            SetWindowLong(_win, GWL_WNDPROC, (LONG)_oldWndProc);
+#else
+            SetWindowLongPtr(_win, GWLP_WNDPROC, (LONG_PTR)_oldWndProc);
+#endif
             lRet = CallWindowProc(_oldWndProc, hWnd, uMsg, wParam, lParam);
         }
         else
@@ -598,7 +606,7 @@ bool RenderSurface::_init()
             wndclass.hInstance     = _hinstance;
             wndclass.hIcon         = LoadIcon( _hinstance, "PRODUCER_ICON" );
             wndclass.hCursor       = _defaultCursor;
-            wndclass.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
+            wndclass.hbrBackground = NULL;
             wndclass.lpszMenuName  = 0;
 
             wndclass.lpszClassName = _windowName.c_str();
@@ -762,7 +770,11 @@ bool RenderSurface::_init()
             }
 
             _isFullScreen = false;
-            _oldWndProc = (WNDPROC)SetWindowLong(_win, GWL_WNDPROC, (long)((WNDPROC)s_proc));
+#ifdef __VS6_IMPLEMENTATION
+            _oldWndProc = (WNDPROC)SetWindowLong(_win, GWL_WNDPROC, (LONG)((WNDPROC)s_proc));
+#else
+            _oldWndProc = (WNDPROC)SetWindowLongPtr(_win, GWLP_WNDPROC, (LONG_PTR)((WNDPROC)s_proc));
+#endif
         }
 
         if (!(_hdc=GetDC(_win)))
@@ -1361,7 +1373,12 @@ void RenderSurface::_fini()
     if( _realized && _drawableType == DrawableType_Window )
     {
         if (!_ownWindow)
-            SetWindowLong(_win, GWL_WNDPROC, (long)_oldWndProc);
+#ifdef __VS6_IMPLEMENTATION
+            SetWindowLong(_win, GWL_WNDPROC, (LONG)_oldWndProc);
+#else
+            SetWindowLongPtr(_win, GWLP_WNDPROC, (LONG_PTR)_oldWndProc);
+#endif
+
         PostThreadMessage(getThreadId(), WM_QUIT, 0, 0);
     }
 }
