@@ -72,7 +72,7 @@ unsigned int RenderSurface::getNumberOfScreens()
         Display *dpy = XOpenDisplay("");
         if( dpy == NULL )
         {
-            std::cerr << "Unable to open display \"" << XDisplayName( ":0.0") << "\".  Is the DISPLAY environmental variable set?\n";
+            std::cerr << "Unable to open display \"" << XDisplayName( "") << "\".  Is the DISPLAY environmental variable set?\n";
             return 0;
         }
         _numScreens = ScreenCount(dpy); 
@@ -852,6 +852,11 @@ void RenderSurface::_positionPointer( int x, int y )
 void RenderSurface::_computeScreenSize( unsigned int &width, unsigned int &height ) const
 {
     Display *dpy = XOpenDisplay("");
+    if( dpy == NULL )
+    {
+        std::cerr << "Unable to open display \"" << XDisplayName("") << "\".  Is the DISPLAY environmental variable set?\n";
+        return;
+    }
     width = DisplayWidth(dpy, _screen);
     height = DisplayHeight(dpy,_screen);
     XCloseDisplay(dpy);
