@@ -550,7 +550,7 @@ void VisualChooser::applyAttribute(const VisualAttribute &va, std::vector<int> &
         case AccumGreenSize: attr.first = WGL_ACCUM_GREEN_BITS_ARB; break;
         case AccumBlueSize:  attr.first = WGL_ACCUM_BLUE_BITS_ARB; break;
         case AccumAlphaSize: attr.first = WGL_ACCUM_ALPHA_BITS_ARB; break;
-#if defined(WGL_SAMPLE_BUFFERS) && defined (WGL_SAMPLES_ARB)
+#if defined(WGL_SAMPLE_BUFFERS_ARB) && defined (WGL_SAMPLES_ARB)
         case SampleBuffers:  attr.first = WGL_SAMPLE_BUFFERS_ARB; break;
         case Samples:        attr.first = WGL_SAMPLES_ARB; break;
 #endif
