@@ -25,9 +25,9 @@ using namespace Producer;
 
 namespace
 {
-	ref_ptr<TemporaryWindow> default_wnd__;
-	static int               tempwnd_id__ = 0;
-	OpenThreads::Mutex       wglext_critical__;
+    ref_ptr<TemporaryWindow> default_wnd__;
+    static int               tempwnd_id__ = 0;
+    OpenThreads::Mutex       wglext_critical__;
 }
 
 
@@ -35,28 +35,37 @@ WGLExtensions::InstanceMap WGLExtensions::instances_;
 
 
 WGLExtensions::WGLExtensions()
+: supp_(0)
 {
-	// WGL_ARB_pbuffer
-	wglCreatePBuffer     = (WGLCreatePBufferProc)wglGetProcAddress("wglCreatePbufferARB");
+    // WGL_ARB_pbuffer
+    wglCreatePBuffer     = (WGLCreatePBufferProc)wglGetProcAddress("wglCreatePbufferARB");
     wglGetPBufferDC      = (WGLGetPBufferDCProc)wglGetProcAddress("wglGetPbufferDCARB");
     wglReleasePBufferDC  = (WGLReleasePBufferDCProc)wglGetProcAddress("wglReleasePbufferDCARB");
     wglDestroyPBuffer    = (WGLDestroyPBufferProc)wglGetProcAddress("wglDestroyPbufferARB");
     wglQueryPBuffer      = (WGLQueryPBufferProc)wglGetProcAddress("wglQueryPbufferARB");
+    if (wglCreatePBuffer && wglGetPBufferDC && wglReleasePBufferDC && wglDestroyPBuffer && wglQueryPBuffer)
+        supp_ |= ARB_pbuffer;
 
-	// WGL_ARB_render_texture
+    // WGL_ARB_render_texture
     wglBindTexImage      = (WGLBindTexImageProc)wglGetProcAddress("wglBindTexImageARB");
     wglReleaseTexImage   = (WGLReleaseTexImageProc)wglGetProcAddress("wglReleaseTexImageARB");
     wglSetPBufferAttrib  = (WGLSetPbufferAttribProc)wglGetProcAddress("wglSetPbufferAttribARB");
+    if (wglBindTexImage && wglReleaseTexImage && wglSetPBufferAttrib)
+        supp_ |= ARB_render_texture;
 
-	// WGL_ARB_pixel_format
-	wglChoosePixelFormat = (WGLChoosePixelFormatProc)wglGetProcAddress("wglChoosePixelFormatARB");
+    // WGL_ARB_pixel_format
+    wglChoosePixelFormat = (WGLChoosePixelFormatProc)wglGetProcAddress("wglChoosePixelFormatARB");
+    if (wglChoosePixelFormat)
+        supp_ |= ARB_pixel_format;
 
-	// WGL_ARB_make_current_read
-	wglMakeContextCurrent = (WGLMakeContextCurrentProc)wglGetProcAddress("wglMakeContextCurrentARB");
-	if (!wglMakeContextCurrent)
-	{
-		wglMakeContextCurrent = (WGLMakeContextCurrentProc)wglGetProcAddress("wglMakeContextCurrentEXT");
-	}
+    // WGL_ARB_make_current_read
+    wglMakeContextCurrent = (WGLMakeContextCurrentProc)wglGetProcAddress("wglMakeContextCurrentARB");
+    if (!wglMakeContextCurrent)
+    {
+        wglMakeContextCurrent = (WGLMakeContextCurrentProc)wglGetProcAddress("wglMakeContextCurrentEXT");
+    }
+    if (wglMakeContextCurrent)
+        supp_ |= ARB_make_current_read;
 }
 
 WGLExtensions::~WGLExtensions()
@@ -65,55 +74,60 @@ WGLExtensions::~WGLExtensions()
 
 WGLExtensions *WGLExtensions::instance()
 {
-	// we must serialize access to this method because
-	// the underlying implementation of the Standard Library
-	// is not guaranted to be thread-safe, and therefore
-	// the map<> container could raise multithreading issues.
-	//OpenThreads::ScopedLock<OpenThreads::Mutex> slock(wglext_critical__);
+    // we must serialize access to this method because
+    // the underlying implementation of the Standard Library
+    // is not guaranted to be thread-safe, and therefore
+    // the map<> container could raise multithreading issues.
+    //OpenThreads::ScopedLock<OpenThreads::Mutex> slock(wglext_critical__);
 
-	HGLRC context = wglGetCurrentContext();
+    HGLRC context = wglGetCurrentContext();
 
-	if (context == 0)
-	{
-		if (!default_wnd__.valid() || !default_wnd__->getHandle())
-		{
-			default_wnd__ = new TemporaryWindow;
-			if (!default_wnd__->getHandle())
-			{
-				std::cerr << "Producer::WGLExtensions: could not create and initialize the temporary window" << std::endl;
-				return 0;
-			}
-		}
+    bool nocontext = (context == 0);
 
-		context = default_wnd__->getContext();
-		if (!default_wnd__->makeCurrent())
-		{
-			std::cerr << "Producer::WGLExtensions: could not make the temporary window's context active" << std::endl;
-		}
-	}
+    if (nocontext || !default_wnd__.valid())
+    {
+        if (!default_wnd__.valid() || !default_wnd__->getHandle())
+        {
+            default_wnd__ = new TemporaryWindow;
+            if (!default_wnd__->getHandle())
+            {
+                std::cerr << "Producer::WGLExtensions: could not create and initialize the temporary window" << std::endl;
+                return 0;
+            }
+        }
 
-	if (!instances_[context])
-	{
-		instances_[context] = new WGLExtensions;
-	}
+        context = default_wnd__->getContext();
+        if (!default_wnd__->makeCurrent())
+        {
+            std::cerr << "Producer::WGLExtensions: could not make the temporary window's context active" << std::endl;
+        }
+    }
 
-	return instances_[context].get();
+    if (!instances_[context])
+    {
+        instances_[context] = new WGLExtensions;
+    }
+
+    if (nocontext)
+        wglMakeCurrent(0, 0);
+
+    return instances_[context].get();
 }
 
 
 
 TemporaryWindow::TemporaryWindow()
-:	handle_(0),
+:    handle_(0),
     dc_(0),
     context_(0),
     instance_(0)
 {
-	create();
+    create();
 }
 
 TemporaryWindow::TemporaryWindow(const TemporaryWindow &)
 {
-	throw "This is TemporaryWindow, please don't copy me!"; 
+    throw "This is TemporaryWindow, please don't copy me!"; 
 }
 
 void TemporaryWindow::create()
@@ -138,22 +152,22 @@ void TemporaryWindow::create()
     wndclass.lpszClassName = classname_.c_str();
 
     if (!RegisterClass(&wndclass))
-		return;
+        return;
 
     if (!(handle_ = CreateWindowEx( 0,
-									classname_.c_str(),
-									TEXT(classname_.c_str()),
-									WS_POPUP,
-									0,
-									0,
-									100,
-									100,
-									0,
-									0,
-									instance_,
-									0)))
+                                    classname_.c_str(),
+                                    TEXT(classname_.c_str()),
+                                    WS_POPUP,
+                                    0,
+                                    0,
+                                    100,
+                                    100,
+                                    0,
+                                    0,
+                                    instance_,
+                                    0)))
     {
-		kill();
+        kill();
         return;
     }
 
@@ -163,33 +177,33 @@ void TemporaryWindow::create()
         return;
     }
 
-	PIXELFORMATDESCRIPTOR pfd = { 
-		sizeof(PIXELFORMATDESCRIPTOR),
-		1,                     
-		PFD_DRAW_TO_WINDOW |   
-		PFD_SUPPORT_OPENGL,      
-		PFD_TYPE_RGBA,         
-		24,                    
-		0, 0, 0, 0, 0, 0,      
-		0,                     
-		0,                     
-		0,                     
-		0, 0, 0, 0,            
-		16,
-		0,                     
-		0,                     
-		PFD_MAIN_PLANE,        
-		0,                     
-		0, 0, 0                
-	}; 
+    PIXELFORMATDESCRIPTOR pfd = { 
+        sizeof(PIXELFORMATDESCRIPTOR),
+        1,                     
+        PFD_DRAW_TO_WINDOW |   
+        PFD_SUPPORT_OPENGL,      
+        PFD_TYPE_RGBA,         
+        24,                    
+        0, 0, 0, 0, 0, 0,      
+        0,                     
+        0,                     
+        0,                     
+        0, 0, 0, 0,            
+        16,
+        0,                     
+        0,                     
+        PFD_MAIN_PLANE,        
+        0,                     
+        0, 0, 0                
+    }; 
 
-	int visual_id = ChoosePixelFormat(dc_, &pfd); 
+    int visual_id = ChoosePixelFormat(dc_, &pfd); 
 
-	if (!SetPixelFormat(dc_, visual_id, &pfd))
-	{
-		kill();
-		return;
-	}
+    if (!SetPixelFormat(dc_, visual_id, &pfd))
+    {
+        kill();
+        return;
+    }
 
     if (!(context_ = wglCreateContext(dc_)))
     {
@@ -233,7 +247,9 @@ void TemporaryWindow::kill()
 
 bool TemporaryWindow::makeCurrent()
 {
-	return wglMakeCurrent(dc_, context_);
+    return wglMakeCurrent(dc_, context_);
 }
 
 #endif
+
+
