@@ -87,6 +87,8 @@ DECLARE_HANDLE(HPBUFFERARB);
 #define WGL_SWAP_UNDEFINED_ARB              0x202A
 #define WGL_TYPE_RGBA_ARB                   0x202B
 #define WGL_TYPE_COLORINDEX_ARB             0x202C
+#define WGL_SAMPLE_BUFFERS_ARB                0x2041
+#define WGL_SAMPLES_ARB                        0x2042
 #endif
 
 #ifndef WGL_ARB_render_texture
@@ -126,6 +128,13 @@ DECLARE_HANDLE(HPBUFFERARB);
 #define WGL_AUX9_ARB                        0x2090
 #endif
 
+#ifndef WGL_SAMPLE_BUFFERS_ARB
+#define        WGL_SAMPLE_BUFFERS_ARB         0x2041
+#endif
+#ifndef WGL_SAMPLES_ARB
+#define        WGL_SAMPLES_ARB                0x2042
+#endif
+
 namespace Producer
 {    
 
@@ -133,79 +142,90 @@ namespace Producer
     {
     public:
 
-		// WGL_ARB_pbuffer
+        enum SupportedExtension
+        {
+            ARB_pbuffer           = 1,
+            ARB_render_texture    = 2,
+            ARB_pixel_format      = 4,
+            ARB_make_current_read = 8
+        };
 
-		typedef HPBUFFERARB (WINAPI * WGLCreatePBufferProc)    (HDC hDC, int iPixelFormat, int iWidth, int iHeight, const int *piAttribList);
-		typedef HDC         (WINAPI * WGLGetPBufferDCProc)     (HPBUFFERARB hPbuffer);
-		typedef int         (WINAPI * WGLReleasePBufferDCProc) (HPBUFFERARB hPbuffer, HDC hDC);
-		typedef bool        (WINAPI * WGLDestroyPBufferProc)   (HPBUFFERARB hPbuffer);
-		typedef bool        (WINAPI * WGLQueryPBufferProc)     (HPBUFFERARB hPbuffer, int iAttribute, int *piValue);
+        // WGL_ARB_pbuffer
 
-		WGLCreatePBufferProc    wglCreatePBuffer;
-		WGLGetPBufferDCProc     wglGetPBufferDC;
-		WGLReleasePBufferDCProc wglReleasePBufferDC;
-		WGLDestroyPBufferProc   wglDestroyPBuffer;
-		WGLQueryPBufferProc     wglQueryPBuffer;
+        typedef HPBUFFERARB (WINAPI * WGLCreatePBufferProc)    (HDC hDC, int iPixelFormat, int iWidth, int iHeight, const int *piAttribList);
+        typedef HDC         (WINAPI * WGLGetPBufferDCProc)     (HPBUFFERARB hPbuffer);
+        typedef int         (WINAPI * WGLReleasePBufferDCProc) (HPBUFFERARB hPbuffer, HDC hDC);
+        typedef bool        (WINAPI * WGLDestroyPBufferProc)   (HPBUFFERARB hPbuffer);
+        typedef bool        (WINAPI * WGLQueryPBufferProc)     (HPBUFFERARB hPbuffer, int iAttribute, int *piValue);
+
+        WGLCreatePBufferProc    wglCreatePBuffer;
+        WGLGetPBufferDCProc     wglGetPBufferDC;
+        WGLReleasePBufferDCProc wglReleasePBufferDC;
+        WGLDestroyPBufferProc   wglDestroyPBuffer;
+        WGLQueryPBufferProc     wglQueryPBuffer;
 
 
-		// WGL_ARB_render_texture
+        // WGL_ARB_render_texture
 
-		typedef bool        (WINAPI * WGLBindTexImageProc)     (HPBUFFERARB hPbuffer, int iBuffer);
-		typedef bool        (WINAPI * WGLReleaseTexImageProc)  (HPBUFFERARB hPbuffer, int iBuffer);
-		typedef bool        (WINAPI * WGLSetPbufferAttribProc) (HPBUFFERARB hPbuffer, const int * piAttribList);
+        typedef bool        (WINAPI * WGLBindTexImageProc)     (HPBUFFERARB hPbuffer, int iBuffer);
+        typedef bool        (WINAPI * WGLReleaseTexImageProc)  (HPBUFFERARB hPbuffer, int iBuffer);
+        typedef bool        (WINAPI * WGLSetPbufferAttribProc) (HPBUFFERARB hPbuffer, const int * piAttribList);
 
-		WGLBindTexImageProc     wglBindTexImage;
-		WGLReleaseTexImageProc  wglReleaseTexImage;
-		WGLSetPbufferAttribProc wglSetPBufferAttrib;
+        WGLBindTexImageProc     wglBindTexImage;
+        WGLReleaseTexImageProc  wglReleaseTexImage;
+        WGLSetPbufferAttribProc wglSetPBufferAttrib;
 
-		// WGL_ARB_pixel_format
+        // WGL_ARB_pixel_format
 
-		typedef bool        (WINAPI * WGLChoosePixelFormatProc) (HDC, const int *, const float *, unsigned int, int *, unsigned int *);
+        typedef bool        (WINAPI * WGLChoosePixelFormatProc) (HDC, const int *, const float *, unsigned int, int *, unsigned int *);
 
-		WGLChoosePixelFormatProc wglChoosePixelFormat;
+        WGLChoosePixelFormatProc wglChoosePixelFormat;
 
-		// WGL_ARB_make_current_read
+        // WGL_ARB_make_current_read
 
-		typedef bool        (WINAPI * WGLMakeContextCurrentProc) (HDC, HDC, HGLRC);
+        typedef bool        (WINAPI * WGLMakeContextCurrentProc) (HDC, HDC, HGLRC);
 
-		WGLMakeContextCurrentProc wglMakeContextCurrent;
+        WGLMakeContextCurrentProc wglMakeContextCurrent;
 
-		// this static method returns an instance of WGLExtensions that
-		// is unique for the active rendering context (queried via
-		// wglGetCurrentContext()). If no rendering context is active, 
-		// the global temporary window's context will be used. If the 
-		// temporary window can't be created, then 0 is returned.
-		static WGLExtensions *instance();
+        // this static method returns an instance of WGLExtensions that
+        // is unique for the active rendering context (queried via
+        // wglGetCurrentContext()). If no rendering context is active, 
+        // the global temporary window's context will be used. If the 
+        // temporary window can't be created, then 0 is returned.
+        static WGLExtensions *instance();
+
+        inline bool isSupported(SupportedExtension ext) const { return (supp_ & ext) != 0; }
 
     protected:
-		WGLExtensions();
-		~WGLExtensions();
+        WGLExtensions();
+        ~WGLExtensions();
 
     private:
-		typedef std::map<HGLRC, Producer::ref_ptr<WGLExtensions> > InstanceMap;
+        typedef std::map<HGLRC, Producer::ref_ptr<WGLExtensions> > InstanceMap;
         static InstanceMap instances_;
+        int supp_;
     };
 
     
 
-	class TemporaryWindow: public Referenced
+    class TemporaryWindow: public Referenced
     {
     public:
-		TemporaryWindow();
-		TemporaryWindow(const TemporaryWindow &);
+        TemporaryWindow();
+        TemporaryWindow(const TemporaryWindow &);
 
         HWND  getHandle() const    { return handle_; }
         HDC   getDC() const        { return dc_; }
         HGLRC getContext() const   { return context_; }
 
-		bool makeCurrent();
+        bool makeCurrent();
 
-	protected:
-		~TemporaryWindow();
-		TemporaryWindow &operator=(const TemporaryWindow &) { return *this; }
+    protected:
+        ~TemporaryWindow();
+        TemporaryWindow &operator=(const TemporaryWindow &) { return *this; }
 
-		void create();
-		void kill();
+        void create();
+        void kill();
 
     private:
         HWND handle_;
@@ -220,3 +240,4 @@ namespace Producer
 #endif
 
 #endif
+
